@@ -15,7 +15,7 @@
  */
 
 import { TalerExchangeApi, TalerExchangeHttpClient, TalerError } from "@gnu-taler/taler-util";
-import { BrowserHttpLib, useTranslationContext } from "@gnu-taler/web-util/browser";
+import { BrowserFetchHttpLib, useTranslationContext } from "@gnu-taler/web-util/browser";
 import { ComponentChildren, createContext, FunctionComponent, h, VNode } from "preact";
 import { useContext, useEffect, useState } from "preact/hooks";
 import { ErrorLoading } from "@gnu-taler/web-util/browser";
@@ -61,11 +61,13 @@ export const ExchangeApiProvider = ({
   const [checked, setChecked] = useState<ConfigResult>()
   const { i18n } = useTranslationContext();
   const url = new URL(baseUrl)
-  const api = new TalerExchangeHttpClient(url.href, new BrowserHttpLib())
+  const api = new TalerExchangeHttpClient(url.href, new BrowserFetchHttpLib())
   useEffect(() => {
     api.getConfig()
       .then((resp) => {
-        if (api.isCompatible(resp.body.version)) {
+        if (resp.type === "fail") {
+          setChecked({ type: "error", error: TalerError.fromUncheckedDetail(resp.detail) });
+        } else if (api.isCompatible(resp.body.version)) {
           setChecked({ type: "ok", config: resp.body });
         } else {
           setChecked({ type: "incompatible", result: resp.body, supported: api.PROTOCOL_VERSION })
