@@ -1,4 +1,4 @@
-import { Amounts, HttpStatusCode, TalerExchangeApi, TalerProtocolTimestamp, TranslatedString } from "@gnu-taler/taler-util";
+import { AbsoluteTime, Amounts, HttpStatusCode, TalerExchangeApi, TalerProtocolTimestamp, TranslatedString } from "@gnu-taler/taler-util";
 import { LocalNotificationBanner, useLocalNotification, useTranslationContext } from "@gnu-taler/web-util/browser";
 import { Fragment, VNode, h } from "preact";
 import { useExchangeApiContext } from "../context/config.js";
@@ -60,18 +60,21 @@ export function NewFormEntry({
                 title: i18n.str`Wrong credentials for "${officer.account}"`,
                 description: resp.detail.hint as TranslatedString,
                 debug: resp.detail,
+                when: AbsoluteTime.now(),
               })
               case HttpStatusCode.NotFound: return notify({
                 type: "error",
                 title: i18n.str`Officer or account not found`,
                 description: resp.detail.hint as TranslatedString,
                 debug: resp.detail,
+                when: AbsoluteTime.now(),
               })
               case HttpStatusCode.Conflict: return notify({
                 type: "error",
                 title: i18n.str`Officer disabled or more recent decision was already submitted.`,
                 description: resp.detail.hint as TranslatedString,
                 debug: resp.detail,
+                when: AbsoluteTime.now(),
               })
             }
           })
