@@ -997,7 +997,7 @@ export class BridgeIDBFactory {
 
         await transaction._waitDone();
 
-        // We re-use the same transaction (as per spec) here.
+        // We reuse the same transaction (as per spec) here.
         transaction._active = true;
 
         if (db._closed || db._closePending) {
@@ -2471,6 +2471,8 @@ export class BridgeIDBTransaction
     return this._committed || this._aborted;
   }
 
+  _counter = 0;
+
   _openRequest: BridgeIDBOpenDBRequest | null = null;
 
   _backendTransaction?: DatabaseTransaction;
@@ -2745,7 +2747,14 @@ export class BridgeIDBTransaction
         // with error handling already built into operation
         await operation();
       } else {
-        await waitMacroQueue();
+        this._counter++;
+        if (this._counter > 100) {
+          this._counter = 0;
+          // Give a chance for macro tasks to do something
+          // If we don't do this at all, we break WPT tests.
+          // If we always wait, performance is bad.
+          await waitMacroQueue();
+        }
 
         let event;
         try {
