@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2021-2023 Taler Systems S.A.
+ (C) 2021-2024 Taler Systems S.A.
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -233,11 +233,11 @@ export function InputPaytoForm<T>({
   }, [str])
 
   // const submit = useCallback((): void => {
-  //   // const accounts: MerchantBackend.BankAccounts.AccountAddDetails[] = paytos;
+  //   // const accounts: TalerMerchantApi.AccountAddDetails[] = paytos;
   //   // const alreadyExists =
   //   //   accounts.findIndex((x) => x.payto_uri === paytoURL) !== -1;
   //   // if (!alreadyExists) {
-  //   const newValue: MerchantBackend.BankAccounts.AccountAddDetails = {
+  //   const newValue: TalerMerchantApi.AccountAddDetails = {
   //     payto_uri: paytoURL,
   //   };
   //   if (value.auth) {
@@ -276,6 +276,7 @@ export function InputPaytoForm<T>({
           label={i18n.str`Account type`}
           tooltip={i18n.str`Method to use for wire transfer`}
           values={targets}
+          readonly={readonly}
           toStr={(v) => (v === noTargetValue ? i18n.str`Choose one...` : v)}
         />
 
@@ -284,11 +285,13 @@ export function InputPaytoForm<T>({
             <Input<Entity>
               name="path1"
               label={i18n.str`Routing`}
+              readonly={readonly}
               tooltip={i18n.str`Routing number.`}
             />
             <Input<Entity>
               name="path2"
               label={i18n.str`Account`}
+              readonly={readonly}
               tooltip={i18n.str`Account number.`}
             />
           </Fragment>
@@ -298,6 +301,7 @@ export function InputPaytoForm<T>({
             <Input<Entity>
               name="path1"
               label={i18n.str`Code`}
+              readonly={readonly}
               tooltip={i18n.str`Business Identifier Code.`}
             />
           </Fragment>
@@ -308,6 +312,7 @@ export function InputPaytoForm<T>({
               name="path1"
               label={i18n.str`IBAN`}
               tooltip={i18n.str`International Bank Account Number.`}
+              readonly={readonly}
               placeholder="DE1231231231"
               inputExtra={{ style: { textTransform: "uppercase" } }}
             />
@@ -317,6 +322,7 @@ export function InputPaytoForm<T>({
           <Fragment>
             <Input<Entity>
               name="path1"
+              readonly={readonly}
               label={i18n.str`Account`}
               tooltip={i18n.str`Unified Payment Interface.`}
             />
@@ -326,6 +332,7 @@ export function InputPaytoForm<T>({
           <Fragment>
             <Input<Entity>
               name="path1"
+              readonly={readonly}
               label={i18n.str`Address`}
               tooltip={i18n.str`Bitcoin protocol.`}
             />
@@ -335,6 +342,7 @@ export function InputPaytoForm<T>({
           <Fragment>
             <Input<Entity>
               name="path1"
+              readonly={readonly}
               label={i18n.str`Address`}
               tooltip={i18n.str`Ethereum protocol.`}
             />
@@ -344,6 +352,7 @@ export function InputPaytoForm<T>({
           <Fragment>
             <Input<Entity>
               name="path1"
+              readonly={readonly}
               label={i18n.str`Address`}
               tooltip={i18n.str`Interledger protocol.`}
             />
@@ -354,11 +363,13 @@ export function InputPaytoForm<T>({
           <Fragment>
             <Input<Entity>
               name="path1"
+              readonly={readonly}
               label={i18n.str`Host`}
               tooltip={i18n.str`Bank host.`}
             />
             <Input<Entity>
               name="path2"
+              readonly={readonly}
               label={i18n.str`Account`}
               tooltip={i18n.str`Bank account.`}
             />
@@ -372,6 +383,7 @@ export function InputPaytoForm<T>({
           <Fragment>
             <Input
               name="params.receiver-name"
+              readonly={readonly}
               label={i18n.str`Owner's name`}
               tooltip={i18n.str`Legal name of the person holding the account.`}
             />
