@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2021-2023 Taler Systems S.A.
+ (C) 2021-2024 Taler Systems S.A.
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -19,12 +19,14 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
-import { useTranslationContext } from "@gnu-taler/web-util/browser";
+import { AmountString, TalerMerchantApi } from "@gnu-taler/taler-util";
+import {
+  useMerchantApiContext,
+  useTranslationContext
+} from "@gnu-taler/web-util/browser";
 import { h } from "preact";
 import { useCallback, useEffect, useState } from "preact/hooks";
 import * as yup from "yup";
-import { useBackendContext } from "../../context/backend.js";
-import { MerchantBackend } from "../../declaration.js";
 import {
   ProductCreateSchema as createSchema,
   ProductUpdateSchema as updateSchema,
@@ -38,7 +40,7 @@ import { InputStock, Stock } from "../form/InputStock.js";
 import { InputTaxes } from "../form/InputTaxes.js";
 import { InputWithAddon } from "../form/InputWithAddon.js";
 
-type Entity = MerchantBackend.Products.ProductDetail & { product_id: string };
+type Entity = TalerMerchantApi.ProductDetail & { product_id: string };
 
 interface Props {
   onSubscribe: (c?: () => Entity | undefined) => void;
@@ -52,18 +54,18 @@ export function ProductForm({ onSubscribe, initial, alreadyExist }: Props) {
     description_i18n: {},
     taxes: [],
     next_restock: { t_s: "never" },
-    price: ":0",
+    price: ":0" as AmountString,
     ...initial,
     stock:
       !initial || initial.total_stock === -1
         ? undefined
         : {
-          current: initial.total_stock || 0,
-          lost: initial.total_lost || 0,
-          sold: initial.total_sold || 0,
-          address: initial.address,
-          nextRestock: initial.next_restock,
-        },
+            current: initial.total_stock || 0,
+            lost: initial.total_lost || 0,
+            sold: initial.total_sold || 0,
+            address: initial.address,
+            nextRestock: initial.next_restock,
+          },
   });
   let errors: FormErrors<Entity> = {};
 
@@ -82,11 +84,11 @@ export function ProductForm({ onSubscribe, initial, alreadyExist }: Props) {
     }
   }
   const hasErrors = Object.keys(errors).some(
-    (k) => (errors as any)[k] !== undefined,
+    (k) => (errors as Record<string, unknown>)[k] !== undefined,
   );
 
   const submit = useCallback((): Entity | undefined => {
-    const stock: Stock = (value as any).stock;
+    const stock = (value).stock;
 
     if (!stock) {
       value.total_stock = -1;
@@ -99,13 +101,13 @@ export function ProductForm({ onSubscribe, initial, alreadyExist }: Props) {
           : stock.nextRestock;
       value.address = stock.address;
     }
-    delete (value as any).stock;
+    delete value.stock;
 
     if (typeof value.minimum_age !== "undefined" && value.minimum_age < 1) {
       delete value.minimum_age;
     }
 
-    return value as MerchantBackend.Products.ProductDetail & {
+    return value as TalerMerchantApi.ProductDetail & {
       product_id: string;
     };
   }, [value]);
@@ -114,7 +116,7 @@ export function ProductForm({ onSubscribe, initial, alreadyExist }: Props) {
     onSubscribe(hasErrors ? undefined : submit);
   }, [submit, hasErrors]);
 
-  const { url: backendURL } = useBackendContext()
+  const { url: backendUrl } = useMerchantApiContext();
   const { i18n } = useTranslationContext();
 
   return (
@@ -128,7 +130,7 @@ export function ProductForm({ onSubscribe, initial, alreadyExist }: Props) {
         {alreadyExist ? undefined : (
           <InputWithAddon<Entity>
             name="product_id"
-            addonBefore={`${backendURL}/product/`}
+            addonBefore={new URL("product/", backendUrl.href).href}
             label={i18n.str`ID`}
             tooltip={i18n.str`product identification to use in URLs (for internal use only)`}
           />
