@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2021-2023 Taler Systems S.A.
+ (C) 2021-2024 Taler Systems S.A.
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -17,30 +17,30 @@
  *
  * @author Sebastian Javier Marchano (sebasjm)
  */
-import { useTranslationContext } from "@gnu-taler/web-util/browser";
-import { Fragment, h, VNode } from "preact";
+import { TalerMerchantApi } from "@gnu-taler/taler-util";
+import {
+  useMerchantApiContext,
+  useTranslationContext,
+} from "@gnu-taler/web-util/browser";
+import { Fragment, VNode, h } from "preact";
 import { useState } from "preact/hooks";
 import { NotificationCard } from "../../../components/menu/index.js";
-import { AccessToken, MerchantBackend } from "../../../declaration.js";
-import { useAdminAPI, useInstanceAPI } from "../../../hooks/instance.js";
+import { useSessionContext } from "../../../context/session.js";
 import { Notification } from "../../../utils/types.js";
 import { CreatePage } from "./CreatePage.js";
-import { useCredentialsChecker } from "../../../hooks/backend.js";
-import { useBackendContext } from "../../../context/backend.js";
 
 interface Props {
   onBack?: () => void;
   onConfirm: () => void;
   forceId?: string;
 }
-export type Entity = MerchantBackend.Instances.InstanceConfigurationMessage;
+export type Entity = TalerMerchantApi.InstanceConfigurationMessage;
 
 export default function Create({ onBack, onConfirm, forceId }: Props): VNode {
-  const { createInstance } = useAdminAPI();
   const [notif, setNotif] = useState<Notification | undefined>(undefined);
   const { i18n } = useTranslationContext();
-  const { requestNewLoginToken } = useCredentialsChecker()
-  const { url: backendURL, updateToken } = useBackendContext()
+  const { lib } = useMerchantApiContext();
+  const { state, logIn } = useSessionContext();
 
   return (
     <Fragment>
@@ -50,17 +50,26 @@ export default function Create({ onBack, onConfirm, forceId }: Props): VNode {
         onBack={onBack}
         forceId={forceId}
         onCreate={async (
-          d: MerchantBackend.Instances.InstanceConfigurationMessage,
+          d: TalerMerchantApi.InstanceConfigurationMessage,
         ) => {
+          if (state.status !== "loggedIn") return;
           try {
-            await createInstance(d)
+            await lib.instance.createInstance(state.token, d);
             if (d.auth.token) {
-              const resp = await requestNewLoginToken(backendURL, d.auth.token as AccessToken)
-              if (resp.valid) {
-                const { token, expiration } = resp
-                updateToken({ token, expiration });
-              } else {
-                updateToken(undefined)
+              //if auth has been updated, request a new access token
+              const result = await lib.authenticate.createAccessTokenBearer(
+                d.auth.token,
+                {
+                  scope: "write",
+                  duration: {
+                    d_us: "forever",
+                  },
+                  refreshable: true,
+                },
+              );
+              if (result.type === "ok") {
+                const { token } = result.body;
+                logIn({ token });
               }
             }
             onConfirm();
@@ -72,7 +81,7 @@ export default function Create({ onBack, onConfirm, forceId }: Props): VNode {
                 description: ex.message,
               });
             } else {
-              console.error(ex)
+              console.error(ex);
             }
           }
         }}
