@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2021-2023 Taler Systems S.A.
+ (C) 2021-2024 Taler Systems S.A.
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -19,58 +19,62 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
-import { Amounts } from "@gnu-taler/taler-util";
-import { useTranslationContext } from "@gnu-taler/web-util/browser";
-import { add, isAfter, isBefore, isFuture } from "date-fns";
-import { Fragment, h, VNode } from "preact";
+import {
+  AbsoluteTime,
+  AmountString,
+  Amounts,
+  Duration,
+  TalerMerchantApi,
+  TalerProtocolDuration,
+} from "@gnu-taler/taler-util";
+import {
+  useMerchantApiContext,
+  useTranslationContext,
+} from "@gnu-taler/web-util/browser";
+import { format, isFuture } from "date-fns";
+import { Fragment, VNode, h } from "preact";
 import { useEffect, useState } from "preact/hooks";
 import {
   FormErrors,
   FormProvider,
 } from "../../../../components/form/FormProvider.js";
 import { Input } from "../../../../components/form/Input.js";
-import { InputBoolean } from "../../../../components/form/InputBoolean.js";
 import { InputCurrency } from "../../../../components/form/InputCurrency.js";
 import { InputDate } from "../../../../components/form/InputDate.js";
+import { InputDuration } from "../../../../components/form/InputDuration.js";
 import { InputGroup } from "../../../../components/form/InputGroup.js";
 import { InputLocation } from "../../../../components/form/InputLocation.js";
 import { InputNumber } from "../../../../components/form/InputNumber.js";
+import { InputToggle } from "../../../../components/form/InputToggle.js";
 import { InventoryProductForm } from "../../../../components/product/InventoryProductForm.js";
 import { NonInventoryProductFrom } from "../../../../components/product/NonInventoryProductForm.js";
 import { ProductList } from "../../../../components/product/ProductList.js";
-import { useConfigContext } from "../../../../context/config.js";
-import { Duration, MerchantBackend, WithId } from "../../../../declaration.js";
-import { OrderCreateSchema as schema } from "../../../../schemas/index.js";
+import { usePreference } from "../../../../hooks/preference.js";
 import { rate } from "../../../../utils/amount.js";
 import { undefinedIfEmpty } from "../../../../utils/table.js";
-import { useSettings } from "../../../../hooks/useSettings.js";
-import { InputToggle } from "../../../../components/form/InputToggle.js";
 
 interface Props {
-  onCreate: (d: MerchantBackend.Orders.PostOrderRequest) => void;
+  onCreate: (d: TalerMerchantApi.PostOrderRequest) => void;
   onBack?: () => void;
   instanceConfig: InstanceConfig;
-  instanceInventory: (MerchantBackend.Products.ProductDetail & WithId)[];
+  instanceInventory: (TalerMerchantApi.ProductDetail & WithId)[];
 }
 interface InstanceConfig {
   use_stefan: boolean;
-  default_pay_delay: Duration;
-  default_wire_transfer_delay: Duration;
+  default_pay_delay: TalerProtocolDuration;
+  default_wire_transfer_delay: TalerProtocolDuration;
 }
 
-function with_defaults(config: InstanceConfig, currency: string): Partial<Entity> {
-  const defaultPayDeadline =
-    !config.default_pay_delay || config.default_pay_delay.d_us === "forever"
-      ? undefined
-      : add(new Date(), {
-        seconds: config.default_pay_delay.d_us / (1000 * 1000),
-      });
-  const defaultWireDeadline =
-    !config.default_wire_transfer_delay || config.default_wire_transfer_delay.d_us === "forever"
-      ? undefined
-      : add(new Date(), {
-        seconds: config.default_wire_transfer_delay.d_us / (1000 * 1000),
-      });
+function with_defaults(
+  config: InstanceConfig,
+  _currency: string,
+): Partial<Entity> {
+  const defaultPayDeadline = Duration.fromTalerProtocolDuration(
+    config.default_pay_delay,
+  );
+  const defaultWireDeadline = Duration.fromTalerProtocolDuration(
+    config.default_wire_transfer_delay,
+  );
 
   return {
     inventoryProducts: {},
@@ -78,9 +82,9 @@ function with_defaults(config: InstanceConfig, currency: string): Partial<Entity
     pricing: {},
     payments: {
       max_fee: undefined,
+      createToken: true,
       pay_deadline: defaultPayDeadline,
       refund_deadline: defaultPayDeadline,
-      createToken: true,
       wire_transfer_deadline: defaultWireDeadline,
     },
     shipping: {},
@@ -89,7 +93,7 @@ function with_defaults(config: InstanceConfig, currency: string): Partial<Entity
 }
 
 interface ProductAndQuantity {
-  product: MerchantBackend.Products.ProductDetail & WithId;
+  product: TalerMerchantApi.ProductDetail & WithId;
   quantity: number;
 }
 export interface ProductMap {
@@ -103,47 +107,38 @@ interface Pricing {
 }
 interface Shipping {
   delivery_date?: Date;
-  delivery_location?: MerchantBackend.Location;
+  delivery_location?: TalerMerchantApi.Location;
   fullfilment_url?: string;
 }
 interface Payments {
-  refund_deadline?: Date;
-  pay_deadline?: Date;
-  wire_transfer_deadline?: Date;
-  auto_refund_deadline?: Date;
+  refund_deadline: Duration;
+  pay_deadline: Duration;
+  wire_transfer_deadline: Duration;
+  auto_refund_deadline: Duration;
   max_fee?: string;
   createToken: boolean;
   minimum_age?: number;
 }
 interface Entity {
   inventoryProducts: ProductMap;
-  products: MerchantBackend.Product[];
+  products: TalerMerchantApi.Product[];
   pricing: Partial<Pricing>;
   payments: Partial<Payments>;
   shipping: Partial<Shipping>;
   extra: Record<string, string>;
 }
 
-const stringIsValidJSON = (value: string) => {
-  try {
-    JSON.parse(value.trim());
-    return true;
-  } catch {
-    return false;
-  }
-};
-
 export function CreatePage({
   onCreate,
   onBack,
   instanceConfig,
   instanceInventory,
 }: Props): VNode {
-  const config = useConfigContext();
-  const instance_default = with_defaults(instanceConfig, config.currency)
+  const { config } = useMerchantApiContext();
+  const instance_default = with_defaults(instanceConfig, config.currency);
   const [value, valueHandler] = useState(instance_default);
   const zero = Amounts.zeroOfCurrency(config.currency);
-  const [settings, updateSettings] = useSettings()
+  const [settings, updateSettings] = usePreference();
   const inventoryList = Object.values(value.inventoryProducts || {});
   const productList = Object.values(value.products || {});
 
@@ -164,54 +159,44 @@ export function CreatePage({
             ? i18n.str`must be greater than 0`
             : undefined,
     }),
-    // extra:
-    //   value.extra && !stringIsValidJSON(value.extra)
-    //     ? i18n.str`not a valid json`
-    //     : undefined,
     payments: undefinedIfEmpty({
       refund_deadline: !value.payments?.refund_deadline
         ? undefined
-        : !isFuture(value.payments.refund_deadline)
-          ? i18n.str`should be in the future`
-          : value.payments.pay_deadline &&
-            isBefore(value.payments.refund_deadline, value.payments.pay_deadline)
-            ? i18n.str`refund deadline cannot be before pay deadline`
-            : value.payments.wire_transfer_deadline &&
-              isBefore(
+        : value.payments.pay_deadline &&
+            Duration.cmp(
+              value.payments.refund_deadline,
+              value.payments.pay_deadline,
+            ) === -1
+          ? i18n.str`refund deadline cannot be before pay deadline`
+          : value.payments.wire_transfer_deadline &&
+              Duration.cmp(
                 value.payments.wire_transfer_deadline,
                 value.payments.refund_deadline,
-              )
-              ? i18n.str`wire transfer deadline cannot be before refund deadline`
-              : undefined,
+              ) === -1
+            ? i18n.str`wire transfer deadline cannot be before refund deadline`
+            : undefined,
       pay_deadline: !value.payments?.pay_deadline
         ? i18n.str`required`
-        : !isFuture(value.payments.pay_deadline)
-          ? i18n.str`should be in the future`
-          : value.payments.wire_transfer_deadline &&
-            isBefore(
+        : value.payments.wire_transfer_deadline &&
+            Duration.cmp(
               value.payments.wire_transfer_deadline,
               value.payments.pay_deadline,
-            )
-            ? i18n.str`wire transfer deadline cannot be before pay deadline`
-            : undefined,
+            ) === -1
+          ? i18n.str`wire transfer deadline cannot be before pay deadline`
+          : undefined,
       wire_transfer_deadline: !value.payments?.wire_transfer_deadline
         ? i18n.str`required`
-        : !isFuture(value.payments.wire_transfer_deadline)
-          ? i18n.str`should be in the future`
-          : undefined,
+        : undefined,
       auto_refund_deadline: !value.payments?.auto_refund_deadline
         ? undefined
-        : !isFuture(value.payments.auto_refund_deadline)
-          ? i18n.str`should be in the future`
-          : !value.payments?.refund_deadline
-            ? i18n.str`should have a refund deadline`
-            : !isAfter(
-              value.payments.refund_deadline,
-              value.payments.auto_refund_deadline,
-            )
-              ? i18n.str`auto refund cannot be after refund deadline`
-              : undefined,
-
+        : !value.payments?.refund_deadline
+          ? i18n.str`should have a refund deadline`
+          : Duration.cmp(
+                value.payments.refund_deadline,
+                value.payments.auto_refund_deadline,
+              ) == -1
+            ? i18n.str`auto refund cannot be after refund deadline`
+            : undefined,
     }),
     shipping: undefinedIfEmpty({
       delivery_date: !value.shipping?.delivery_date
@@ -226,42 +211,40 @@ export function CreatePage({
   );
 
   const submit = (): void => {
-    const order = schema.cast(value);
+    const order = value as any; //schema.cast(value);
     if (!value.payments) return;
     if (!value.shipping) return;
 
-    const request: MerchantBackend.Orders.PostOrderRequest = {
+    const request: TalerMerchantApi.PostOrderRequest = {
       order: {
         amount: order.pricing.order_price,
         summary: order.pricing.summary,
         products: productList,
-        extra: JSON.stringify(value.extra),
-        pay_deadline: value.payments.pay_deadline
-          ? {
-            t_s: Math.floor(value.payments.pay_deadline.getTime() / 1000),
-          }
-          : undefined,
-        wire_transfer_deadline: value.payments.wire_transfer_deadline
-          ? {
-            t_s: Math.floor(
-              value.payments.wire_transfer_deadline.getTime() / 1000,
-            ),
-          }
-          : undefined,
-        refund_deadline: value.payments.refund_deadline
-          ? {
-            t_s: Math.floor(value.payments.refund_deadline.getTime() / 1000),
-          }
-          : undefined,
+        extra: undefinedIfEmpty(value.extra),
+        pay_deadline: AbsoluteTime.toProtocolTimestamp(
+          AbsoluteTime.addDuration(
+            AbsoluteTime.now(),
+            value.payments.pay_deadline!,
+          ),
+        ),
+        wire_transfer_deadline: AbsoluteTime.toProtocolTimestamp(
+          AbsoluteTime.addDuration(
+            AbsoluteTime.now(),
+            value.payments.wire_transfer_deadline!,
+          ),
+        ),
+        refund_deadline: AbsoluteTime.toProtocolTimestamp(
+          AbsoluteTime.addDuration(
+            AbsoluteTime.now(),
+            value.payments.refund_deadline!,
+          ),
+        ),
         auto_refund: value.payments.auto_refund_deadline
-          ? {
-            d_us: Math.floor(
-              value.payments.auto_refund_deadline.getTime() * 1000,
-            ),
-          }
+          ? Duration.toTalerProtocolDuration(
+              value.payments.auto_refund_deadline,
+            )
           : undefined,
-        max_fee: value.payments.max_fee as string,
-
+        max_fee: value.payments.max_fee as AmountString,
         delivery_date: value.shipping.delivery_date
           ? { t_s: value.shipping.delivery_date.getTime() / 1000 }
           : undefined,
@@ -280,7 +263,7 @@ export function CreatePage({
   };
 
   const addProductToTheInventoryList = (
-    product: MerchantBackend.Products.ProductDetail & WithId,
+    product: TalerMerchantApi.ProductDetail & WithId,
     quantity: number,
   ) => {
     valueHandler((v) => {
@@ -298,7 +281,7 @@ export function CreatePage({
     });
   };
 
-  const addNewProduct = async (product: MerchantBackend.Product) => {
+  const addNewProduct = async (product: TalerMerchantApi.Product) => {
     return valueHandler((v) => {
       const products = v.products ? [...v.products, product] : [];
       return { ...v, products };
@@ -314,7 +297,7 @@ export function CreatePage({
   };
 
   const [editingProduct, setEditingProduct] = useState<
-    MerchantBackend.Product | undefined
+    TalerMerchantApi.Product | undefined
   >(undefined);
 
   const totalPriceInventory = inventoryList.reduce((prev, cur) => {
@@ -325,7 +308,7 @@ export function CreatePage({
   const totalPriceProducts = productList.reduce((prev, cur) => {
     if (!cur.price) return zero;
     const p = Amounts.parseOrThrow(cur.price);
-    return Amounts.add(prev, Amounts.mult(p, cur.quantity).amount).amount;
+    return Amounts.add(prev, Amounts.mult(p, cur.quantity ?? 0).amount).amount;
   }, zero);
 
   const hasProducts = inventoryList.length > 0 || productList.length > 0;
@@ -334,7 +317,7 @@ export function CreatePage({
   const totalAsString = Amounts.stringify(totalPrice.amount);
   const allProducts = productList.concat(inventoryList.map(asProduct));
 
-  const [newField, setNewField] = useState("")
+  const [newField, setNewField] = useState("");
 
   useEffect(() => {
     valueHandler((v) => {
@@ -354,40 +337,50 @@ export function CreatePage({
     totalPrice.amount,
   );
 
-  const minAgeByProducts = allProducts.reduce(
+  const minAgeByProducts = inventoryList.reduce(
     (cur, prev) =>
-      !prev.minimum_age || cur > prev.minimum_age ? cur : prev.minimum_age,
+      !prev.product.minimum_age || cur > prev.product.minimum_age ? cur : prev.product.minimum_age,
     0,
   );
 
-  const noDefault_payDeadline = !instance_default.payments || !instance_default.payments.pay_deadline
-  const noDefault_wireDeadline = !instance_default.payments || !instance_default.payments.wire_transfer_deadline
-  const requiresSomeTalerOptions = noDefault_payDeadline || noDefault_wireDeadline
+  // if there is no default pay deadline
+  const noDefault_payDeadline =
+    !instance_default.payments || !instance_default.payments.pay_deadline;
+  // and there is no default wire deadline
+  const noDefault_wireDeadline =
+    !instance_default.payments ||
+    !instance_default.payments.wire_transfer_deadline;
+  // user required to set the taler options
+  const requiresSomeTalerOptions =
+    noDefault_payDeadline || noDefault_wireDeadline;
 
   return (
     <div>
-
       <section class="section is-main-section">
         <div class="tabs is-toggle is-fullwidth is-small">
           <ul>
-            <li class={!settings.advanceOrderMode ? "is-active" : ""} onClick={() => {
-              updateSettings({
-                ...settings,
-                advanceOrderMode: false
-              })
-            }}>
-              <a >
-                <span><i18n.Translate>Simple</i18n.Translate></span>
+            <li
+              class={!settings.advanceOrderMode ? "is-active" : ""}
+              onClick={() => {
+                updateSettings("advanceOrderMode", false);
+              }}
+            >
+              <a>
+                <span>
+                  <i18n.Translate>Simple</i18n.Translate>
+                </span>
               </a>
             </li>
-            <li class={settings.advanceOrderMode ? "is-active" : ""} onClick={() => {
-              updateSettings({
-                ...settings,
-                advanceOrderMode: true
-              })
-            }}>
-              <a >
-                <span><i18n.Translate>Advanced</i18n.Translate></span>
+            <li
+              class={settings.advanceOrderMode ? "is-active" : ""}
+              onClick={() => {
+                updateSettings("advanceOrderMode", true);
+              }}
+            >
+              <a>
+                <span>
+                  <i18n.Translate>Advanced</i18n.Translate>
+                </span>
               </a>
             </li>
           </ul>
@@ -415,7 +408,7 @@ export function CreatePage({
                 inventory={instanceInventory}
               />
 
-              {settings.advanceOrderMode &&
+              {settings.advanceOrderMode && (
                 <NonInventoryProductFrom
                   productToEdit={editingProduct}
                   onAddProduct={(p) => {
@@ -423,7 +416,7 @@ export function CreatePage({
                     return addNewProduct(p);
                   }}
                 />
-              }
+              )}
 
               {allProducts.length > 0 && (
                 <ProductList
@@ -466,8 +459,8 @@ export function CreatePage({
                       discountOrRise > 0 &&
                       (discountOrRise < 1
                         ? `discount of %${Math.round(
-                          (1 - discountOrRise) * 100,
-                        )}`
+                            (1 - discountOrRise) * 100,
+                          )}`
                         : `rise of %${Math.round((discountOrRise - 1) * 100)}`)
                     }
                     tooltip={i18n.str`Amount to be paid by the customer`}
@@ -488,7 +481,7 @@ export function CreatePage({
                 tooltip={i18n.str`Title of the order to be shown to the customer`}
               />
 
-              {settings.advanceOrderMode &&
+              {settings.advanceOrderMode && (
                 <InputGroup
                   name="shipping"
                   label={i18n.str`Shipping and Fulfillment`}
@@ -514,135 +507,201 @@ export function CreatePage({
                     tooltip={i18n.str`URL to which the user will be redirected after successful payment.`}
                   />
                 </InputGroup>
-              }
+              )}
 
-              {(settings.advanceOrderMode || requiresSomeTalerOptions) &&
+              {(settings.advanceOrderMode || requiresSomeTalerOptions) && (
                 <InputGroup
                   name="payments"
                   label={i18n.str`Taler payment options`}
                   tooltip={i18n.str`Override default Taler payment settings for this order`}
                 >
-                  {(settings.advanceOrderMode || noDefault_payDeadline) && <InputDate
-                    name="payments.pay_deadline"
-                    label={i18n.str`Payment deadline`}
-                    tooltip={i18n.str`Deadline for the customer to pay for the offer before it expires. Inventory products will be reserved until this deadline.`}
-                    side={
-                      <span>
-                        <button class="button" onClick={() => {
-                          const c = {
-                            ...value,
-                            payments: {
-                              ...(value.payments ?? {}),
-                              pay_deadline: instance_default.payments?.pay_deadline
-                            }
-                          }
-                          valueHandler(c)
-                        }}>
-                          <i18n.Translate>default</i18n.Translate>
-                        </button>
-                      </span>
-                    }
-                  />}
-                  {settings.advanceOrderMode && <InputDate
-                    name="payments.refund_deadline"
-                    label={i18n.str`Refund deadline`}
-                    tooltip={i18n.str`Time until which the order can be refunded by the merchant.`}
-                    side={
-                      <span>
-                        <button class="button" onClick={() => {
-                          valueHandler({
-                            ...value,
-                            payments: {
-                              ...(value.payments ?? {}),
-                              refund_deadline: instance_default.payments?.refund_deadline
-                            }
-                          })
-                        }}>
-                          <i18n.Translate>default</i18n.Translate>
-                        </button>
-                      </span>
-                    }
-                  />}
-                  {(settings.advanceOrderMode || noDefault_wireDeadline) && <InputDate
-                    name="payments.wire_transfer_deadline"
-                    label={i18n.str`Wire transfer deadline`}
-                    tooltip={i18n.str`Deadline for the exchange to make the wire transfer.`}
-                    side={
-                      <span>
-                        <button class="button" onClick={() => {
-                          valueHandler({
-                            ...value,
-                            payments: {
-                              ...(value.payments ?? {}),
-                              wire_transfer_deadline: instance_default.payments?.wire_transfer_deadline
-                            }
-                          })
-                        }}>
-                          <i18n.Translate>default</i18n.Translate>
-                        </button>
-                      </span>
-                    }
-                  />}
-                  {settings.advanceOrderMode && <InputDate
-                    name="payments.auto_refund_deadline"
-                    label={i18n.str`Auto-refund deadline`}
-                    tooltip={i18n.str`Time until which the wallet will automatically check for refunds without user interaction.`}
-                  />}
+                  {(settings.advanceOrderMode || noDefault_payDeadline) && (
+                    <InputDuration
+                      name="payments.pay_deadline"
+                      label={i18n.str`Payment time`}
+                      help={
+                        <DeadlineHelp duration={value.payments?.pay_deadline} />
+                      }
+                      withForever
+                      withoutClear
+                      tooltip={i18n.str`Time for the customer to pay for the offer before it expires. Inventory products will be reserved until this deadline. Time start to run after the order is created.`}
+                      side={
+                        <span>
+                          <button
+                            class="button"
+                            onClick={() => {
+                              const c = {
+                                ...value,
+                                payments: {
+                                  ...(value.payments ?? {}),
+                                  pay_deadline:
+                                    instance_default.payments?.pay_deadline,
+                                },
+                              };
+                              valueHandler(c);
+                            }}
+                          >
+                            <i18n.Translate>default</i18n.Translate>
+                          </button>
+                        </span>
+                      }
+                    />
+                  )}
+                  {settings.advanceOrderMode && (
+                    <InputDuration
+                      name="payments.refund_deadline"
+                      label={i18n.str`Refund time`}
+                      help={
+                        <DeadlineHelp
+                          duration={value.payments?.refund_deadline}
+                        />
+                      }
+                      withForever
+                      withoutClear
+                      tooltip={i18n.str`Time while the order can be refunded by the merchant. Time starts after the order is created.`}
+                      side={
+                        <span>
+                          <button
+                            class="button"
+                            onClick={() => {
+                              valueHandler({
+                                ...value,
+                                payments: {
+                                  ...(value.payments ?? {}),
+                                  refund_deadline:
+                                    instance_default.payments?.refund_deadline,
+                                },
+                              });
+                            }}
+                          >
+                            <i18n.Translate>default</i18n.Translate>
+                          </button>
+                        </span>
+                      }
+                    />
+                  )}
+                  {(settings.advanceOrderMode || noDefault_wireDeadline) && (
+                    <InputDuration
+                      name="payments.wire_transfer_deadline"
+                      label={i18n.str`Wire transfer time`}
+                      help={
+                        <DeadlineHelp
+                          duration={value.payments?.wire_transfer_deadline}
+                        />
+                      }
+                      withoutClear
+                      withForever
+                      tooltip={i18n.str`Time for the exchange to make the wire transfer. Time starts after the order is created.`}
+                      side={
+                        <span>
+                          <button
+                            class="button"
+                            onClick={() => {
+                              valueHandler({
+                                ...value,
+                                payments: {
+                                  ...(value.payments ?? {}),
+                                  wire_transfer_deadline:
+                                    instance_default.payments
+                                      ?.wire_transfer_deadline,
+                                },
+                              });
+                            }}
+                          >
+                            <i18n.Translate>default</i18n.Translate>
+                          </button>
+                        </span>
+                      }
+                    />
+                  )}
+                  {settings.advanceOrderMode && (
+                    <InputDuration
+                      name="payments.auto_refund_deadline"
+                      label={i18n.str`Auto-refund time`}
+                      help={
+                        <DeadlineHelp
+                          duration={value.payments?.auto_refund_deadline}
+                        />
+                      }
+                      tooltip={i18n.str`Time until which the wallet will automatically check for refunds without user interaction.`}
+                      withForever
+                    />
+                  )}
 
-                  {settings.advanceOrderMode && <InputCurrency
-                    name="payments.max_fee"
-                    label={i18n.str`Maximum fee`}
-                    tooltip={i18n.str`Maximum fees the merchant is willing to cover for this order. Higher deposit fees must be covered in full by the consumer.`}
-                  />}
-                  {settings.advanceOrderMode && <InputToggle
-                    name="payments.createToken"
-                    label={i18n.str`Create token`}
-                    tooltip={i18n.str`If the order ID is easy to guess the token will prevent user to steal orders from others.`}
-                  />}
-                  {settings.advanceOrderMode && <InputNumber
-                    name="payments.minimum_age"
-                    label={i18n.str`Minimum age required`}
-                    tooltip={i18n.str`Any value greater than 0 will limit the coins able be used to pay this contract. If empty the age restriction will be defined by the products`}
-                    help={
-                      minAgeByProducts > 0
-                        ? i18n.str`Min age defined by the producs is ${minAgeByProducts}`
-                        : i18n.str`No product with age restriction in this order`
-                    }
-                  />}
+                  {settings.advanceOrderMode && (
+                    <InputCurrency
+                      name="payments.max_fee"
+                      label={i18n.str`Maximum fee`}
+                      tooltip={i18n.str`Maximum fees the merchant is willing to cover for this order. Higher deposit fees must be covered in full by the consumer.`}
+                    />
+                  )}
+                  {settings.advanceOrderMode && (
+                    <InputToggle
+                      name="payments.createToken"
+                      label={i18n.str`Create token`}
+                      tooltip={i18n.str`If the order ID is easy to guess the token will prevent user to steal orders from others.`}
+                    />
+                  )}
+                  {settings.advanceOrderMode && (
+                    <InputNumber
+                      name="payments.minimum_age"
+                      label={i18n.str`Minimum age required`}
+                      tooltip={i18n.str`Any value greater than 0 will limit the coins able be used to pay this contract. If empty the age restriction will be defined by the products`}
+                      help={
+                        minAgeByProducts > 0
+                          ? i18n.str`Min age defined by the producs is ${minAgeByProducts}`
+                          : i18n.str`No product with age restriction in this order`
+                      }
+                    />
+                  )}
                 </InputGroup>
-              }
+              )}
 
-              {settings.advanceOrderMode &&
+              {settings.advanceOrderMode && (
                 <InputGroup
                   name="extra"
                   label={i18n.str`Additional information`}
                   tooltip={i18n.str`Custom information to be included in the contract for this order.`}
                 >
-                  {Object.keys(value.extra ?? {}).map((key) => {
-
-                    return <Input
-                      name={`extra.${key}`}
-                      inputType="multiline"
-                      label={key}
-                      tooltip={i18n.str`You must enter a value in JavaScript Object Notation (JSON).`}
-                      side={
-                        <button class="button" onClick={(e) => {
-                          if (value.extra && value.extra[key] !== undefined) {
-                            console.log(value.extra)
-                            delete value.extra[key]
-                          }
-                          valueHandler({
-                            ...value,
-                          })
-                        }}>remove</button>
-                      }
-                    />
+                  {Object.keys(value.extra ?? {}).map((key, idx) => {
+                    return (
+                      <Input
+                        name={`extra.${key}`}
+                        key={String(idx)}
+                        inputType="multiline"
+                        label={key}
+                        tooltip={i18n.str`You must enter a value in JavaScript Object Notation (JSON).`}
+                        side={
+                          <button
+                            class="button"
+                            onClick={(e) => {
+                              if (
+                                value.extra &&
+                                value.extra[key] !== undefined
+                              ) {
+                                console.log(value.extra);
+                                delete value.extra[key];
+                              }
+                              valueHandler({
+                                ...value,
+                              });
+                              e.preventDefault();
+                            }}
+                          >
+                            remove
+                          </button>
+                        }
+                      />
+                    );
                   })}
                   <div class="field is-horizontal">
                     <div class="field-label is-normal">
                       <label class="label">
                         <i18n.Translate>Custom field name</i18n.Translate>
-                        <span class="icon has-tooltip-right" data-tooltip={"new extra field"}>
+                        <span
+                          class="icon has-tooltip-right"
+                          data-tooltip={"new extra field"}
+                        >
                           <i class="mdi mdi-information" />
                         </span>
                       </label>
@@ -650,23 +709,33 @@ export function CreatePage({
                     <div class="field-body is-flex-grow-3">
                       <div class="field">
                         <p class="control">
-                          <input class="input " value={newField} onChange={(e) => setNewField(e.currentTarget.value)} />
+                          <input
+                            class="input "
+                            value={newField}
+                            onChange={(e) => setNewField(e.currentTarget.value)}
+                          />
                         </p>
                       </div>
                     </div>
-                    <button class="button" onClick={(e) => {
-                      setNewField("")
-                      valueHandler({
-                        ...value,
-                        extra: {
-                          ...(value.extra ?? {}),
-                          [newField]: ""
-                        }
-                      })
-                    }}>add</button>
+                    <button
+                      class="button"
+                      onClick={(e) => {
+                        setNewField("");
+                        valueHandler({
+                          ...value,
+                          extra: {
+                            ...(value.extra ?? {}),
+                            [newField]: "",
+                          },
+                        });
+                        e.preventDefault();
+                      }}
+                    >
+                      add
+                    </button>
                   </div>
                 </InputGroup>
-              }
+              )}
             </FormProvider>
 
             <div class="buttons is-right mt-5">
@@ -691,7 +760,7 @@ export function CreatePage({
   );
 }
 
-function asProduct(p: ProductAndQuantity): MerchantBackend.Product {
+function asProduct(p: ProductAndQuantity): TalerMerchantApi.Product {
   return {
     product_id: p.product.id,
     image: p.product.image,
@@ -700,6 +769,27 @@ function asProduct(p: ProductAndQuantity): MerchantBackend.Product {
     quantity: p.quantity,
     description: p.product.description,
     taxes: p.product.taxes,
-    minimum_age: p.product.minimum_age,
   };
 }
+
+function DeadlineHelp({ duration }: { duration?: Duration }): VNode {
+  const { i18n } = useTranslationContext();
+  const [now, setNow] = useState(AbsoluteTime.now());
+  useEffect(() => {
+    const iid = setInterval(() => {
+      setNow(AbsoluteTime.now());
+    }, 60 * 1000);
+    return () => {
+      clearInterval(iid);
+    };
+  });
+  if (!duration) return <i18n.Translate>Disabled</i18n.Translate>;
+  const when = AbsoluteTime.addDuration(now, duration);
+  if (when.t_ms === "never")
+    return <i18n.Translate>No deadline</i18n.Translate>;
+  return (
+    <i18n.Translate>
+      Deadline at {format(when.t_ms, "dd/MM/yy HH:mm")}
+    </i18n.Translate>
+  );
+}
