@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2021 Taler Systems S.A.
+ (C) 2021-2024 Taler Systems S.A.
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -19,121 +19,119 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
-import { AmountJson, Amounts } from "@gnu-taler/taler-util";
-import { format } from "date-fns";
-import { Fragment, h, VNode } from "preact";
+import {
+  AmountJson,
+  Amounts,
+  TalerMerchantApi,
+  stringifyRefundUri,
+} from "@gnu-taler/taler-util";
+import {
+  useMerchantApiContext,
+  useTranslationContext,
+} from "@gnu-taler/web-util/browser";
+import { format, formatDistance } from "date-fns";
+import { Fragment, VNode, h } from "preact";
 import { useState } from "preact/hooks";
-import { FormProvider } from "../../../../components/form/FormProvider";
-import { Input } from "../../../../components/form/Input";
-import { InputCurrency } from "../../../../components/form/InputCurrency";
-import { InputDate } from "../../../../components/form/InputDate";
-import { InputDuration } from "../../../../components/form/InputDuration";
-import { InputGroup } from "../../../../components/form/InputGroup";
-import { InputLocation } from "../../../../components/form/InputLocation";
-import { TextField } from "../../../../components/form/TextField";
-import { ProductList } from "../../../../components/product/ProductList";
-import { useBackendContext } from "../../../../context/backend";
-import { MerchantBackend } from "../../../../declaration";
-import { Translate, useTranslator } from "../../../../i18n";
-import { mergeRefunds } from "../../../../utils/amount";
-import { RefundModal } from "../list/Table";
-import { Event, Timeline } from "./Timeline";
+import { FormProvider } from "../../../../components/form/FormProvider.js";
+import { Input } from "../../../../components/form/Input.js";
+import { InputCurrency } from "../../../../components/form/InputCurrency.js";
+import { InputDate } from "../../../../components/form/InputDate.js";
+import { InputDuration } from "../../../../components/form/InputDuration.js";
+import { InputGroup } from "../../../../components/form/InputGroup.js";
+import { InputLocation } from "../../../../components/form/InputLocation.js";
+import { TextField } from "../../../../components/form/TextField.js";
+import { ProductList } from "../../../../components/product/ProductList.js";
+import {
+  datetimeFormatForSettings,
+  usePreference,
+} from "../../../../hooks/preference.js";
+import { mergeRefunds } from "../../../../utils/amount.js";
+import { RefundModal } from "../list/Table.js";
+import { Event, Timeline } from "./Timeline.js";
 
-type Entity = MerchantBackend.Orders.MerchantOrderStatusResponse;
-type CT = MerchantBackend.ContractTerms;
+type Entity = TalerMerchantApi.MerchantOrderStatusResponse;
+type CT = TalerMerchantApi.ContractTerms;
 
 interface Props {
   onBack: () => void;
   selected: Entity;
   id: string;
-  onRefund: (id: string, value: MerchantBackend.Orders.RefundRequest) => void;
+  onRefund: (id: string, value: TalerMerchantApi.RefundRequest) => void;
 }
 
-type Paid = MerchantBackend.Orders.CheckPaymentPaidResponse & {
+type Paid = TalerMerchantApi.CheckPaymentPaidResponse & {
   refund_taken: string;
 };
-type Unpaid = MerchantBackend.Orders.CheckPaymentUnpaidResponse;
-type Claimed = MerchantBackend.Orders.CheckPaymentClaimedResponse;
+type Unpaid = TalerMerchantApi.CheckPaymentUnpaidResponse;
+type Claimed = TalerMerchantApi.CheckPaymentClaimedResponse;
 
 function ContractTerms({ value }: { value: CT }) {
-  const i18n = useTranslator();
+  const { i18n } = useTranslationContext();
 
   return (
-    <InputGroup name="contract_terms" label={i18n`Contract Terms`}>
+    <InputGroup name="contract_terms" label={i18n.str`Contract Terms`}>
       <FormProvider<CT> object={value} valueHandler={null}>
         <Input<CT>
           readonly
           name="summary"
-          label={i18n`Summary`}
-          tooltip={i18n`human-readable description of the whole purchase`}
+          label={i18n.str`Summary`}
+          tooltip={i18n.str`human-readable description of the whole purchase`}
         />
         <InputCurrency<CT>
           readonly
           name="amount"
-          label={i18n`Amount`}
-          tooltip={i18n`total price for the transaction`}
+          label={i18n.str`Amount`}
+          tooltip={i18n.str`total price for the transaction`}
         />
         {value.fulfillment_url && (
           <Input<CT>
             readonly
             name="fulfillment_url"
-            label={i18n`Fulfillment URL`}
-            tooltip={i18n`URL for this purchase`}
+            label={i18n.str`Fulfillment URL`}
+            tooltip={i18n.str`URL for this purchase`}
           />
         )}
         <Input<CT>
           readonly
           name="max_fee"
-          label={i18n`Max fee`}
-          tooltip={i18n`maximum total deposit fee accepted by the merchant for this contract`}
-        />
-        <Input<CT>
-          readonly
-          name="max_wire_fee"
-          label={i18n`Max wire fee`}
-          tooltip={i18n`maximum wire fee accepted by the merchant`}
-        />
-        <Input<CT>
-          readonly
-          name="wire_fee_amortization"
-          label={i18n`Wire fee amortization`}
-          tooltip={i18n`over how many customer transactions does the merchant expect to amortize wire fees on average`}
+          label={i18n.str`Max fee`}
+          tooltip={i18n.str`maximum total deposit fee accepted by the merchant for this contract`}
         />
         <InputDate<CT>
           readonly
           name="timestamp"
-          label={i18n`Created at`}
-          tooltip={i18n`time when this contract was generated`}
+          label={i18n.str`Created at`}
+          tooltip={i18n.str`time when this contract was generated`}
         />
         <InputDate<CT>
           readonly
           name="refund_deadline"
-          label={i18n`Refund deadline`}
-          tooltip={i18n`after this deadline has passed no refunds will be accepted`}
+          label={i18n.str`Refund deadline`}
+          tooltip={i18n.str`after this deadline has passed no refunds will be accepted`}
         />
         <InputDate<CT>
           readonly
           name="pay_deadline"
-          label={i18n`Payment deadline`}
-          tooltip={i18n`after this deadline, the merchant won't accept payments for the contract`}
+          label={i18n.str`Payment deadline`}
+          tooltip={i18n.str`after this deadline, the merchant won't accept payments for the contract`}
         />
         <InputDate<CT>
           readonly
           name="wire_transfer_deadline"
-          label={i18n`Wire transfer deadline`}
-          tooltip={i18n`transfer deadline for the exchange`}
+          label={i18n.str`Wire transfer deadline`}
+          tooltip={i18n.str`transfer deadline for the exchange`}
         />
         <InputDate<CT>
           readonly
           name="delivery_date"
-          label={i18n`Delivery date`}
-          tooltip={i18n`time indicating when the order should be delivered`}
+          label={i18n.str`Delivery date`}
+          tooltip={i18n.str`time indicating when the order should be delivered`}
         />
         {value.delivery_date && (
           <InputGroup
             name="delivery_location"
-            label={i18n`Location`}
-            tooltip={i18n`where the order will be delivered`}
+            label={i18n.str`Location`}
+            tooltip={i18n.str`where the order will be delivered`}
           >
             <InputLocation name="payments.delivery_location" />
           </InputGroup>
@@ -141,14 +139,14 @@ function ContractTerms({ value }: { value: CT }) {
         <InputDuration<CT>
           readonly
           name="auto_refund"
-          label={i18n`Auto-refund delay`}
-          tooltip={i18n`how long the wallet should try to get an automatic refund for the purchase`}
+          label={i18n.str`Auto-refund delay`}
+          tooltip={i18n.str`how long the wallet should try to get an automatic refund for the purchase`}
         />
         <Input<CT>
           readonly
           name="extra"
-          label={i18n`Extra info`}
-          tooltip={i18n`extra data that is only interpreted by the merchant frontend`}
+          label={i18n.str`Extra info`}
+          tooltip={i18n.str`extra data that is only interpreted by the merchant frontend`}
         />
       </FormProvider>
     </InputGroup>
@@ -160,8 +158,12 @@ function ClaimedPage({
   order,
 }: {
   id: string;
-  order: MerchantBackend.Orders.CheckPaymentClaimedResponse;
+  order: TalerMerchantApi.CheckPaymentClaimedResponse;
 }) {
+  const now = new Date();
+  const refundable =
+    order.contract_terms.refund_deadline.t_s !== "never" &&
+    now.getTime() < order.contract_terms.refund_deadline.t_s * 1000;
   const events: Event[] = [];
   if (order.contract_terms.timestamp.t_s !== "never") {
     events.push({
@@ -177,20 +179,20 @@ function ClaimedPage({
       type: "deadline",
     });
   }
-  if (order.contract_terms.refund_deadline.t_s !== "never") {
+  if (order.contract_terms.refund_deadline.t_s !== "never" && refundable) {
     events.push({
       when: new Date(order.contract_terms.refund_deadline.t_s * 1000),
       description: "refund deadline",
       type: "deadline",
     });
   }
-  if (order.contract_terms.wire_transfer_deadline.t_s !== "never") {
-    events.push({
-      when: new Date(order.contract_terms.wire_transfer_deadline.t_s * 1000),
-      description: "wire deadline",
-      type: "deadline",
-    });
-  }
+  // if (order.contract_terms.wire_transfer_deadline.t_s !== "never") {
+  //   events.push({
+  //     when: new Date(order.contract_terms.wire_transfer_deadline.t_s * 1000),
+  //     description: "wire deadline",
+  //     type: "deadline",
+  //   });
+  // }
   if (
     order.contract_terms.delivery_date &&
     order.contract_terms.delivery_date.t_s !== "never"
@@ -203,7 +205,8 @@ function ClaimedPage({
   }
 
   const [value, valueHandler] = useState<Partial<Claimed>>(order);
-  const i18n = useTranslator();
+  const { i18n } = useTranslationContext();
+  const [settings] = usePreference();
 
   return (
     <div>
@@ -216,13 +219,14 @@ function ClaimedPage({
                 <div class="level">
                   <div class="level-left">
                     <div class="level-item">
-                      <Translate>Order</Translate> #{id}
+                      <i18n.Translate>Order</i18n.Translate> #{id}
                       <div class="tag is-info ml-4">
-                        <Translate>claimed</Translate>
+                        <i18n.Translate>claimed</i18n.Translate>
                       </div>
                     </div>
                   </div>
                 </div>
+
                 <div class="level">
                   <div class="level-left">
                     <div class="level-item">
@@ -244,12 +248,16 @@ function ClaimedPage({
                       >
                         <p>
                           <b>
-                            <Translate>claimed at</Translate>:
+                            <i18n.Translate>claimed at</i18n.Translate>:
                           </b>{" "}
-                          {format(
-                            new Date(order.contract_terms.timestamp.t_s * 1000),
-                            "yyyy-MM-dd HH:mm:ss"
-                          )}
+                          {order.contract_terms.timestamp.t_s === "never"
+                            ? "never"
+                            : format(
+                                new Date(
+                                  order.contract_terms.timestamp.t_s * 1000,
+                                ),
+                                datetimeFormatForSettings(settings),
+                              )}
                         </p>
                       </div>
                     </div>
@@ -262,13 +270,13 @@ function ClaimedPage({
               <div class="columns">
                 <div class="column is-4">
                   <div class="title">
-                    <Translate>Timeline</Translate>
+                    <i18n.Translate>Timeline</i18n.Translate>
                   </div>
                   <Timeline events={events} />
                 </div>
                 <div class="column is-8">
                   <div class="title">
-                    <Translate>Payment details</Translate>
+                    <i18n.Translate>Payment details</i18n.Translate>
                   </div>
                   <FormProvider<Claimed>
                     object={value}
@@ -278,17 +286,17 @@ function ClaimedPage({
                       name="contract_terms.summary"
                       readonly
                       inputType="multiline"
-                      label={i18n`Summary`}
+                      label={i18n.str`Summary`}
                     />
                     <InputCurrency
                       name="contract_terms.amount"
                       readonly
-                      label={i18n`Amount`}
+                      label={i18n.str`Amount`}
                     />
                     <Input<Claimed>
                       name="order_status"
                       readonly
-                      label={i18n`Order status`}
+                      label={i18n.str`Order status`}
                     />
                   </FormProvider>
                 </div>
@@ -298,7 +306,7 @@ function ClaimedPage({
             {order.contract_terms.products.length ? (
               <Fragment>
                 <div class="title">
-                  <Translate>Product list</Translate>
+                  <i18n.Translate>Product list</i18n.Translate>
                 </div>
                 <ProductList list={order.contract_terms.products} />
               </Fragment>
@@ -320,25 +328,16 @@ function PaidPage({
   onRefund,
 }: {
   id: string;
-  order: MerchantBackend.Orders.CheckPaymentPaidResponse;
+  order: TalerMerchantApi.CheckPaymentPaidResponse;
   onRefund: (id: string) => void;
 }) {
+  const now = new Date();
+  const refundable =
+    order.contract_terms.refund_deadline.t_s !== "never" &&
+    now.getTime() < order.contract_terms.refund_deadline.t_s * 1000;
+
   const events: Event[] = [];
-  if (order.contract_terms.timestamp.t_s !== "never") {
-    events.push({
-      when: new Date(order.contract_terms.timestamp.t_s * 1000),
-      description: "order created",
-      type: "start",
-    });
-  }
-  if (order.contract_terms.pay_deadline.t_s !== "never") {
-    events.push({
-      when: new Date(order.contract_terms.pay_deadline.t_s * 1000),
-      description: "pay deadline",
-      type: "deadline",
-    });
-  }
-  if (order.contract_terms.refund_deadline.t_s !== "never") {
+  if (order.contract_terms.refund_deadline.t_s !== "never" && refundable) {
     events.push({
       when: new Date(order.contract_terms.refund_deadline.t_s * 1000),
       description: "refund deadline",
@@ -372,67 +371,78 @@ function PaidPage({
       });
     }
   });
-  if (order.wire_details && order.wire_details.length) {
-    if (order.wire_details.length > 1) {
-      let last: MerchantBackend.Orders.TransactionWireTransfer | null = null;
-      let first: MerchantBackend.Orders.TransactionWireTransfer | null = null;
-      let total: AmountJson | null = null;
+  const ra = !order.refunded ? undefined : Amounts.parse(order.refund_amount);
+  const am = Amounts.parseOrThrow(order.contract_terms.amount);
+  if (ra && Amounts.cmp(ra, am) === 1) {
+    if (order.wire_details && order.wire_details.length) {
+      if (order.wire_details.length > 1) {
+        let last: TalerMerchantApi.TransactionWireTransfer | null = null;
+        let first: TalerMerchantApi.TransactionWireTransfer | null = null;
+        let total: AmountJson | null = null;
 
-      order.wire_details.forEach((w) => {
-        if (last === null || last.execution_time.t_s < w.execution_time.t_s) {
-          last = w;
-        }
-        if (first === null || first.execution_time.t_s > w.execution_time.t_s) {
-          first = w;
-        }
-        total =
-          total === null
-            ? Amounts.parseOrThrow(w.amount)
-            : Amounts.add(total, Amounts.parseOrThrow(w.amount)).amount;
-      });
-      const last_time = last!.execution_time.t_s;
-      if (last_time !== "never") {
-        events.push({
-          when: new Date(last_time * 1000),
-          description: `wired ${Amounts.stringify(total!)}`,
-          type: "wired-range",
+        order.wire_details.forEach((w) => {
+          if (last === null || last.execution_time.t_s < w.execution_time.t_s) {
+            last = w;
+          }
+          if (
+            first === null ||
+            first.execution_time.t_s > w.execution_time.t_s
+          ) {
+            first = w;
+          }
+          total =
+            total === null
+              ? Amounts.parseOrThrow(w.amount)
+              : Amounts.add(total, Amounts.parseOrThrow(w.amount)).amount;
         });
-      }
-      const first_time = first!.execution_time.t_s;
-      if (first_time !== "never") {
-        events.push({
-          when: new Date(first_time * 1000),
-          description: `wire transfer started...`,
-          type: "wired-range",
-        });
-      }
-    } else {
-      order.wire_details.forEach((e) => {
-        if (e.execution_time.t_s !== "never") {
+        const last_time = last!.execution_time.t_s;
+        if (last_time !== "never") {
           events.push({
-            when: new Date(e.execution_time.t_s * 1000),
-            description: `wired ${e.amount}`,
-            type: "wired",
+            when: new Date(last_time * 1000),
+            description: `wired ${Amounts.stringify(total!)}`,
+            type: "wired-range",
           });
         }
-      });
+        const first_time = first!.execution_time.t_s;
+        if (first_time !== "never") {
+          events.push({
+            when: new Date(first_time * 1000),
+            description: `wire transfer started...`,
+            type: "wired-range",
+          });
+        }
+      } else {
+        order.wire_details.forEach((e) => {
+          if (e.execution_time.t_s !== "never") {
+            events.push({
+              when: new Date(e.execution_time.t_s * 1000),
+              description: `wired ${e.amount}`,
+              type: "wired",
+            });
+          }
+        });
+      }
     }
   }
 
+  const nextEvent = events.find((e) => {
+    return e.when.getTime() > now.getTime();
+  });
+
   const [value, valueHandler] = useState<Partial<Paid>>(order);
-  const { url } = useBackendContext();
-  const refundHost = url.replace(/.*:\/\//, ""); // remove protocol part
-  const proto = url.startsWith("http://") ? "taler+http" : "taler";
-  const refundurl = `${proto}://refund/${refundHost}/${order.contract_terms.order_id}/`;
-  const refundable =
-    new Date().getTime() < order.contract_terms.refund_deadline.t_s * 1000;
-  const i18n = useTranslator();
+  const { url: backendUrl } = useMerchantApiContext();
+
+  const refundurl = stringifyRefundUri({
+    merchantBaseUrl: backendUrl.href,
+    orderId: order.contract_terms.order_id,
+  });
+  const { i18n } = useTranslationContext();
 
   const amount = Amounts.parseOrThrow(order.contract_terms.amount);
   const refund_taken = order.refund_details.reduce((prev, cur) => {
     if (cur.pending) return prev;
     return Amounts.add(prev, Amounts.parseOrThrow(cur.amount)).amount;
-  }, Amounts.getZero(amount.currency));
+  }, Amounts.zeroOfCurrency(amount.currency));
   value.refund_taken = Amounts.stringify(refund_taken);
 
   return (
@@ -446,18 +456,18 @@ function PaidPage({
                 <div class="level">
                   <div class="level-left">
                     <div class="level-item">
-                      <Translate>Order</Translate> #{id}
+                      <i18n.Translate>Order</i18n.Translate> #{id}
                       <div class="tag is-success ml-4">
-                        <Translate>paid</Translate>
+                        <i18n.Translate>paid</i18n.Translate>
                       </div>
                       {order.wired ? (
                         <div class="tag is-success ml-4">
-                          <Translate>wired</Translate>
+                          <i18n.Translate>wired</i18n.Translate>
                         </div>
                       ) : null}
                       {order.refunded ? (
                         <div class="tag is-danger ml-4">
-                          <Translate>refunded</Translate>
+                          <i18n.Translate>refunded</i18n.Translate>
                         </div>
                       ) : null}
                     </div>
@@ -477,8 +487,8 @@ function PaidPage({
                             class="has-tooltip-left"
                             data-tooltip={
                               refundable
-                                ? i18n`refund order`
-                                : i18n`not refundable`
+                                ? i18n.str`refund order`
+                                : i18n.str`not refundable`
                             }
                           >
                             <button
@@ -486,7 +496,7 @@ function PaidPage({
                               disabled={!refundable}
                               onClick={() => onRefund(id)}
                             >
-                              <Translate>refund</Translate>
+                              <i18n.Translate>refund</i18n.Translate>
                             </button>
                           </span>
                         </div>
@@ -504,24 +514,18 @@ function PaidPage({
                           whiteSpace: "nowrap",
                           overflow: "hidden",
                           textOverflow: "ellipsis",
-                          // maxWidth: '100%',
                         }}
                       >
-                        <p>
-                          <a
-                            href={order.contract_terms.fulfillment_url}
-                            rel="nofollow"
-                            target="new"
-                          >
-                            {order.contract_terms.fulfillment_url}
-                          </a>
-                        </p>
-                        <p>
-                          {format(
-                            new Date(order.contract_terms.timestamp.t_s * 1000),
-                            "yyyy/MM/dd HH:mm:ss"
-                          )}
-                        </p>
+                        {nextEvent && (
+                          <p>
+                            <i18n.Translate>Next event in </i18n.Translate>{" "}
+                            {formatDistance(
+                              nextEvent.when,
+                              new Date(),
+                              // "yyyy/MM/dd HH:mm:ss",
+                            )}
+                          </p>
+                        )}
                       </div>
                     </div>
                   </div>
@@ -533,41 +537,41 @@ function PaidPage({
               <div class="columns">
                 <div class="column is-4">
                   <div class="title">
-                    <Translate>Timeline</Translate>
+                    <i18n.Translate>Timeline</i18n.Translate>
                   </div>
                   <Timeline events={events} />
                 </div>
                 <div class="column is-8">
                   <div class="title">
-                    <Translate>Payment details</Translate>
+                    <i18n.Translate>Payment details</i18n.Translate>
                   </div>
                   <FormProvider<Paid>
                     object={value}
                     valueHandler={valueHandler}
                   >
-                    {/* <InputCurrency<Paid> name="deposit_total" readonly label={i18n`Deposit total`} /> */}
+                    {/* <InputCurrency<Paid> name="deposit_total" readonly label={i18n.str`Deposit total`} /> */}
                     {order.refunded && (
                       <InputCurrency<Paid>
                         name="refund_amount"
                         readonly
-                        label={i18n`Refunded amount`}
+                        label={i18n.str`Refunded amount`}
                       />
                     )}
                     {order.refunded && (
                       <InputCurrency<Paid>
                         name="refund_taken"
                         readonly
-                        label={i18n`Refund taken`}
+                        label={i18n.str`Refund taken`}
                       />
                     )}
                     <Input<Paid>
                       name="order_status"
                       readonly
-                      label={i18n`Order status`}
+                      label={i18n.str`Order status`}
                     />
                     <TextField<Paid>
                       name="order_status_url"
-                      label={i18n`Status URL`}
+                      label={i18n.str`Status URL`}
                     >
                       <a
                         target="_blank"
@@ -580,7 +584,7 @@ function PaidPage({
                     {order.refunded && (
                       <TextField<Paid>
                         name="order_status_url"
-                        label={i18n`Refund URI`}
+                        label={i18n.str`Refund URI`}
                       >
                         <a target="_blank" rel="noreferrer" href={refundurl}>
                           {refundurl}
@@ -595,7 +599,7 @@ function PaidPage({
             {order.contract_terms.products.length ? (
               <Fragment>
                 <div class="title">
-                  <Translate>Product list</Translate>
+                  <i18n.Translate>Product list</i18n.Translate>
                 </div>
                 <ProductList list={order.contract_terms.products} />
               </Fragment>
@@ -617,10 +621,11 @@ function UnpaidPage({
   order,
 }: {
   id: string;
-  order: MerchantBackend.Orders.CheckPaymentUnpaidResponse;
+  order: TalerMerchantApi.CheckPaymentUnpaidResponse;
 }) {
   const [value, valueHandler] = useState<Partial<Unpaid>>(order);
-  const i18n = useTranslator();
+  const { i18n } = useTranslationContext();
+  const [settings] = usePreference();
   return (
     <div>
       <section class="hero is-hero-bar">
@@ -629,11 +634,11 @@ function UnpaidPage({
             <div class="level-left">
               <div class="level-item">
                 <h1 class="title">
-                  <Translate>Order</Translate> #{id}
+                  <i18n.Translate>Order</i18n.Translate> #{id}
                 </h1>
               </div>
               <div class="tag is-dark">
-                <Translate>unpaid</Translate>
+                <i18n.Translate>unpaid</i18n.Translate>
               </div>
             </div>
           </div>
@@ -651,7 +656,7 @@ function UnpaidPage({
                 >
                   <p>
                     <b>
-                      <Translate>pay at</Translate>:
+                      <i18n.Translate>pay at</i18n.Translate>:
                     </b>{" "}
                     <a
                       href={order.order_status_url}
@@ -663,13 +668,13 @@ function UnpaidPage({
                   </p>
                   <p>
                     <b>
-                      <Translate>created at</Translate>:
+                      <i18n.Translate>created at</i18n.Translate>:
                     </b>{" "}
                     {order.creation_time.t_s === "never"
                       ? "never"
                       : format(
                           new Date(order.creation_time.t_s * 1000),
-                          "yyyy-MM-dd HH:mm:ss"
+                          datetimeFormatForSettings(settings),
                         )}
                   </p>
                 </div>
@@ -687,26 +692,29 @@ function UnpaidPage({
               <Input<Unpaid>
                 readonly
                 name="summary"
-                label={i18n`Summary`}
-                tooltip={i18n`human-readable description of the whole purchase`}
+                label={i18n.str`Summary`}
+                tooltip={i18n.str`human-readable description of the whole purchase`}
               />
               <InputCurrency<Unpaid>
                 readonly
                 name="total_amount"
-                label={i18n`Amount`}
-                tooltip={i18n`total price for the transaction`}
+                label={i18n.str`Amount`}
+                tooltip={i18n.str`total price for the transaction`}
               />
               <Input<Unpaid>
                 name="order_status"
                 readonly
-                label={i18n`Order status`}
+                label={i18n.str`Order status`}
               />
               <Input<Unpaid>
                 name="order_status_url"
                 readonly
-                label={i18n`Order status URL`}
+                label={i18n.str`Order status URL`}
               />
-              <TextField<Unpaid> name="taler_pay_uri" label={i18n`Payment URI`}>
+              <TextField<Unpaid>
+                name="taler_pay_uri"
+                label={i18n.str`Payment URI`}
+              >
                 <a target="_blank" rel="noreferrer" href={value.taler_pay_uri}>
                   {value.taler_pay_uri}
                 </a>
@@ -722,7 +730,7 @@ function UnpaidPage({
 
 export function DetailPage({ id, selected, onRefund, onBack }: Props): VNode {
   const [showRefund, setShowRefund] = useState<string | undefined>(undefined);
-
+  const { i18n } = useTranslationContext();
   const DetailByStatus = function () {
     switch (selected.order_status) {
       case "claimed":
@@ -734,10 +742,10 @@ export function DetailPage({ id, selected, onRefund, onBack }: Props): VNode {
       default:
         return (
           <div>
-            <Translate>
+            <i18n.Translate>
               Unknown order status. This is an error, please contact the
               administrator.
-            </Translate>
+            </i18n.Translate>
           </div>
         );
     }
@@ -761,7 +769,7 @@ export function DetailPage({ id, selected, onRefund, onBack }: Props): VNode {
         <div class="column is-four-fifths">
           <div class="buttons is-right mt-5">
             <button class="button" onClick={onBack}>
-              <Translate>Back</Translate>
+              <i18n.Translate>Back</i18n.Translate>
             </button>
           </div>
         </div>
@@ -770,7 +778,3 @@ export function DetailPage({ id, selected, onRefund, onBack }: Props): VNode {
     </Fragment>
   );
 }
-
-async function copyToClipboard(text: string) {
-  return navigator.clipboard.writeText(text);
-}
