@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2021-2023 Taler Systems S.A.
+ (C) 2021-2024 Taler Systems S.A.
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -19,61 +19,60 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
+import { HttpStatusCode, TalerError, TalerMerchantApi, assertUnreachable } from "@gnu-taler/taler-util";
 import {
-  ErrorType,
-  HttpError,
-  useTranslationContext,
+  useMerchantApiContext,
+  useTranslationContext
 } from "@gnu-taler/web-util/browser";
-import { h, VNode } from "preact";
+import { VNode, h } from "preact";
 import { useState } from "preact/hooks";
+import { ErrorLoadingMerchant } from "../../../../components/ErrorLoadingMerchant.js";
 import { Loading } from "../../../../components/exception/loading.js";
+import { JumpToElementById } from "../../../../components/form/JumpToElementById.js";
 import { NotificationCard } from "../../../../components/menu/index.js";
-import { MerchantBackend, WithId } from "../../../../declaration.js";
+import { ConfirmModal } from "../../../../components/modal/index.js";
+import { useSessionContext } from "../../../../context/session.js";
 import {
-  useInstanceProducts,
-  useProductAPI,
+  useInstanceProducts
 } from "../../../../hooks/product.js";
 import { Notification } from "../../../../utils/types.js";
+import { NotFoundPageOrAdminCreate } from "../../../notfound/index.js";
 import { CardTable } from "./Table.js";
-import { HttpStatusCode } from "@gnu-taler/taler-util";
-import { ConfirmModal, DeleteModal } from "../../../../components/modal/index.js";
-import { JumpToElementById } from "../../../../components/form/JumpToElementById.js";
+import { LoginPage } from "../../../login/index.js";
 
 interface Props {
-  onUnauthorized: () => VNode;
-  onNotFound: () => VNode;
   onCreate: () => void;
   onSelect: (id: string) => void;
-  onLoadError: (e: HttpError<MerchantBackend.ErrorDetail>) => VNode;
 }
 export default function ProductList({
-  onUnauthorized,
-  onLoadError,
   onCreate,
   onSelect,
-  onNotFound,
 }: Props): VNode {
   const result = useInstanceProducts();
-  const { deleteProduct, updateProduct, getProduct } = useProductAPI();
+  const { lib } = useMerchantApiContext();
+  const { state } = useSessionContext();
   const [deleting, setDeleting] =
-    useState<MerchantBackend.Products.ProductDetail & WithId | null>(null);
+    useState<TalerMerchantApi.ProductDetail & WithId | null>(null);
   const [notif, setNotif] = useState<Notification | undefined>(undefined);
 
   const { i18n } = useTranslationContext();
 
-  if (result.loading) return <Loading />;
-  if (!result.ok) {
-    if (
-      result.type === ErrorType.CLIENT &&
-      result.status === HttpStatusCode.Unauthorized
-    )
-      return onUnauthorized();
-    if (
-      result.type === ErrorType.CLIENT &&
-      result.status === HttpStatusCode.NotFound
-    )
-      return onNotFound();
-    return onLoadError(result);
+  if (!result) return <Loading />;
+  if (result instanceof TalerError) {
+    return <ErrorLoadingMerchant error={result} />;
+  }
+  if (result.type === "fail") {
+    switch (result.case) {
+      case HttpStatusCode.NotFound: {
+        return <NotFoundPageOrAdminCreate />;
+      }
+      case HttpStatusCode.Unauthorized: {
+        return <LoginPage />
+      }
+      default: {
+        assertUnreachable(result);
+      }
+    }
   }
 
   return (
@@ -81,33 +80,38 @@ export default function ProductList({
       <NotificationCard notification={notif} />
 
       <JumpToElementById
-        testIfExist={getProduct} 
+        testIfExist={async (id) => {
+          const resp = await lib.instance.getProductDetails(state.token, id);
+          return resp.type === "ok";
+        }}
         onSelect={onSelect}
         description={i18n.str`jump to product with the given product ID`}
-        palceholder={i18n.str`product id`}
+        placeholder={i18n.str`product id`}
       />
 
       <CardTable
-        instances={result.data}
+        instances={result.body}
+        onLoadMoreBefore={result.isFirstPage ? undefined : result.loadFirst}
+        onLoadMoreAfter={result.isLastPage ? undefined : result.loadNext}
         onCreate={onCreate}
-        onUpdate={(id, prod) =>
-          updateProduct(id, prod)
-            .then(() =>
-              setNotif({
-                message: i18n.str`product updated successfully`,
-                type: "SUCCESS",
-              }),
-            )
-            .catch((error) =>
-              setNotif({
-                message: i18n.str`could not update the product`,
-                type: "ERROR",
-                description: error.message,
-              }),
-            )
-        }
+        onUpdate={async (id, prod) => {
+          try {
+            await lib.instance.updateProduct(state.token, id, prod);
+            setNotif({
+              message: i18n.str`product updated successfully`,
+              type: "SUCCESS",
+            });
+          } catch (error) {
+            setNotif({
+              message: i18n.str`could not update the product`,
+              type: "ERROR",
+              description: error instanceof Error ? error.message : undefined,
+            });
+          }
+          return
+        }}
         onSelect={(product) => onSelect(product.id)}
-        onDelete={(prod: MerchantBackend.Products.ProductDetail & WithId) =>
+        onDelete={(prod: TalerMerchantApi.ProductDetail & WithId) =>
           setDeleting(prod)
         }
       />
@@ -121,7 +125,7 @@ export default function ProductList({
           onCancel={() => setDeleting(null)}
           onConfirm={async (): Promise<void> => {
             try {
-              await deleteProduct(deleting.id);
+              await lib.instance.deleteProduct(state.token, deleting.id);
               setNotif({
                 message: i18n.str`Product "${deleting.description}" (ID: ${deleting.id}) has been deleted`,
                 type: "SUCCESS",
