@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2021-2023 Taler Systems S.A.
+ (C) 2021-2024 Taler Systems S.A.
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -19,30 +19,28 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
+import { HttpStatusCode, TalerError, TalerMerchantApi, assertUnreachable } from "@gnu-taler/taler-util";
 import {
-  ErrorType,
-  HttpError,
-  useTranslationContext,
+  useMerchantApiContext,
+  useTranslationContext
 } from "@gnu-taler/web-util/browser";
-import { Fragment, h, VNode } from "preact";
+import { VNode, h } from "preact";
 import { useState } from "preact/hooks";
+import { ErrorLoadingMerchant } from "../../../../components/ErrorLoadingMerchant.js";
 import { Loading } from "../../../../components/exception/loading.js";
+import { JumpToElementById } from "../../../../components/form/JumpToElementById.js";
 import { NotificationCard } from "../../../../components/menu/index.js";
-import { MerchantBackend } from "../../../../declaration.js";
+import { ConfirmModal } from "../../../../components/modal/index.js";
+import { useSessionContext } from "../../../../context/session.js";
 import {
-  useInstanceTemplates,
-  useTemplateAPI,
+  useInstanceTemplates
 } from "../../../../hooks/templates.js";
 import { Notification } from "../../../../utils/types.js";
+import { NotFoundPageOrAdminCreate } from "../../../notfound/index.js";
 import { ListPage } from "./ListPage.js";
-import { HttpStatusCode, TranslatedString } from "@gnu-taler/taler-util";
-import { ConfirmModal } from "../../../../components/modal/index.js";
-import { JumpToElementById } from "../../../../components/form/JumpToElementById.js";
+import { LoginPage } from "../../../login/index.js";
 
 interface Props {
-  onUnauthorized: () => VNode;
-  onLoadError: (error: HttpError<MerchantBackend.ErrorDetail>) => VNode;
-  onNotFound: () => VNode;
   onCreate: () => void;
   onSelect: (id: string) => void;
   onNewOrder: (id: string) => void;
@@ -50,35 +48,35 @@ interface Props {
 }
 
 export default function ListTemplates({
-  onUnauthorized,
-  onLoadError,
   onCreate,
   onQR,
   onSelect,
   onNewOrder,
-  onNotFound,
 }: Props): VNode {
-  const [position, setPosition] = useState<string | undefined>(undefined);
   const { i18n } = useTranslationContext();
   const [notif, setNotif] = useState<Notification | undefined>(undefined);
-  const { deleteTemplate, testTemplateExist } = useTemplateAPI();
-  const result = useInstanceTemplates({ position }, (id) => setPosition(id));
+  const { lib } = useMerchantApiContext();
+  const result = useInstanceTemplates();
   const [deleting, setDeleting] =
-    useState<MerchantBackend.Template.TemplateEntry | null>(null);
+    useState<TalerMerchantApi.TemplateEntry | null>(null);
+  const { state } = useSessionContext();
 
-  if (result.loading) return <Loading />;
-  if (!result.ok) {
-    if (
-      result.type === ErrorType.CLIENT &&
-      result.status === HttpStatusCode.Unauthorized
-    )
-      return onUnauthorized();
-    if (
-      result.type === ErrorType.CLIENT &&
-      result.status === HttpStatusCode.NotFound
-    )
-      return onNotFound();
-    return onLoadError(result);
+  if (!result) return <Loading />
+  if (result instanceof TalerError) {
+    return <ErrorLoadingMerchant error={result} />
+  }
+  if (result.type === "fail") {
+    switch(result.case) {
+      case HttpStatusCode.NotFound: {
+        return <NotFoundPageOrAdminCreate />
+      }
+      case HttpStatusCode.Unauthorized: {
+        return <LoginPage />
+      }
+      default: {
+        assertUnreachable(result)
+      }
+    }
   }
 
   return (
@@ -86,18 +84,21 @@ export default function ListTemplates({
       <NotificationCard notification={notif} />
 
       <JumpToElementById
-        testIfExist={testTemplateExist}
+        testIfExist={async (id) => {
+          const resp = await lib.instance.getTemplateDetails(state.token, id)
+          return resp.type === "ok"
+        }}
         onSelect={onSelect}
         description={i18n.str`jump to template with the given template ID`}
         placeholder={i18n.str`template id`}
       />
 
       <ListPage
-        templates={result.data.templates}
+        templates={result.body}
         onLoadMoreBefore={
-          result.isReachingStart ? result.loadMorePrev : undefined
+          result.isFirstPage ? undefined: result.loadFirst
         }
-        onLoadMoreAfter={result.isReachingEnd ? result.loadMore : undefined}
+        onLoadMoreAfter={result.isLastPage ? undefined : result.loadNext}
         onCreate={onCreate}
         onSelect={(e) => {
           onSelect(e.template_id);
@@ -108,7 +109,7 @@ export default function ListTemplates({
         onQR={(e) => {
           onQR(e.template_id);
         }}
-        onDelete={(e: MerchantBackend.Template.TemplateEntry) => {
+        onDelete={(e: TalerMerchantApi.TemplateEntry) => {
           setDeleting(e)
         }
         }
@@ -123,7 +124,7 @@ export default function ListTemplates({
           onCancel={() => setDeleting(null)}
           onConfirm={async (): Promise<void> => {
             try {
-              await deleteTemplate(deleting.template_id);
+              await lib.instance.deleteTemplate(state.token, deleting.template_id);
               setNotif({
                 message: i18n.str`Template "${deleting.template_description}" (ID: ${deleting.template_id}) has been deleted`,
                 type: "SUCCESS",
