@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2021-2023 Taler Systems S.A.
+ (C) 2021-2024 Taler Systems S.A.
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -19,8 +19,9 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
+import { Duration, TalerMerchantApi } from "@gnu-taler/taler-util";
 import { useTranslationContext } from "@gnu-taler/web-util/browser";
-import { h, VNode } from "preact";
+import { VNode, h } from "preact";
 import { useState } from "preact/hooks";
 import { AsyncButton } from "../../../components/exception/AsyncButton.js";
 import {
@@ -28,71 +29,41 @@ import {
   FormProvider,
 } from "../../../components/form/FormProvider.js";
 import { DefaultInstanceFormFields } from "../../../components/instance/DefaultInstanceFormFields.js";
-import { useInstanceContext } from "../../../context/instance.js";
-import { MerchantBackend } from "../../../declaration.js";
+import { useSessionContext } from "../../../context/session.js";
 import { undefinedIfEmpty } from "../../../utils/table.js";
 
-type Entity = MerchantBackend.Instances.InstanceReconfigurationMessage & {
-  auth_token?: string;
+export type Entity = Omit<Omit<TalerMerchantApi.InstanceReconfigurationMessage, "default_pay_delay">, "default_wire_transfer_delay"> & {
+  default_pay_delay: Duration,
+  default_wire_transfer_delay: Duration,
 };
 
-//MerchantBackend.Instances.InstanceAuthConfigurationMessage
+//TalerMerchantApi.InstanceAuthConfigurationMessage
 interface Props {
-  onUpdate: (d: Entity) => void;
-  selected: MerchantBackend.Instances.QueryInstancesResponse;
+  onUpdate: (d: TalerMerchantApi.InstanceReconfigurationMessage) => void;
+  selected: TalerMerchantApi.QueryInstancesResponse;
   isLoading: boolean;
   onBack: () => void;
 }
 
 function convert(
-  from: MerchantBackend.Instances.QueryInstancesResponse,
+  from: TalerMerchantApi.QueryInstancesResponse,
 ): Entity {
-  const { ...rest } = from;
-  // const accounts = qAccounts
-  //   .filter((a) => a.active)
-  //   .map(
-  //     (a) =>
-  //     ({
-  //       payto_uri: a.payto_uri,
-  //       credit_facade_url: a.credit_facade_url,
-  //       credit_facade_credentials: a.credit_facade_credentials,
-  //     } as MerchantBackend.Instances.MerchantBankAccount),
-  //   );
+  const { default_pay_delay, default_wire_transfer_delay, ...rest } = from;
+
   const defaults = {
     use_stefan: false,
-    default_pay_delay: { d_us: 2 * 1000 * 1000 * 60 * 60 }, //two hours
-    default_wire_transfer_delay: { d_us: 2 * 1000 * 1000 * 60 * 60 * 2 }, //two hours
+    default_pay_delay: Duration.fromTalerProtocolDuration(default_pay_delay),
+    default_wire_transfer_delay: Duration.fromTalerProtocolDuration(default_wire_transfer_delay),
   };
   return { ...defaults, ...rest };
 }
 
-function getTokenValuePart(t?: string): string | undefined {
-  if (!t) return t;
-  const match = /secret-token:(.*)/.exec(t);
-  if (!match || !match[1]) return undefined;
-  return match[1];
-}
-
 export function UpdatePage({
   onUpdate,
   selected,
   onBack,
 }: Props): VNode {
-  const { id } = useInstanceContext();
-  // const currentTokenValue = getTokenValuePart(token);
-
-  // function updateToken(token: string | undefined | null) {
-  //   const value =
-  //     token && token.startsWith("secret-token:")
-  //       ? token.substring("secret-token:".length)
-  //       : token;
-
-  //   if (!token) {
-  //     onChangeAuth({ method: "external" });
-  //   } else {
-  //     onChangeAuth({ method: "token", token: `secret-token:${value}` });
-  //   }
-  // }
+  const { state } = useSessionContext();
 
   const [value, valueHandler] = useState<Partial<Entity>>(convert(selected));
 
@@ -108,9 +79,9 @@ export function UpdatePage({
     default_pay_delay: !value.default_pay_delay
       ? i18n.str`required`
       : !!value.default_wire_transfer_delay &&
-        value.default_wire_transfer_delay.d_us !== "forever" &&
-        value.default_pay_delay.d_us !== "forever" &&
-        value.default_pay_delay.d_us > value.default_wire_transfer_delay.d_us ?
+        value.default_wire_transfer_delay.d_ms !== "forever" &&
+        value.default_pay_delay.d_ms !== "forever" &&
+        value.default_pay_delay.d_ms > value.default_wire_transfer_delay.d_ms ?
         i18n.str`pay delay can't be greater than wire transfer delay` : undefined,
     default_wire_transfer_delay: !value.default_wire_transfer_delay
       ? i18n.str`required`
@@ -134,7 +105,13 @@ export function UpdatePage({
   );
 
   const submit = async (): Promise<void> => {
-    await onUpdate(value as Entity);
+    const { default_pay_delay, default_wire_transfer_delay, ...rest } = value as Required<Entity>;
+    const result: TalerMerchantApi.InstanceReconfigurationMessage = {
+      default_pay_delay: Duration.toTalerProtocolDuration(default_pay_delay),
+      default_wire_transfer_delay: Duration.toTalerProtocolDuration(default_wire_transfer_delay),
+      ...rest,
+    }
+    await onUpdate(result);
   };
   // const [active, setActive] = useState(false);
 
@@ -147,30 +124,10 @@ export function UpdatePage({
               <div class="level-left">
                 <div class="level-item">
                   <span class="is-size-4">
-                    <i18n.Translate>Instance id</i18n.Translate>: <b>{id}</b>
+                    <i18n.Translate>Instance id</i18n.Translate>: <b>{state.instance}</b>
                   </span>
                 </div>
               </div>
-              {/* <div class="level-right">
-                <div class="level-item">
-                  <h1 class="title">
-                    <button
-                      class="button is-danger"
-                      data-tooltip={i18n.str`Change the authorization method use for this instance.`}
-                      onClick={(): void => {
-                        setActive(!active);
-                      }}
-                    >
-                      <div class="icon is-left">
-                        <i class="mdi mdi-lock-reset" />
-                      </div>
-                      <span>
-                        <i18n.Translate>Manage access token</i18n.Translate>
-                      </span>
-                    </button>
-                  </h1>
-                </div>
-              </div> */}
             </div>
           </div>
         </section>
