@@ -48,10 +48,6 @@ export async function runAgeRestrictionsMerchantTest(t: GlobalTestState) {
     },
   );
 
-  const merchantClient = new MerchantApiClient(
-    merchant.makeInstanceBaseUrl("default"),
-  );
-
   const { walletClient: walletClientTwo } = await createWalletDaemonWithClient(
     t,
     {
