@@ -297,7 +297,11 @@ async function testWithClaimToken(
     url.searchParams.set("session_id", sessionId);
     const httpResp = await httpLib.fetch(url.href, {
       headers: { Accept: "text/html" },
+      redirect: "manual",
     });
+    console.log(
+      `requesting GET ${url.href}, expected 302 got ${httpResp.status}`,
+    );
     t.assertDeepEqual(httpResp.status, 302);
     const location = httpResp.headers.get("Location");
     console.log("location header:", location);
@@ -553,6 +557,7 @@ async function testWithoutClaimToken(
     url.searchParams.set("session_id", sessionId);
     const httpResp = await httpLib.fetch(url.href, {
       headers: { Accept: "text/html" },
+      redirect: "manual",
     });
     t.assertDeepEqual(httpResp.status, 302);
     const location = httpResp.headers.get("Location");
@@ -568,9 +573,8 @@ async function testWithoutClaimToken(
  * specification of the endpoint.
  */
 export async function runMerchantSpecPublicOrdersTest(t: GlobalTestState) {
-  const { bank, exchange, merchant } = await createSimpleTestkudosEnvironmentV2(
-    t,
-  );
+  const { bank, exchange, merchant } =
+    await createSimpleTestkudosEnvironmentV2(t);
 
   // Base URL for the default instance.
   const merchantBaseUrl = merchant.makeInstanceBaseUrl();
