@@ -55,7 +55,7 @@ export async function runPaymentTemplateTest(t: GlobalTestState) {
       summary: "hello, I'm a summary",
     },
   });
-  narrowOpSuccessOrThrow(createTemplateRes);
+  narrowOpSuccessOrThrow("createTemplate", createTemplateRes);
 
   // Withdraw digital cash into the wallet.
 
