@@ -138,7 +138,7 @@ export async function runPeerRepairTest(t: GlobalTestState) {
   );
 
   await wallet2.client.call(WalletApiOperation.ConfirmPeerPushCredit, {
-    peerPushCreditId: resp2.peerPushCreditId,
+    transactionId: resp2.transactionId,
   });
 
   await peerPushCreditDone1Cond;
