@@ -157,7 +157,7 @@ async function checkNormalPeerPull(
   );
 
   await wallet2.client.call(WalletApiOperation.ConfirmPeerPullDebit, {
-    peerPullDebitId: checkResp.peerPullDebitId,
+    transactionId: checkResp.transactionId,
   });
 
   await peerPullCreditDoneCond;
@@ -257,7 +257,7 @@ async function checkAbortedPeerPull(
   });
 
   await wallet2.client.call(WalletApiOperation.ConfirmPeerPullDebit, {
-    peerPullDebitId: checkResp.peerPullDebitId,
+    transactionId: checkResp.transactionId,
   });
 
   console.log(`waiting for ${resp.transactionId} to go to state aborted`);
