@@ -76,6 +76,15 @@ export async function runPeerToPeerPushTest(t: GlobalTestState) {
     ),
   );
 
+  const checkResp0 = await w1.walletClient.call(
+    WalletApiOperation.CheckPeerPushDebit,
+    {
+      amount: "TESTKUDOS:5" as AmountString,
+    },
+  );
+
+  t.assertAmountEquals(checkResp0.amountEffective, "TESTKUDOS:5.49");
+
   {
     const resp = await w1.walletClient.call(
       WalletApiOperation.InitiatePeerPushDebit,
@@ -91,6 +100,11 @@ export async function runPeerToPeerPushTest(t: GlobalTestState) {
     console.log(resp);
   }
 
+  {
+    const bal = await w1.walletClient.call(WalletApiOperation.GetBalances, {});
+    t.assertAmountEquals(bal.balances[0].pendingOutgoing, "TESTKUDOS:5.49");
+  }
+
   await w1.walletClient.call(WalletApiOperation.TestingWaitRefreshesFinal, {});
 
   const resp = await w1.walletClient.call(
