@@ -20,15 +20,15 @@
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
 import { CoinConfig } from "../harness/denomStructures.js";
 import {
-  GlobalTestState,
+  BankService,
   ExchangeService,
+  GlobalTestState,
   MerchantService,
   WalletCli,
-  setupDb,
-  BankService,
+  WalletClient,
   delayMs,
   generateRandomPayto,
-  WalletClient,
+  setupDb,
 } from "../harness/harness.js";
 import {
   SimpleTestEnvironmentNg,
@@ -208,7 +208,7 @@ export async function runRevocationTest(t: GlobalTestState) {
   console.log(coinDump);
   const coinPubList = coinDump.coins.map((x) => x.coin_pub);
   await walletClient.call(WalletApiOperation.ForceRefresh, {
-    coinPubList,
+    refreshCoinSpecs: coinPubList.map((x) => ({ coinPub: x })),
   });
   await walletClient.call(WalletApiOperation.TestingWaitTransactionsFinal, {});
 
