@@ -35,7 +35,7 @@ import {
   downloadExchangeInfo,
   findDenomOrThrow,
   refreshCoin,
-  topupReserveWithDemobank,
+  topupReserveWithBank,
   withdrawCoin,
 } from "@gnu-taler/taler-wallet-core/dbless";
 import { GlobalTestState, harnessHttpLib } from "../harness/harness.js";
@@ -71,7 +71,7 @@ export async function runWalletDblessTest(t: GlobalTestState) {
       method: "GET",
     });
 
-    await topupReserveWithDemobank({
+    await topupReserveWithBank({
       amount: "TESTKUDOS:10" as AmountString,
       http,
       reservePub: reserveKeyPair.pub,
