@@ -90,16 +90,23 @@ import { runWalletBackupDoublespendTest } from "./test-wallet-backup-doublespend
 import { runWalletBalanceNotificationsTest } from "./test-wallet-balance-notifications.js";
 import { runWalletBalanceZeroTest } from "./test-wallet-balance-zero.js";
 import { runWalletBalanceTest } from "./test-wallet-balance.js";
+import { runWalletBlockedDepositTest } from "./test-wallet-blocked-deposit.js";
+import { runWalletBlockedPayMerchantTest } from "./test-wallet-blocked-pay-merchant.js";
+import { runWalletBlockedPayPeerPullTest } from "./test-wallet-blocked-pay-peer-pull.js";
+import { runWalletBlockedPayPeerPushTest } from "./test-wallet-blocked-pay-peer-push.js";
 import { runWalletCliTerminationTest } from "./test-wallet-cli-termination.js";
 import { runWalletConfigTest } from "./test-wallet-config.js";
 import { runWalletCryptoWorkerTest } from "./test-wallet-cryptoworker.js";
 import { runWalletDblessTest } from "./test-wallet-dbless.js";
 import { runWalletDd48Test } from "./test-wallet-dd48.js";
+import { runWalletDenomExpireTest } from "./test-wallet-denom-expire.js";
 import { runWalletDevExperimentsTest } from "./test-wallet-dev-experiments.js";
+import { runWalletExchangeUpdateTest } from "./test-wallet-exchange-update.js";
 import { runWalletGenDbTest } from "./test-wallet-gendb.js";
 import { runWalletInsufficientBalanceTest } from "./test-wallet-insufficient-balance.js";
 import { runWalletNotificationsTest } from "./test-wallet-notifications.js";
 import { runWalletObservabilityTest } from "./test-wallet-observability.js";
+import { runWalletRefreshErrorsTest } from "./test-wallet-refresh-errors.js";
 import { runWalletRefreshTest } from "./test-wallet-refresh.js";
 import { runWalletWirefeesTest } from "./test-wallet-wirefees.js";
 import { runWallettestingTest } from "./test-wallettesting.js";
@@ -210,6 +217,13 @@ const allTests: TestMainFunction[] = [
   runWalletInsufficientBalanceTest,
   runWalletWirefeesTest,
   runDenomLostTest,
+  runWalletDenomExpireTest,
+  runWalletBlockedDepositTest,
+  runWalletBlockedPayMerchantTest,
+  runWalletBlockedPayPeerPushTest,
+  runWalletBlockedPayPeerPullTest,
+  runWalletExchangeUpdateTest,
+  runWalletRefreshErrorsTest,
 ];
 
 export interface TestRunSpec {
