@@ -18,23 +18,23 @@ import { codecForAny } from "./codec.js";
 import {
   TalerMerchantApi,
   codecForMerchantConfig,
+  codecForMerchantOrderPrivateStatusResponse,
 } from "./http-client/types.js";
 import { HttpStatusCode } from "./http-status-codes.js";
 import {
   createPlatformHttpLib,
   expectSuccessResponseOrThrow,
   readSuccessResponseJsonOrThrow,
+  readTalerErrorResponse,
 } from "./http.js";
 import { FacadeCredentials } from "./libeufin-api-types.js";
 import { LibtoolVersion } from "./libtool-version.js";
 import { Logger } from "./logging.js";
 import {
   MerchantInstancesResponse,
-  MerchantOrderPrivateStatusResponse,
   MerchantPostOrderRequest,
   MerchantPostOrderResponse,
   MerchantTemplateAddDetails,
-  codecForMerchantOrderPrivateStatusResponse,
   codecForMerchantPostOrderResponse,
 } from "./merchant-api-types.js";
 import {
@@ -44,7 +44,7 @@ import {
   ResultByMethod,
   opEmptySuccess,
   opKnownHttpFailure,
-  opSuccess,
+  opSuccessFromHttp,
   opUnknownFailure,
 } from "./operation.js";
 import { AmountString } from "./taler-types.js";
@@ -258,7 +258,7 @@ export class MerchantApiClient {
 
   async queryPrivateOrderStatus(
     query: PrivateOrderStatusQuery,
-  ): Promise<MerchantOrderPrivateStatusResponse> {
+  ): Promise<TalerMerchantApi.MerchantOrderStatusResponse> {
     const reqUrl = new URL(`private/orders/${query.orderId}`, this.baseUrl);
     if (query.sessionId) {
       reqUrl.searchParams.set("session_id", query.sessionId);
@@ -306,7 +306,7 @@ export class MerchantApiClient {
       case HttpStatusCode.NotFound:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 
@@ -318,11 +318,11 @@ export class MerchantApiClient {
     });
     switch (resp.status) {
       case HttpStatusCode.Ok:
-        return opSuccess(resp, codecForAny());
+        return opSuccessFromHttp(resp, codecForAny());
       case HttpStatusCode.NotFound:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 
@@ -341,9 +341,9 @@ export class MerchantApiClient {
     });
     switch (resp.status) {
       case HttpStatusCode.Ok:
-        return opSuccess(resp, codecForMerchantConfig());
+        return opSuccessFromHttp(resp, codecForMerchantConfig());
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 
@@ -363,7 +363,7 @@ export class MerchantApiClient {
       case HttpStatusCode.NotFound:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 
