@@ -22,10 +22,7 @@
  * Imports.
  */
 import { codecForAmountString } from "./amounts.js";
-import {
-  Codec,
-  buildCodecForObject
-} from "./codec.js";
+import { Codec, buildCodecForObject } from "./codec.js";
 import { AmountString } from "./taler-types.js";
 
 /**
