@@ -45,6 +45,7 @@ import {
   createPlatformHttpLib,
   HttpRequestLibrary,
   readSuccessResponseJsonOrThrow,
+  readTalerErrorResponse,
 } from "@gnu-taler/taler-util/http";
 
 const logger = new Logger("bank-api-client.ts");
@@ -420,7 +421,7 @@ export class TalerCorebankApiClient {
       case HttpStatusCode.NotFound:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 
