@@ -25,6 +25,7 @@
  */
 import {
   AbsoluteTime,
+  CancellationToken,
   PaymentInsufficientBalanceDetails,
   TalerErrorCode,
   TalerErrorDetail,
@@ -93,6 +94,11 @@ export interface DetailsMap {
     requestMethod: string;
     timeoutMs: number;
   };
+  [TalerErrorCode.GENERIC_TIMEOUT]: {
+    requestUrl: string;
+    requestMethod: string;
+    timeoutMs: number;
+  };
   [TalerErrorCode.WALLET_NETWORK_ERROR]: {
     requestUrl: string;
     requestMethod: string;
@@ -108,6 +114,11 @@ export interface DetailsMap {
      */
     contentType?: string;
   };
+  [TalerErrorCode.GENERIC_CLIENT_INTERNAL_ERROR]: {
+    operation: string;
+    error: string;
+    detail: TalerErrorDetail | undefined;
+  };
   [TalerErrorCode.WALLET_EXCHANGE_COIN_SIGNATURE_INVALID]: empty;
   [TalerErrorCode.WALLET_WITHDRAWAL_GROUP_INCOMPLETE]: {
     numErrors: number;
@@ -275,6 +286,13 @@ export function getErrorDetailFromException(e: any): TalerErrorDetail {
   if (e instanceof TalerError) {
     return e.errorDetail;
   }
+  if (e instanceof CancellationToken.CancellationError) {
+    const err = makeErrorDetail(
+      TalerErrorCode.WALLET_CORE_REQUEST_CANCELLED,
+      {},
+    );
+    return err;
+  }
   if (e instanceof Error) {
     const err = makeErrorDetail(
       TalerErrorCode.WALLET_UNEXPECTED_EXCEPTION,
