@@ -22,18 +22,20 @@ import {
   HttpRequestLibrary,
   createPlatformHttpLib,
   makeBasicAuthHeader,
+  readTalerErrorResponse,
 } from "../http.js";
 import { LibtoolVersion } from "../libtool-version.js";
 import {
   opEmptySuccess,
   opKnownHttpFailure,
-  opSuccess,
+  opSuccessFromHttp,
   opUnknownFailure,
 } from "../operation.js";
 import {
   AccessToken,
   TalerAuthentication,
   codecForTokenSuccessResponse,
+  codecForTokenSuccessResponseMerchant,
 } from "./types.js";
 import { makeBearerTokenAuthHeader } from "./utils.js";
 
@@ -44,7 +46,6 @@ export class TalerAuthenticationHttpClient {
 
   constructor(
     readonly baseUrl: string,
-    readonly username: string,
     httpClient?: HttpRequestLibrary,
   ) {
     this.httpLib = httpClient ?? createPlatformHttpLib();
@@ -60,7 +61,8 @@ export class TalerAuthenticationHttpClient {
    *
    * @returns
    */
-  async createAccessToken(
+  async createAccessTokenBasic(
+    username: string,
     password: string,
     body: TalerAuthentication.TokenRequest,
   ) {
@@ -68,20 +70,49 @@ export class TalerAuthenticationHttpClient {
     const resp = await this.httpLib.fetch(url.href, {
       method: "POST",
       headers: {
-        Authorization: makeBasicAuthHeader(this.username, password),
+        Authorization: makeBasicAuthHeader(username, password),
       },
       body,
     });
     switch (resp.status) {
       case HttpStatusCode.Ok:
-        return opSuccess(resp, codecForTokenSuccessResponse());
+        return opSuccessFromHttp(resp, codecForTokenSuccessResponse());
       //FIXME: missing in docs
       case HttpStatusCode.Unauthorized:
         return opKnownHttpFailure(resp.status, resp);
       case HttpStatusCode.NotFound:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
+    }
+  }
+
+  /**
+   *
+   * @returns
+   */
+  async createAccessTokenBearer(
+    token: string,
+    body: TalerAuthentication.TokenRequest,
+  ) {
+    const url = new URL(`token`, this.baseUrl);
+    const resp = await this.httpLib.fetch(url.href, {
+      method: "POST",
+      headers: {
+        Authorization: makeBearerTokenAuthHeader(token as AccessToken),
+      },
+      body,
+    });
+    switch (resp.status) {
+      case HttpStatusCode.Ok:
+        return opSuccessFromHttp(resp, codecForTokenSuccessResponseMerchant());
+      //FIXME: missing in docs
+      case HttpStatusCode.Unauthorized:
+        return opKnownHttpFailure(resp.status, resp);
+      case HttpStatusCode.NotFound:
+        return opKnownHttpFailure(resp.status, resp);
+      default:
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 
@@ -100,7 +131,7 @@ export class TalerAuthenticationHttpClient {
       case HttpStatusCode.NotFound:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 }
