@@ -18,7 +18,7 @@
  * Imports.
  */
 import { AmountJson, Amounts } from "../amounts.js";
-import { HttpRequestLibrary } from "../http-common.js";
+import { HttpRequestLibrary, readTalerErrorResponse } from "../http-common.js";
 import { HttpStatusCode } from "../http-status-codes.js";
 import { createPlatformHttpLib } from "../http.js";
 import { LibtoolVersion } from "../libtool-version.js";
@@ -27,7 +27,7 @@ import {
   ResultByMethod,
   opEmptySuccess,
   opKnownHttpFailure,
-  opSuccess,
+  opSuccessFromHttp,
   opUnknownFailure,
 } from "../operation.js";
 import { TalerErrorCode } from "../taler-error-codes.js";
@@ -39,7 +39,11 @@ import {
   codecForCashoutConversionResponse,
   codecForConversionBankConfig,
 } from "./types.js";
-import { makeBearerTokenAuthHeader } from "./utils.js";
+import {
+  CacheEvictor,
+  makeBearerTokenAuthHeader,
+  nullEvictor,
+} from "./utils.js";
 
 export type TalerBankConversionResultByMethod<
   prop extends keyof TalerBankConversionHttpClient,
@@ -48,6 +52,10 @@ export type TalerBankConversionErrorsByMethod<
   prop extends keyof TalerBankConversionHttpClient,
 > = FailCasesByMethod<TalerBankConversionHttpClient, prop>;
 
+export enum TalerBankConversionCacheEviction {
+  UPDATE_RATE,
+}
+
 /**
  * The API is used by the wallets.
  */
@@ -55,12 +63,15 @@ export class TalerBankConversionHttpClient {
   public readonly PROTOCOL_VERSION = "0:0:0";
 
   httpLib: HttpRequestLibrary;
+  cacheEvictor: CacheEvictor<TalerBankConversionCacheEviction>;
 
   constructor(
     readonly baseUrl: string,
     httpClient?: HttpRequestLibrary,
+    cacheEvictor?: CacheEvictor<TalerBankConversionCacheEviction>,
   ) {
     this.httpLib = httpClient ?? createPlatformHttpLib();
+    this.cacheEvictor = cacheEvictor ?? nullEvictor;
   }
 
   isCompatible(version: string): boolean {
@@ -79,11 +90,11 @@ export class TalerBankConversionHttpClient {
     });
     switch (resp.status) {
       case HttpStatusCode.Ok:
-        return opSuccess(resp, codecForConversionBankConfig());
+        return opSuccessFromHttp(resp, codecForConversionBankConfig());
       case HttpStatusCode.NotImplemented:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 
@@ -107,7 +118,7 @@ export class TalerBankConversionHttpClient {
     });
     switch (resp.status) {
       case HttpStatusCode.Ok:
-        return opSuccess(resp, codecForCashinConversionResponse());
+        return opSuccessFromHttp(resp, codecForCashinConversionResponse());
       case HttpStatusCode.BadRequest: {
         const body = await resp.json();
         const details = codecForTalerErrorDetail().decode(body);
@@ -127,7 +138,7 @@ export class TalerBankConversionHttpClient {
       case HttpStatusCode.NotImplemented:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 
@@ -154,7 +165,7 @@ export class TalerBankConversionHttpClient {
     });
     switch (resp.status) {
       case HttpStatusCode.Ok:
-        return opSuccess(resp, codecForCashoutConversionResponse());
+        return opSuccessFromHttp(resp, codecForCashoutConversionResponse());
       case HttpStatusCode.BadRequest: {
         const body = await resp.json();
         const details = codecForTalerErrorDetail().decode(body);
@@ -174,7 +185,7 @@ export class TalerBankConversionHttpClient {
       case HttpStatusCode.NotImplemented:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 
@@ -195,14 +206,18 @@ export class TalerBankConversionHttpClient {
       body,
     });
     switch (resp.status) {
-      case HttpStatusCode.NoContent:
+      case HttpStatusCode.NoContent: {
+        this.cacheEvictor.notifySuccess(
+          TalerBankConversionCacheEviction.UPDATE_RATE,
+        );
         return opEmptySuccess(resp);
+      }
       case HttpStatusCode.Unauthorized:
         return opKnownHttpFailure(resp.status, resp);
       case HttpStatusCode.NotImplemented:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 }
