@@ -15,9 +15,13 @@
  */
 
 import {
+  AbsoluteTime,
   HttpStatusCode,
   LibtoolVersion,
   LongPollParams,
+  OperationAlternative,
+  OperationFail,
+  OperationOk,
   TalerErrorCode,
   codecForChallenge,
   codecForTalerErrorDetail,
@@ -29,20 +33,16 @@ import {
 import {
   HttpRequestLibrary,
   createPlatformHttpLib,
+  readTalerErrorResponse,
 } from "@gnu-taler/taler-util/http";
 import {
   FailCasesByMethod,
   ResultByMethod,
   opEmptySuccess,
   opFixedSuccess,
-  opSuccess,
+  opSuccessFromHttp,
   opUnknownFailure,
 } from "../operation.js";
-import { TalerAuthenticationHttpClient } from "./authentication.js";
-import { TalerBankConversionHttpClient } from "./bank-conversion.js";
-import { TalerBankIntegrationHttpClient } from "./bank-integration.js";
-import { TalerRevenueHttpClient } from "./bank-revenue.js";
-import { TalerWireGatewayHttpClient } from "./bank-wire.js";
 import {
   AccessToken,
   PaginationParams,
@@ -66,9 +66,12 @@ import {
   codecForWithdrawalPublicInfo,
 } from "./types.js";
 import {
+  CacheEvictor,
+  IdempotencyRetry,
   addLongPollingParam,
   addPaginationParams,
   makeBearerTokenAuthHeader,
+  nullEvictor,
 } from "./utils.js";
 
 export type TalerCoreBankResultByMethod<
@@ -78,6 +81,17 @@ export type TalerCoreBankErrorsByMethod<
   prop extends keyof TalerCoreBankHttpClient,
 > = FailCasesByMethod<TalerCoreBankHttpClient, prop>;
 
+export enum TalerCoreBankCacheEviction {
+  DELETE_ACCOUNT,
+  CREATE_ACCOUNT,
+  UPDATE_ACCOUNT,
+  UPDATE_PASSWORD,
+  CREATE_TRANSACTION,
+  CONFIRM_WITHDRAWAL,
+  ABORT_WITHDRAWAL,
+  CREATE_WITHDRAWAL,
+  CREATE_CASHOUT,
+}
 /**
  * Protocol version spoken with the core bank.
  *
@@ -91,12 +105,14 @@ export class TalerCoreBankHttpClient {
   public readonly PROTOCOL_VERSION = "4:0:0";
 
   httpLib: HttpRequestLibrary;
-
+  cacheEvictor: CacheEvictor<TalerCoreBankCacheEviction>;
   constructor(
     readonly baseUrl: string,
     httpClient?: HttpRequestLibrary,
+    cacheEvictor?: CacheEvictor<TalerCoreBankCacheEviction>,
   ) {
     this.httpLib = httpClient ?? createPlatformHttpLib();
+    this.cacheEvictor = cacheEvictor ?? nullEvictor;
   }
 
   isCompatible(version: string): boolean {
@@ -115,9 +131,11 @@ export class TalerCoreBankHttpClient {
     });
     switch (resp.status) {
       case HttpStatusCode.Ok:
-        return opSuccess(resp, codecForCoreBankConfig());
+        return opSuccessFromHttp(resp, codecForCoreBankConfig());
+      case HttpStatusCode.NotFound:
+        return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 
@@ -130,50 +148,55 @@ export class TalerCoreBankHttpClient {
    *
    */
   async createAccount(
-    auth: AccessToken,
+    auth: AccessToken | undefined,
     body: TalerCorebankApi.RegisterAccountRequest,
   ) {
     const url = new URL(`accounts`, this.baseUrl);
+    const headers: Record<string, string> = {};
+    if (auth) {
+      headers.Authorization = makeBearerTokenAuthHeader(auth);
+    }
     const resp = await this.httpLib.fetch(url.href, {
       method: "POST",
       body,
-      headers: {
-        Authorization: makeBearerTokenAuthHeader(auth),
-      },
+      headers: headers,
     });
     switch (resp.status) {
-      case HttpStatusCode.Ok:
-        return opSuccess(resp, codecForRegisterAccountResponse());
+      case HttpStatusCode.Ok: {
+        await this.cacheEvictor.notifySuccess(
+          TalerCoreBankCacheEviction.CREATE_ACCOUNT,
+        );
+        return opSuccessFromHttp(resp, codecForRegisterAccountResponse());
+      }
       case HttpStatusCode.BadRequest:
         return opKnownHttpFailure(resp.status, resp);
       case HttpStatusCode.Unauthorized:
         return opKnownHttpFailure(resp.status, resp);
       case HttpStatusCode.Conflict: {
-        const body = await resp.json();
-        const details = codecForTalerErrorDetail().decode(body);
+        const details = await readTalerErrorResponse(resp);
         switch (details.code) {
           case TalerErrorCode.BANK_REGISTER_USERNAME_REUSE:
-            return opKnownTalerFailure(details.code, resp);
+            return opKnownTalerFailure(details.code, details);
           case TalerErrorCode.BANK_REGISTER_PAYTO_URI_REUSE:
-            return opKnownTalerFailure(details.code, resp);
+            return opKnownTalerFailure(details.code, details);
           case TalerErrorCode.BANK_UNALLOWED_DEBIT:
-            return opKnownTalerFailure(details.code, resp);
+            return opKnownTalerFailure(details.code, details);
           case TalerErrorCode.BANK_RESERVED_USERNAME_CONFLICT:
-            return opKnownTalerFailure(details.code, resp);
+            return opKnownTalerFailure(details.code, details);
           case TalerErrorCode.BANK_NON_ADMIN_PATCH_DEBT_LIMIT:
-            return opKnownTalerFailure(details.code, resp);
+            return opKnownTalerFailure(details.code, details);
           case TalerErrorCode.BANK_NON_ADMIN_SET_TAN_CHANNEL:
-            return opKnownTalerFailure(details.code, resp);
+            return opKnownTalerFailure(details.code, details);
           case TalerErrorCode.BANK_TAN_CHANNEL_NOT_SUPPORTED:
-            return opKnownTalerFailure(details.code, resp);
+            return opKnownTalerFailure(details.code, details);
           case TalerErrorCode.BANK_MISSING_TAN_INFO:
-            return opKnownTalerFailure(details.code, resp);
+            return opKnownTalerFailure(details.code, details);
           default:
-            return opUnknownFailure(resp, body);
+            return opUnknownFailure(resp, details);
         }
       }
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
   /**
@@ -203,19 +226,18 @@ export class TalerCoreBankHttpClient {
       case HttpStatusCode.NotFound:
         return opKnownHttpFailure(resp.status, resp);
       case HttpStatusCode.Conflict: {
-        const body = await resp.json();
-        const details = codecForTalerErrorDetail().decode(body);
+        const details = await readTalerErrorResponse(resp);
         switch (details.code) {
           case TalerErrorCode.BANK_RESERVED_USERNAME_CONFLICT:
-            return opKnownTalerFailure(details.code, resp);
+            return opKnownTalerFailure(details.code, details);
           case TalerErrorCode.BANK_ACCOUNT_BALANCE_NOT_ZERO:
-            return opKnownTalerFailure(details.code, resp);
+            return opKnownTalerFailure(details.code, details);
           default:
-            return opUnknownFailure(resp, body);
+            return opUnknownFailure(resp, details);
         }
       }
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 
@@ -251,25 +273,24 @@ export class TalerCoreBankHttpClient {
       case HttpStatusCode.NotFound:
         return opKnownHttpFailure(resp.status, resp);
       case HttpStatusCode.Conflict: {
-        const body = await resp.json();
-        const details = codecForTalerErrorDetail().decode(body);
+        const details = await readTalerErrorResponse(resp);
         switch (details.code) {
           case TalerErrorCode.BANK_NON_ADMIN_PATCH_LEGAL_NAME:
-            return opKnownTalerFailure(details.code, resp);
+            return opKnownTalerFailure(details.code, details);
           case TalerErrorCode.BANK_NON_ADMIN_PATCH_CASHOUT:
-            return opKnownTalerFailure(details.code, resp);
+            return opKnownTalerFailure(details.code, details);
           case TalerErrorCode.BANK_NON_ADMIN_PATCH_DEBT_LIMIT:
-            return opKnownTalerFailure(details.code, resp);
+            return opKnownTalerFailure(details.code, details);
           case TalerErrorCode.BANK_TAN_CHANNEL_NOT_SUPPORTED:
-            return opKnownTalerFailure(details.code, resp);
+            return opKnownTalerFailure(details.code, details);
           case TalerErrorCode.BANK_MISSING_TAN_INFO:
-            return opKnownTalerFailure(details.code, resp);
+            return opKnownTalerFailure(details.code, details);
           default:
-            return opUnknownFailure(resp, body);
+            return opUnknownFailure(resp, details);
         }
       }
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 
@@ -305,19 +326,18 @@ export class TalerCoreBankHttpClient {
       case HttpStatusCode.Unauthorized:
         return opKnownHttpFailure(resp.status, resp);
       case HttpStatusCode.Conflict: {
-        const body = await resp.json();
-        const details = codecForTalerErrorDetail().decode(body);
+        const details = await readTalerErrorResponse(resp);
         switch (details.code) {
           case TalerErrorCode.BANK_NON_ADMIN_PATCH_MISSING_OLD_PASSWORD:
-            return opKnownTalerFailure(details.code, resp);
+            return opKnownTalerFailure(details.code, details);
           case TalerErrorCode.BANK_PATCH_BAD_OLD_PASSWORD:
-            return opKnownTalerFailure(details.code, resp);
+            return opKnownTalerFailure(details.code, details);
           default:
-            return opUnknownFailure(resp, body);
+            return opUnknownFailure(resp, details);
         }
       }
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 
@@ -339,13 +359,13 @@ export class TalerCoreBankHttpClient {
     });
     switch (resp.status) {
       case HttpStatusCode.Ok:
-        return opSuccess(resp, codecForPublicAccountsResponse());
+        return opSuccessFromHttp(resp, codecForPublicAccountsResponse());
       case HttpStatusCode.NoContent:
-        return opFixedSuccess(resp, { public_accounts: [] });
+        return opFixedSuccess({ public_accounts: [] });
       case HttpStatusCode.NotFound:
-        return opFixedSuccess(resp, { public_accounts: [] });
+        return opFixedSuccess({ public_accounts: [] });
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 
@@ -371,13 +391,13 @@ export class TalerCoreBankHttpClient {
     });
     switch (resp.status) {
       case HttpStatusCode.Ok:
-        return opSuccess(resp, codecForListBankAccountsResponse());
+        return opSuccessFromHttp(resp, codecForListBankAccountsResponse());
       case HttpStatusCode.NoContent:
-        return opFixedSuccess(resp, { accounts: [] });
+        return opFixedSuccess({ accounts: [] });
       case HttpStatusCode.Unauthorized:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 
@@ -395,13 +415,13 @@ export class TalerCoreBankHttpClient {
     });
     switch (resp.status) {
       case HttpStatusCode.Ok:
-        return opSuccess(resp, codecForAccountData());
+        return opSuccessFromHttp(resp, codecForAccountData());
       case HttpStatusCode.Unauthorized:
         return opKnownHttpFailure(resp.status, resp);
       case HttpStatusCode.NotFound:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 
@@ -428,15 +448,18 @@ export class TalerCoreBankHttpClient {
     });
     switch (resp.status) {
       case HttpStatusCode.Ok:
-        return opSuccess(resp, codecForBankAccountTransactionsResponse());
+        return opSuccessFromHttp(
+          resp,
+          codecForBankAccountTransactionsResponse(),
+        );
       case HttpStatusCode.NoContent:
-        return opFixedSuccess(resp, { transactions: [] });
+        return opFixedSuccess({ transactions: [] });
       case HttpStatusCode.Unauthorized:
         return opKnownHttpFailure(resp.status, resp);
       case HttpStatusCode.NotFound:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 
@@ -457,13 +480,13 @@ export class TalerCoreBankHttpClient {
     });
     switch (resp.status) {
       case HttpStatusCode.Ok:
-        return opSuccess(resp, codecForBankAccountTransactionInfo());
+        return opSuccessFromHttp(resp, codecForBankAccountTransactionInfo());
       case HttpStatusCode.NotFound:
         return opKnownHttpFailure(resp.status, resp);
       case HttpStatusCode.Unauthorized:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 
@@ -474,9 +497,25 @@ export class TalerCoreBankHttpClient {
   async createTransaction(
     auth: UserAndToken,
     body: TalerCorebankApi.CreateTransactionRequest,
+    idempotencyCheck: IdempotencyRetry | undefined,
     cid?: string,
-  ) {
+  ): Promise<
+    //manually definition all return types because of recursion
+    | OperationOk<TalerCorebankApi.CreateTransactionResponse>
+    | OperationAlternative<HttpStatusCode.Accepted, TalerCorebankApi.Challenge>
+    | OperationFail<HttpStatusCode.NotFound>
+    | OperationFail<HttpStatusCode.BadRequest>
+    | OperationFail<HttpStatusCode.Unauthorized>
+    | OperationFail<TalerErrorCode.BANK_UNALLOWED_DEBIT>
+    | OperationFail<TalerErrorCode.BANK_ADMIN_CREDITOR>
+    | OperationFail<TalerErrorCode.BANK_SAME_ACCOUNT>
+    | OperationFail<TalerErrorCode.BANK_UNKNOWN_CREDITOR>
+    | OperationFail<TalerErrorCode.BANK_TRANSFER_REQUEST_UID_REUSED>
+  > {
     const url = new URL(`accounts/${auth.username}/transactions`, this.baseUrl);
+    if (idempotencyCheck) {
+      body.request_uid = idempotencyCheck.uid
+    }
     const resp = await this.httpLib.fetch(url.href, {
       method: "POST",
       headers: {
@@ -487,7 +526,7 @@ export class TalerCoreBankHttpClient {
     });
     switch (resp.status) {
       case HttpStatusCode.Ok:
-        return opSuccess(resp, codecForCreateTransactionResponse());
+        return opSuccessFromHttp(resp, codecForCreateTransactionResponse());
       case HttpStatusCode.Accepted:
         return opKnownAlternativeFailure(
           resp,
@@ -501,21 +540,28 @@ export class TalerCoreBankHttpClient {
       case HttpStatusCode.NotFound:
         return opKnownHttpFailure(resp.status, resp);
       case HttpStatusCode.Conflict: {
-        const body = await resp.json();
-        const details = codecForTalerErrorDetail().decode(body);
+        const details = await readTalerErrorResponse(resp);
         switch (details.code) {
+          case TalerErrorCode.BANK_ADMIN_CREDITOR:
+            return opKnownTalerFailure(details.code, details);
           case TalerErrorCode.BANK_SAME_ACCOUNT:
-            return opKnownTalerFailure(details.code, resp);
+            return opKnownTalerFailure(details.code, details);
           case TalerErrorCode.BANK_UNKNOWN_CREDITOR:
-            return opKnownTalerFailure(details.code, resp);
+            return opKnownTalerFailure(details.code, details);
           case TalerErrorCode.BANK_UNALLOWED_DEBIT:
-            return opKnownTalerFailure(details.code, resp);
+            return opKnownTalerFailure(details.code, details);
+          case TalerErrorCode.BANK_TRANSFER_REQUEST_UID_REUSED:
+            if (!idempotencyCheck) {
+              return opKnownTalerFailure(details.code, details);
+            }
+            const nextRetry = idempotencyCheck.next();
+            return this.createTransaction(auth, body, nextRetry, cid);
           default:
-            return opUnknownFailure(resp, body);
+            return opUnknownFailure(resp, details);
         }
       }
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 
@@ -541,7 +587,10 @@ export class TalerCoreBankHttpClient {
     });
     switch (resp.status) {
       case HttpStatusCode.Ok:
-        return opSuccess(resp, codecForBankAccountCreateWithdrawalResponse());
+        return opSuccessFromHttp(
+          resp,
+          codecForBankAccountCreateWithdrawalResponse(),
+        );
       case HttpStatusCode.NotFound:
         return opKnownHttpFailure(resp.status, resp);
       case HttpStatusCode.Conflict:
@@ -550,7 +599,7 @@ export class TalerCoreBankHttpClient {
       case HttpStatusCode.Unauthorized:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 
@@ -585,21 +634,20 @@ export class TalerCoreBankHttpClient {
       case HttpStatusCode.NotFound:
         return opKnownHttpFailure(resp.status, resp);
       case HttpStatusCode.Conflict: {
-        const body = await resp.json();
-        const details = codecForTalerErrorDetail().decode(body);
+        const details = await readTalerErrorResponse(resp);
         switch (details.code) {
           case TalerErrorCode.BANK_CONFIRM_ABORT_CONFLICT:
-            return opKnownTalerFailure(details.code, resp);
+            return opKnownTalerFailure(details.code, details);
           case TalerErrorCode.BANK_CONFIRM_INCOMPLETE:
-            return opKnownTalerFailure(details.code, resp);
+            return opKnownTalerFailure(details.code, details);
           case TalerErrorCode.BANK_UNALLOWED_DEBIT:
-            return opKnownTalerFailure(details.code, resp);
+            return opKnownTalerFailure(details.code, details);
           default:
-            return opUnknownFailure(resp, body);
+            return opUnknownFailure(resp, details);
         }
       }
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 
@@ -629,7 +677,7 @@ export class TalerCoreBankHttpClient {
       case HttpStatusCode.Conflict:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 
@@ -656,14 +704,14 @@ export class TalerCoreBankHttpClient {
     });
     switch (resp.status) {
       case HttpStatusCode.Ok:
-        return opSuccess(resp, codecForWithdrawalPublicInfo());
+        return opSuccessFromHttp(resp, codecForWithdrawalPublicInfo());
       //FIXME: missing in docs
       case HttpStatusCode.BadRequest:
         return opKnownHttpFailure(resp.status, resp);
       case HttpStatusCode.NotFound:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 
@@ -691,7 +739,7 @@ export class TalerCoreBankHttpClient {
     });
     switch (resp.status) {
       case HttpStatusCode.Ok:
-        return opSuccess(resp, codecForCashoutPending());
+        return opSuccessFromHttp(resp, codecForCashoutPending());
       case HttpStatusCode.Accepted:
         return opKnownAlternativeFailure(
           resp,
@@ -701,35 +749,33 @@ export class TalerCoreBankHttpClient {
       case HttpStatusCode.NotFound:
         return opKnownHttpFailure(resp.status, resp);
       case HttpStatusCode.Conflict: {
-        const body = await resp.json();
-        const details = codecForTalerErrorDetail().decode(body);
+        const details = await readTalerErrorResponse(resp);
         switch (details.code) {
           case TalerErrorCode.BANK_TRANSFER_REQUEST_UID_REUSED:
-            return opKnownTalerFailure(details.code, resp);
+            return opKnownTalerFailure(details.code, details);
           case TalerErrorCode.BANK_BAD_CONVERSION:
-            return opKnownTalerFailure(details.code, resp);
+            return opKnownTalerFailure(details.code, details);
           case TalerErrorCode.BANK_UNALLOWED_DEBIT:
-            return opKnownTalerFailure(details.code, resp);
+            return opKnownTalerFailure(details.code, details);
           case TalerErrorCode.BANK_CONFIRM_INCOMPLETE:
-            return opKnownTalerFailure(details.code, resp);
+            return opKnownTalerFailure(details.code, details);
           default:
-            return opUnknownFailure(resp, body);
+            return opUnknownFailure(resp, details);
         }
       }
       case HttpStatusCode.BadGateway: {
-        const body = await resp.json();
-        const details = codecForTalerErrorDetail().decode(body);
+        const details = await readTalerErrorResponse(resp);
         switch (details.code) {
           case TalerErrorCode.BANK_TAN_CHANNEL_SCRIPT_FAILED:
-            return opKnownTalerFailure(details.code, resp);
+            return opKnownTalerFailure(details.code, details);
           default:
-            return opUnknownFailure(resp, body);
+            return opUnknownFailure(resp, details);
         }
       }
       case HttpStatusCode.NotImplemented:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 
@@ -750,13 +796,13 @@ export class TalerCoreBankHttpClient {
     });
     switch (resp.status) {
       case HttpStatusCode.Ok:
-        return opSuccess(resp, codecForCashoutStatusResponse());
+        return opSuccessFromHttp(resp, codecForCashoutStatusResponse());
       case HttpStatusCode.NotFound:
         return opKnownHttpFailure(resp.status, resp);
       case HttpStatusCode.NotImplemented:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 
@@ -775,13 +821,13 @@ export class TalerCoreBankHttpClient {
     });
     switch (resp.status) {
       case HttpStatusCode.Ok:
-        return opSuccess(resp, codecForCashouts());
+        return opSuccessFromHttp(resp, codecForCashouts());
       case HttpStatusCode.NoContent:
-        return opFixedSuccess(resp, { cashouts: [] });
+        return opFixedSuccess({ cashouts: [] });
       case HttpStatusCode.NotImplemented:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 
@@ -800,13 +846,13 @@ export class TalerCoreBankHttpClient {
     });
     switch (resp.status) {
       case HttpStatusCode.Ok:
-        return opSuccess(resp, codecForGlobalCashouts());
+        return opSuccessFromHttp(resp, codecForGlobalCashouts());
       case HttpStatusCode.NoContent:
-        return opFixedSuccess(resp, { cashouts: [] });
+        return opFixedSuccess({ cashouts: [] });
       case HttpStatusCode.NotImplemented:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 
@@ -831,23 +877,22 @@ export class TalerCoreBankHttpClient {
     });
     switch (resp.status) {
       case HttpStatusCode.Ok:
-        return opSuccess(resp, codecForTanTransmission());
+        return opSuccessFromHttp(resp, codecForTanTransmission());
       case HttpStatusCode.Unauthorized:
         return opKnownHttpFailure(resp.status, resp);
       case HttpStatusCode.NotFound:
         return opKnownHttpFailure(resp.status, resp);
       case HttpStatusCode.BadGateway: {
-        const body = await resp.json();
-        const details = codecForTalerErrorDetail().decode(body);
+        const details = await readTalerErrorResponse(resp);
         switch (details.code) {
           case TalerErrorCode.BANK_TAN_CHANNEL_SCRIPT_FAILED:
-            return opKnownTalerFailure(details.code, resp);
+            return opKnownTalerFailure(details.code, details);
           default:
-            return opUnknownFailure(resp, body);
+            return opUnknownFailure(resp, details);
         }
       }
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 
@@ -879,21 +924,20 @@ export class TalerCoreBankHttpClient {
       case HttpStatusCode.NotFound:
         return opKnownHttpFailure(resp.status, resp);
       case HttpStatusCode.Conflict: {
-        const body = await resp.json();
-        const details = codecForTalerErrorDetail().decode(body);
+        const details = await readTalerErrorResponse(resp);
         switch (details.code) {
           case TalerErrorCode.BANK_TAN_CHALLENGE_EXPIRED:
-            return opKnownTalerFailure(details.code, resp);
+            return opKnownTalerFailure(details.code, details);
           case TalerErrorCode.BANK_TAN_CHALLENGE_FAILED:
-            return opKnownTalerFailure(details.code, resp);
+            return opKnownTalerFailure(details.code, details);
           default:
-            return opUnknownFailure(resp, body);
+            return opUnknownFailure(resp, details);
         }
       }
       case HttpStatusCode.TooManyRequests:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 
@@ -909,7 +953,7 @@ export class TalerCoreBankHttpClient {
     auth: AccessToken,
     params: {
       timeframe?: TalerCorebankApi.MonitorTimeframeParam;
-      which?: number;
+      date?: AbsoluteTime;
     } = {},
   ) {
     const url = new URL(`monitor`, this.baseUrl);
@@ -919,8 +963,11 @@ export class TalerCoreBankHttpClient {
         TalerCorebankApi.MonitorTimeframeParam[params.timeframe],
       );
     }
-    if (params.which) {
-      url.searchParams.set("which", String(params.which));
+    if (params.date) {
+      const { t_s: seconds } = AbsoluteTime.toProtocolTimestamp(params.date);
+      if (seconds !== "never") {
+        url.searchParams.set("date_s", String(seconds));
+      }
     }
     const resp = await this.httpLib.fetch(url.href, {
       method: "GET",
@@ -930,13 +977,13 @@ export class TalerCoreBankHttpClient {
     });
     switch (resp.status) {
       case HttpStatusCode.Ok:
-        return opSuccess(resp, codecForMonitorResponse());
+        return opSuccessFromHttp(resp, codecForMonitorResponse());
       case HttpStatusCode.BadRequest:
         return opKnownHttpFailure(resp.status, resp);
       case HttpStatusCode.Unauthorized:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 
@@ -948,47 +995,39 @@ export class TalerCoreBankHttpClient {
    * https://docs.taler.net/core/api-corebank.html#taler-bank-integration-api
    *
    */
-  getIntegrationAPI(): TalerBankIntegrationHttpClient {
-    const url = new URL(`taler-integration/`, this.baseUrl);
-    return new TalerBankIntegrationHttpClient(url.href, this.httpLib);
+  getIntegrationAPI(): URL {
+    return new URL(`taler-integration/`, this.baseUrl);
   }
 
   /**
    * https://docs.taler.net/core/api-corebank.html#taler-bank-integration-api
    *
    */
-  getWireGatewayAPI(username: string): TalerWireGatewayHttpClient {
-    const url = new URL(
-      `accounts/${username}/taler-wire-gateway/`,
-      this.baseUrl,
-    );
-    return new TalerWireGatewayHttpClient(url.href, username, this.httpLib);
+  getWireGatewayAPI(username: string): URL {
+    return new URL(`accounts/${username}/taler-wire-gateway/`, this.baseUrl);
   }
 
   /**
    * https://docs.taler.net/core/api-corebank.html#taler-bank-integration-api
    *
    */
-  getRevenueAPI(username: string): TalerRevenueHttpClient {
-    const url = new URL(`accounts/${username}/taler-revenue/`, this.baseUrl);
-    return new TalerRevenueHttpClient(url.href, username, this.httpLib);
+  getRevenueAPI(username: string): URL {
+    return new URL(`accounts/${username}/taler-revenue/`, this.baseUrl);
   }
 
   /**
    * https://docs.taler.net/core/api-corebank.html#post--accounts-$USERNAME-token
    *
    */
-  getAuthenticationAPI(username: string): TalerAuthenticationHttpClient {
-    const url = new URL(`accounts/${username}/`, this.baseUrl);
-    return new TalerAuthenticationHttpClient(url.href, username, this.httpLib);
+  getAuthenticationAPI(username: string): URL {
+    return new URL(`accounts/${username}/`, this.baseUrl);
   }
 
   /**
    * https://docs.taler.net/core/api-corebank.html#post--accounts-$USERNAME-token
    *
    */
-  getConversionInfoAPI(): TalerBankConversionHttpClient {
-    const url = new URL(`conversion-info/`, this.baseUrl);
-    return new TalerBankConversionHttpClient(url.href, this.httpLib);
+  getConversionInfoAPI(): URL {
+    return new URL(`conversion-info/`, this.baseUrl);
   }
 }
