@@ -1,4 +1,4 @@
-import { HttpRequestLibrary } from "../http-common.js";
+import { HttpRequestLibrary, readTalerErrorResponse } from "../http-common.js";
 import { HttpStatusCode } from "../http-status-codes.js";
 import { createPlatformHttpLib } from "../http.js";
 import { LibtoolVersion } from "../libtool-version.js";
@@ -9,7 +9,7 @@ import {
   opEmptySuccess,
   opFixedSuccess,
   opKnownHttpFailure,
-  opSuccess,
+  opSuccessFromHttp,
   opUnknownFailure,
 } from "../operation.js";
 import {
@@ -70,9 +70,11 @@ export class TalerExchangeHttpClient {
     });
     switch (resp.status) {
       case HttpStatusCode.Ok:
-        return opSuccess(resp, codecForExchangeConfig());
+        return opSuccessFromHttp(resp, codecForExchangeConfig());
+      case HttpStatusCode.NotFound:
+        return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
   /**
@@ -87,9 +89,9 @@ export class TalerExchangeHttpClient {
     });
     switch (resp.status) {
       case HttpStatusCode.Ok:
-        return opSuccess(resp, codecForExchangeKeys());
+        return opSuccessFromHttp(resp, codecForExchangeKeys());
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 
@@ -123,9 +125,9 @@ export class TalerExchangeHttpClient {
 
     switch (resp.status) {
       case HttpStatusCode.Ok:
-        return opSuccess(resp, codecForAmlRecords());
+        return opSuccessFromHttp(resp, codecForAmlRecords());
       case HttpStatusCode.NoContent:
-        return opFixedSuccess(resp, { records: [] });
+        return opFixedSuccess({ records: [] });
       //this should be unauthorized
       case HttpStatusCode.Forbidden:
         return opKnownHttpFailure(resp.status, resp);
@@ -136,7 +138,7 @@ export class TalerExchangeHttpClient {
       case HttpStatusCode.Conflict:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 
@@ -156,9 +158,9 @@ export class TalerExchangeHttpClient {
 
     switch (resp.status) {
       case HttpStatusCode.Ok:
-        return opSuccess(resp, codecForAmlDecisionDetails());
+        return opSuccessFromHttp(resp, codecForAmlDecisionDetails());
       case HttpStatusCode.NoContent:
-        return opFixedSuccess(resp, { aml_history: [], kyc_attributes: [] });
+        return opFixedSuccess({ aml_history: [], kyc_attributes: [] });
       //this should be unauthorized
       case HttpStatusCode.Forbidden:
         return opKnownHttpFailure(resp.status, resp);
@@ -169,7 +171,7 @@ export class TalerExchangeHttpClient {
       case HttpStatusCode.Conflict:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 
@@ -203,7 +205,7 @@ export class TalerExchangeHttpClient {
       case HttpStatusCode.Conflict:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 }
