@@ -119,8 +119,13 @@ export class HttpLibImpl implements HttpRequestLibrary {
       timeoutMs = DEFAULT_REQUEST_TIMEOUT_MS;
     }
 
-    const requestHeadersMap = { ...getDefaultHeaders(method), ...opt?.headers };
-
+    const requestHeadersMap = getDefaultHeaders(method);
+    if (opt?.headers) {
+      Object.entries(opt?.headers).forEach(([key, value]) => {
+        if (value === undefined) return;
+        requestHeadersMap[key] = value
+      })
+    }
     logger.trace(`request timeout ${timeoutMs} ms`);
 
     let reqBody: ArrayBuffer | undefined;
