@@ -98,7 +98,13 @@ export class HttpLibImpl implements HttpRequestLibrary {
     }
 
     let data: ArrayBuffer | undefined = undefined;
-    const requestHeadersMap = { ...getDefaultHeaders(method), ...opt?.headers };
+    const requestHeadersMap = getDefaultHeaders(method);
+    if (opt?.headers) {
+      Object.entries(opt?.headers).forEach(([key, value]) => {
+        if (value === undefined) return;
+        requestHeadersMap[key] = value
+      })
+    }
     let headersList: string[] = [];
     for (let headerName of Object.keys(requestHeadersMap)) {
       headersList.push(`${headerName}: ${requestHeadersMap[headerName]}`);
