@@ -19,11 +19,14 @@ export * from "./errors.js";
 export { fnutil } from "./fnutils.js";
 export * from "./helpers.js";
 export * from "./http-client/bank-conversion.js";
+export * from "./http-client/authentication.js";
 export * from "./http-client/bank-core.js";
+export * from "./http-client/merchant.js";
 export * from "./http-client/bank-integration.js";
 export * from "./http-client/bank-revenue.js";
 export * from "./http-client/bank-wire.js";
 export * from "./http-client/exchange.js";
+export { CacheEvictor } from "./http-client/utils.js";
 export * from "./http-client/officer-account.js";
 export * from "./http-client/types.js";
 export * from "./http-status-codes.js";
