@@ -40,6 +40,8 @@ import {
   codecForAmountString,
   codecForAny,
   codecForBoolean,
+  codecForCheckPaymentClaimedResponse,
+  codecForCheckPaymentUnpaidResponse,
   codecForConstString,
   codecForExchangeWireAccount,
   codecForList,
@@ -112,31 +114,6 @@ export const codecForMerchantCheckPaymentPaidResponse =
       .property("refund_details", codecForAny())
       .build("CheckPaymentPaidResponse");
 
-export const codecForCheckPaymentUnpaidResponse =
-  (): Codec<CheckPaymentUnpaidResponse> =>
-    buildCodecForObject<CheckPaymentUnpaidResponse>()
-      .property("order_status", codecForConstString("unpaid"))
-      .property("taler_pay_uri", codecForString())
-      .property("order_status_url", codecForString())
-      .property("already_paid_order_id", codecOptional(codecForString()))
-      .build("CheckPaymentPaidResponse");
-
-export const codecForCheckPaymentClaimedResponse =
-  (): Codec<CheckPaymentClaimedResponse> =>
-    buildCodecForObject<CheckPaymentClaimedResponse>()
-      .property("order_status", codecForConstString("claimed"))
-      .property("contract_terms", codecForMerchantContractTerms())
-      .build("CheckPaymentClaimedResponse");
-
-export const codecForMerchantOrderPrivateStatusResponse =
-  (): Codec<MerchantOrderPrivateStatusResponse> =>
-    buildCodecForUnion<MerchantOrderPrivateStatusResponse>()
-      .discriminateOn("order_status")
-      .alternative("paid", codecForMerchantCheckPaymentPaidResponse())
-      .alternative("unpaid", codecForCheckPaymentUnpaidResponse())
-      .alternative("claimed", codecForCheckPaymentClaimedResponse())
-      .build("MerchantOrderPrivateStatusResponse");
-
 export type MerchantOrderPrivateStatusResponse =
   | MerchantCheckPaymentPaidResponse
   | CheckPaymentUnpaidResponse
