@@ -57,8 +57,6 @@ export function isOperationFail<T, E>(
 export interface OperationOk<BodyT> {
   type: "ok";
 
-  httpResp: HttpResponse;
-
   /**
    * Parsed response body.
    */
@@ -71,8 +69,6 @@ export interface OperationOk<BodyT> {
 export interface OperationFail<T> {
   type: "fail";
 
-  httpResp: HttpResponse;
-
   /**
    * Error case (either HTTP status code or TalerErrorCode)
    */
@@ -87,8 +83,6 @@ export interface OperationFail<T> {
 export interface OperationAlternative<T, B> {
   type: "fail";
 
-  httpResp: HttpResponse;
-
   case: T;
   body: B;
 }
@@ -96,37 +90,35 @@ export interface OperationAlternative<T, B> {
 export interface OperationFailWithBody<B> {
   type: "fail";
 
-  httpResp: HttpResponse;
-
   case: keyof B;
   body: B[OperationFailWithBody<B>["case"]];
 }
 
-export async function opSuccess<T>(
+export async function opSuccessFromHttp<T>(
   resp: HttpResponse,
   codec: Codec<T>,
 ): Promise<OperationOk<T>> {
   const body = await readSuccessResponseJsonOrThrow(resp, codec);
-  return { type: "ok" as const, body, httpResp: resp };
+  return { type: "ok" as const, body };
 }
 
 /**
  * Success case, but instead of the body we're returning a fixed response
  * to the client.
  */
-export function opFixedSuccess<T>(resp: HttpResponse, body: T): OperationOk<T> {
-  return { type: "ok" as const, body, httpResp: resp };
+export function opFixedSuccess<T>(body: T): OperationOk<T> {
+  return { type: "ok" as const, body };
 }
 
 export function opEmptySuccess(resp: HttpResponse): OperationOk<void> {
-  return { type: "ok" as const, body: void 0, httpResp: resp };
+  return { type: "ok" as const, body: void 0 };
 }
 
 export async function opKnownFailureWithBody<B>(
   case_: keyof B,
   body: B[typeof case_],
 ): Promise<OperationFailWithBody<B>> {
-  return { type: "fail", case: case_, body, httpResp: {} as any };
+  return { type: "fail", case: case_, body };
 }
 
 export async function opKnownAlternativeFailure<T extends HttpStatusCode, B>(
@@ -135,7 +127,7 @@ export async function opKnownAlternativeFailure<T extends HttpStatusCode, B>(
   codec: Codec<B>,
 ): Promise<OperationAlternative<T, B>> {
   const body = await readSuccessResponseJsonOrThrow(resp, codec);
-  return { type: "fail", case: s, body, httpResp: resp };
+  return { type: "fail", case: s, body };
 }
 
 export async function opKnownHttpFailure<T extends HttpStatusCode>(
@@ -143,25 +135,24 @@ export async function opKnownHttpFailure<T extends HttpStatusCode>(
   resp: HttpResponse,
 ): Promise<OperationFail<T>> {
   const detail = await readTalerErrorResponse(resp);
-  return { type: "fail", case: s, detail, httpResp: resp };
+  return { type: "fail", case: s, detail };
 }
 
-export async function opKnownTalerFailure<T extends TalerErrorCode>(
+export function opKnownTalerFailure<T extends TalerErrorCode>(
   s: T,
-  resp: HttpResponse,
-): Promise<OperationFail<T>> {
-  const detail = await readTalerErrorResponse(resp);
-  return { type: "fail", case: s, detail, httpResp: resp };
+  detail: TalerErrorDetail,
+): OperationFail<T> {
+  return { type: "fail", case: s, detail };
 }
 
-export function opUnknownFailure(resp: HttpResponse, text: string): never {
+export function opUnknownFailure(resp: HttpResponse, error: TalerErrorDetail): never {
   throw TalerError.fromDetail(
     TalerErrorCode.WALLET_UNEXPECTED_REQUEST_ERROR,
     {
       requestUrl: resp.requestUrl,
       requestMethod: resp.requestMethod,
       httpStatusCode: resp.status,
-      errorResponse: text,
+      errorResponse: error,
     },
     `Unexpected HTTP status ${resp.status} in response`,
   );
@@ -171,24 +162,18 @@ export function opUnknownFailure(resp: HttpResponse, text: string): never {
  * Convenience function to throw an error if the operation is not a success.
  */
 export function narrowOpSuccessOrThrow<Body, ErrorEnum>(
+  opName: string,
   opRes: OperationResult<Body, ErrorEnum>,
 ): asserts opRes is OperationOk<Body> {
-  const httpResponse = opRes.httpResp;
   if (opRes.type !== "ok") {
     throw TalerError.fromDetail(
-      TalerErrorCode.WALLET_UNEXPECTED_REQUEST_ERROR,
+      TalerErrorCode.GENERIC_CLIENT_INTERNAL_ERROR,
       {
-        requestUrl: httpResponse.requestUrl,
-        requestMethod: httpResponse.requestMethod,
-        httpStatusCode: httpResponse.status,
-        errorResponse:
-          "detail" in opRes
-            ? opRes.detail
-            : "body" in opRes
-              ? opRes.body
-              : undefined,
+        operation: opName,
+        error: String(opRes.case),
+        detail: "detail" in opRes ? opRes.detail : undefined,
       },
-      `Unexpected HTTP status ${httpResponse.status} in response`,
+      `Operation ${opName} failed: ${String(opRes.case)}`,
     );
   }
 }
@@ -198,10 +183,10 @@ export type ResultByMethod<
   p extends keyof TT,
 > = TT[p] extends (...args: any[]) => infer Ret
   ? Ret extends Promise<infer Result>
-    ? Result extends OperationResult<any, any>
-      ? Result
-      : never
-    : never //api always use Promises
+  ? Result extends OperationResult<any, any>
+  ? Result
+  : never
+  : never //api always use Promises
   : never; //error cases just for functions
 
 export type FailCasesByMethod<TT extends object, p extends keyof TT> = Exclude<
