@@ -32,7 +32,6 @@ import {
   buildCodecForUnion,
   codecForAny,
   codecForBoolean,
-  codecForConstNumber,
   codecForConstString,
   codecForList,
   codecForMap,
@@ -46,6 +45,7 @@ import {
   CurrencySpecification,
   codecForCurrencySpecificiation,
   codecForEither,
+  codecForProduct,
 } from "./index.js";
 import { Edx25519PublicKeyEnc } from "./taler-crypto.js";
 import {
@@ -303,15 +303,11 @@ export interface CoinDepositPermission {
  * merchant's contract terms.
  */
 export interface ExchangeHandle {
-  /**
-   * Master public signing key of the exchange.
-   */
-  master_pub: string;
-
-  /**
-   * Base URL of the exchange.
-   */
+  // The exchange's base URL.
   url: string;
+
+  // Master public key of the exchange.
+  master_pub: EddsaPublicKeyString;
 }
 
 export interface AuditorHandle {
@@ -323,7 +319,7 @@ export interface AuditorHandle {
   /**
    * Master public signing key of the auditor.
    */
-  auditor_pub: string;
+  auditor_pub: EddsaPublicKeyString;
 
   /**
    * Base URL of the auditor.
@@ -367,12 +363,24 @@ export interface Location {
 }
 
 export interface MerchantInfo {
+  // The merchant's legal name of business.
   name: string;
-  jurisdiction?: Location;
-  address?: Location;
-  logo?: string;
-  website?: string;
+
+  // Label for a location with the business address of the merchant.
   email?: string;
+
+  // Label for a location with the business address of the merchant.
+  website?: string;
+
+  // An optional base64-encoded product image.
+  logo?: ImageDataUrl;
+
+  // Label for a location with the business address of the merchant.
+  address?: Location;
+
+  // Label for a location that denotes the jurisdiction for disputes.
+  // Some of the typical fields for a location (such as a street address) may be absent.
+  jurisdiction?: Location;
 }
 
 export interface Tax {
@@ -391,10 +399,10 @@ export interface Product {
   description: string;
 
   // Map from IETF BCP 47 language tags to localized descriptions
-  description_i18n?: { [lang_tag: string]: string };
+  description_i18n?: InternationalizedString;
 
   // The number of units of the product to deliver to the customer.
-  quantity?: number;
+  quantity?: Integer;
 
   // The unit in which the product is measured (liters, kilograms, packages, etc.)
   unit?: string;
@@ -403,7 +411,7 @@ export interface Product {
   price?: AmountString;
 
   // An optional base64-encoded product image
-  image?: string;
+  image?: ImageDataUrl;
 
   // a list of taxes paid by the merchant for this product. Can be empty.
   taxes?: Tax[];
@@ -421,142 +429,147 @@ export interface InternationalizedString {
  * FIXME: Add type field!
  */
 export interface MerchantContractTerms {
-  /**
-   * Hash of the merchant's wire details.
-   */
+  // The hash of the merchant instance's wire details.
   h_wire: string;
 
-  /**
-   * Hash of the merchant's wire details.
-   */
+  // Specifies for how long the wallet should try to get an
+  // automatic refund for the purchase. If this field is
+  // present, the wallet should wait for a few seconds after
+  // the purchase and then automatically attempt to obtain
+  // a refund.  The wallet should probe until "delay"
+  // after the payment was successful (i.e. via long polling
+  // or via explicit requests with exponential back-off).
+  //
+  // In particular, if the wallet is offline
+  // at that time, it MUST repeat the request until it gets
+  // one response from the merchant after the delay has expired.
+  // If the refund is granted, the wallet MUST automatically
+  // recover the payment.  This is used in case a merchant
+  // knows that it might be unable to satisfy the contract and
+  // desires for the wallet to attempt to get the refund without any
+  // customer interaction.  Note that it is NOT an error if the
+  // merchant does not grant a refund.
   auto_refund?: TalerProtocolDuration;
 
-  /**
-   * Wire method the merchant wants to use.
-   */
+  // Wire transfer method identifier for the wire method associated with h_wire.
+  // The wallet may only select exchanges via a matching auditor if the
+  // exchange also supports this wire method.
+  // The wire transfer fees must be added based on this wire transfer method.
   wire_method: string;
 
-  /**
-   * Human-readable short summary of the contract.
-   */
+  // Human-readable description of the whole purchase.
   summary: string;
 
+  // Map from IETF BCP 47 language tags to localized summaries.
   summary_i18n?: InternationalizedString;
 
-  /**
-   * Nonce used to ensure freshness.
-   */
-  nonce: string;
-
-  /**
-   * Total amount payable.
-   */
-  amount: string;
-
-  /**
-   * Deadline to pay for the contract.
-   */
-  pay_deadline: TalerProtocolTimestamp;
-
-  /**
-   * Maximum deposit fee covered by the merchant.
-   */
-  max_fee: string;
-
-  /**
-   * Information about the merchant.
-   */
-  merchant: MerchantInfo;
-
-  /**
-   * Public key of the merchant.
-   */
-  merchant_pub: string;
-
-  /**
-   * Time indicating when the order should be delivered.
-   * May be overwritten by individual products.
-   */
-  delivery_date?: TalerProtocolTimestamp;
-
-  /**
-   * Delivery location for (all!) products.
-   */
-  delivery_location?: Location;
-
-  /**
-   * List of accepted exchanges.
-   */
-  exchanges: ExchangeHandle[];
-
-  /**
-   * Products that are sold in this contract.
-   */
-  products?: Product[];
-
-  /**
-   * Deadline for refunds.
-   */
-  refund_deadline: TalerProtocolTimestamp;
-
-  /**
-   * Deadline for the wire transfer.
-   */
-  wire_transfer_deadline: TalerProtocolTimestamp;
-
-  /**
-   * Time when the contract was generated by the merchant.
-   */
-  timestamp: TalerProtocolTimestamp;
-
-  /**
-   * Order id to uniquely identify the purchase within
-   * one merchant instance.
-   */
+  // Unique, free-form identifier for the proposal.
+  // Must be unique within a merchant instance.
+  // For merchants that do not store proposals in their DB
+  // before the customer paid for them, the order_id can be used
+  // by the frontend to restore a proposal from the information
+  // encoded in it (such as a short product identifier and timestamp).
   order_id: string;
 
-  /**
-   * Base URL of the merchant's backend.
-   */
+  // Total price for the transaction.
+  // The exchange will subtract deposit fees from that amount
+  // before transferring it to the merchant.
+  amount: string;
+
+  // Nonce generated by the wallet and echoed by the merchant
+  // in this field when the proposal is generated.
+  nonce: string;
+
+  // After this deadline, the merchant won't accept payments for the contract.
+  pay_deadline: TalerProtocolTimestamp;
+
+  // More info about the merchant, see below.
+  merchant: MerchantInfo;
+
+  // Merchant's public key used to sign this proposal; this information
+  // is typically added by the backend. Note that this can be an ephemeral key.
+  merchant_pub: string;
+
+  // Time indicating when the order should be delivered.
+  // May be overwritten by individual products.
+  delivery_date?: TalerProtocolTimestamp;
+
+  // Delivery location for (all!) products.
+  delivery_location?: Location;
+
+  // Exchanges that the merchant accepts even if it does not accept any auditors that audit them.
+  exchanges: ExchangeHandle[];
+
+  // List of products that are part of the purchase (see Product).
+  products?: Product[];
+
+  // After this deadline has passed, no refunds will be accepted.
+  refund_deadline: TalerProtocolTimestamp;
+
+  // Transfer deadline for the exchange.  Must be in the
+  // deposit permissions of coins used to pay for this order.
+  wire_transfer_deadline: TalerProtocolTimestamp;
+
+  // Time when this contract was generated.
+  timestamp: TalerProtocolTimestamp;
+
+  // Base URL of the (public!) merchant backend API.
+  // Must be an absolute URL that ends with a slash.
   merchant_base_url: string;
 
-  /**
-   * Fulfillment URL to view the product or
-   * delivery status.
-   */
+  // URL that will show that the order was successful after
+  // it has been paid for.  Optional, but either fulfillment_url
+  // or fulfillment_message must be specified in every
+  // contract terms.
+  //
+  // If a non-unique fulfillment URL is used, a customer can only
+  // buy the order once and will be redirected to a previous purchase
+  // when trying to buy an order with the same fulfillment URL a second
+  // time. This is useful for digital goods that a customer only needs
+  // to buy once but should be able to repeatedly download.
+  //
+  // For orders where the customer is expected to be able to make
+  // repeated purchases (for equivalent goods), the fulfillment URL
+  // should be made unique for every order. The easiest way to do
+  // this is to include a unique order ID in the fulfillment URL.
+  //
+  // When POSTing to the merchant, the placeholder text "${ORDER_ID}"
+  // is be replaced with the actual order ID (useful if the
+  // order ID is generated server-side and needs to be
+  // in the URL). Note that this placeholder can only be used once.
+  // Front-ends may use other means to generate a unique fulfillment URL.
   fulfillment_url?: string;
 
-  /**
-   * URL meant to share the shopping cart.
-   */
+  // URL where the same contract could be ordered again (if
+  // available). Returned also at the public order endpoint
+  // for people other than the actual buyer (hence public,
+  // in case order IDs are guessable).
   public_reorder_url?: string;
 
-  /**
-   * Plain text fulfillment message in the merchant's default language.
-   */
+  // Message shown to the customer after paying for the order.
+  // Either fulfillment_url or fulfillment_message must be specified.
   fulfillment_message?: string;
 
-  /**
-   * Internationalized fulfillment messages.
-   */
+  // Map from IETF BCP 47 language tags to localized fulfillment
+  // messages.
   fulfillment_message_i18n?: InternationalizedString;
 
-  /**
-   * Share of the wire fee that must be settled with one payment.
-   */
-  wire_fee_amortization?: number;
+  // Maximum total deposit fee accepted by the merchant for this contract.
+  // Overrides defaults of the merchant instance.
+  max_fee: string;
 
-  /**
-   * Maximum wire fee that the merchant agrees to pay for.
-   */
-  max_wire_fee?: string;
-
-  minimum_age?: number;
-
-  /**
-   * Extra data, interpreted by the mechant only.
-   */
+  // Extra data that is only interpreted by the merchant frontend.
+  // Useful when the merchant needs to store extra information on a
+  // contract without storing it separately in their database.
+  // Must really be an Object (not a string, integer, float or array).
   extra?: any;
+
+  // Minimum age the buyer must have (in years). Default is 0.
+  // This value is at least as large as the maximum over all
+  // minimum age requirements of the products in this contract.
+  // It might also be set independent of any product, due to
+  // legal requirements.
+  minimum_age?: Integer;
 }
 
 /**
@@ -610,27 +623,6 @@ export interface MerchantAbortPayRefundDetails {
   exchange_http_status: number;
 }
 
-/**
- * Response for a refund pickup or a /pay in abort mode.
- */
-export interface MerchantRefundResponse {
-  /**
-   * Public key of the merchant
-   */
-  merchant_pub: string;
-
-  /**
-   * Contract terms hash of the contract that
-   * is being refunded.
-   */
-  h_contract_terms: string;
-
-  /**
-   * The signed refund permissions, to be sent to the exchange.
-   */
-  refunds: MerchantAbortPayRefundDetails[];
-}
-
 /**
  * Planchet detail sent to the merchant.
  */
@@ -831,6 +823,7 @@ export interface DenomCommon {
 
 export type RsaPublicKeySring = string;
 export type AgeMask = number;
+export type ImageDataUrl = string;
 
 /**
  * 32-byte value representing a point on Curve25519.
@@ -1406,28 +1399,9 @@ export const codecForMerchantInfo = (): Codec<MerchantInfo> =>
     .property("jurisdiction", codecOptional(codecForLocation()))
     .build("MerchantInfo");
 
-export const codecForTax = (): Codec<Tax> =>
-  buildCodecForObject<Tax>()
-    .property("name", codecForString())
-    .property("tax", codecForAmountString())
-    .build("Tax");
-
 export const codecForInternationalizedString =
   (): Codec<InternationalizedString> => codecForMap(codecForString());
 
-export const codecForProduct = (): Codec<Product> =>
-  buildCodecForObject<Product>()
-    .property("product_id", codecOptional(codecForString()))
-    .property("description", codecForString())
-    .property(
-      "description_i18n",
-      codecOptional(codecForInternationalizedString()),
-    )
-    .property("quantity", codecOptional(codecForNumber()))
-    .property("unit", codecOptional(codecForString()))
-    .property("price", codecOptional(codecForAmountString()))
-    .build("Tax");
-
 export const codecForMerchantContractTerms = (): Codec<MerchantContractTerms> =>
   buildCodecForObject<MerchantContractTerms>()
     .property("order_id", codecForString())
@@ -1444,15 +1418,14 @@ export const codecForMerchantContractTerms = (): Codec<MerchantContractTerms> =>
     .property("summary", codecForString())
     .property("summary_i18n", codecOptional(codecForInternationalizedString()))
     .property("nonce", codecForString())
-    .property("amount", codecForString())
+    .property("amount", codecForAmountString())
     .property("pay_deadline", codecForTimestamp)
     .property("refund_deadline", codecForTimestamp)
     .property("wire_transfer_deadline", codecForTimestamp)
     .property("timestamp", codecForTimestamp)
     .property("delivery_location", codecOptional(codecForLocation()))
     .property("delivery_date", codecOptional(codecForTimestamp))
-    .property("max_fee", codecForString())
-    .property("max_wire_fee", codecOptional(codecForString()))
+    .property("max_fee", codecForAmountString())
     .property("merchant", codecForMerchantInfo())
     .property("merchant_pub", codecForString())
     .property("exchanges", codecForList(codecForExchangeHandle()))
@@ -1482,14 +1455,6 @@ export const codecForMerchantRefundPermission =
       .property("exchange_pub", codecOptional(codecForString()))
       .build("MerchantRefundPermission");
 
-export const codecForMerchantRefundResponse =
-  (): Codec<MerchantRefundResponse> =>
-    buildCodecForObject<MerchantRefundResponse>()
-      .property("merchant_pub", codecForString())
-      .property("h_contract_terms", codecForString())
-      .property("refunds", codecForList(codecForMerchantRefundPermission()))
-      .build("MerchantRefundResponse");
-
 export const codecForBlindSigWrapperV2 = (): Codec<MerchantBlindSigWrapperV2> =>
   buildCodecForObject<MerchantBlindSigWrapperV2>()
     .property("blind_sig", codecForBlindedDenominationSignature())
@@ -1645,40 +1610,6 @@ export const codecForExchangeRevealResponse =
       .property("ev_sigs", codecForList(codecForExchangeRevealItem()))
       .build("ExchangeRevealResponse");
 
-export const codecForMerchantCoinRefundSuccessStatus =
-  (): Codec<MerchantCoinRefundSuccessStatus> =>
-    buildCodecForObject<MerchantCoinRefundSuccessStatus>()
-      .property("type", codecForConstString("success"))
-      .property("coin_pub", codecForString())
-      .property("exchange_status", codecForConstNumber(200))
-      .property("exchange_sig", codecForString())
-      .property("rtransaction_id", codecForNumber())
-      .property("refund_amount", codecForAmountString())
-      .property("exchange_pub", codecForString())
-      .property("execution_time", codecForTimestamp)
-      .build("MerchantCoinRefundSuccessStatus");
-
-export const codecForMerchantCoinRefundFailureStatus =
-  (): Codec<MerchantCoinRefundFailureStatus> =>
-    buildCodecForObject<MerchantCoinRefundFailureStatus>()
-      .property("type", codecForConstString("failure"))
-      .property("coin_pub", codecForString())
-      .property("exchange_status", codecForNumber())
-      .property("rtransaction_id", codecForNumber())
-      .property("refund_amount", codecForAmountString())
-      .property("exchange_code", codecOptional(codecForNumber()))
-      .property("exchange_reply", codecOptional(codecForAny()))
-      .property("execution_time", codecForTimestamp)
-      .build("MerchantCoinRefundFailureStatus");
-
-export const codecForMerchantCoinRefundStatus =
-  (): Codec<MerchantCoinRefundStatus> =>
-    buildCodecForUnion<MerchantCoinRefundStatus>()
-      .discriminateOn("type")
-      .alternative("success", codecForMerchantCoinRefundSuccessStatus())
-      .alternative("failure", codecForMerchantCoinRefundFailureStatus())
-      .build("MerchantCoinRefundStatus");
-
 export const codecForMerchantOrderStatusPaid =
   (): Codec<MerchantOrderStatusPaid> =>
     buildCodecForObject<MerchantOrderStatusPaid>()
@@ -1688,14 +1619,6 @@ export const codecForMerchantOrderStatusPaid =
       .property("refunded", codecForBoolean())
       .build("MerchantOrderStatusPaid");
 
-export const codecForMerchantOrderRefundPickupResponse =
-  (): Codec<MerchantOrderRefundResponse> =>
-    buildCodecForObject<MerchantOrderRefundResponse>()
-      .property("merchant_pub", codecForString())
-      .property("refund_amount", codecForAmountString())
-      .property("refunds", codecForList(codecForMerchantCoinRefundStatus()))
-      .build("MerchantOrderRefundPickupResponse");
-
 export const codecForMerchantOrderStatusUnpaid =
   (): Codec<MerchantOrderStatusUnpaid> =>
     buildCodecForObject<MerchantOrderStatusUnpaid>()
@@ -1733,37 +1656,6 @@ export interface AbortResponse {
   refunds: MerchantAbortPayRefundStatus[];
 }
 
-export const codecForMerchantAbortPayRefundSuccessStatus =
-  (): Codec<MerchantAbortPayRefundSuccessStatus> =>
-    buildCodecForObject<MerchantAbortPayRefundSuccessStatus>()
-      .property("exchange_pub", codecForString())
-      .property("exchange_sig", codecForString())
-      .property("exchange_status", codecForConstNumber(200))
-      .property("type", codecForConstString("success"))
-      .build("MerchantAbortPayRefundSuccessStatus");
-
-export const codecForMerchantAbortPayRefundFailureStatus =
-  (): Codec<MerchantAbortPayRefundFailureStatus> =>
-    buildCodecForObject<MerchantAbortPayRefundFailureStatus>()
-      .property("exchange_code", codecForNumber())
-      .property("exchange_reply", codecForAny())
-      .property("exchange_status", codecForNumber())
-      .property("type", codecForConstString("failure"))
-      .build("MerchantAbortPayRefundFailureStatus");
-
-export const codecForMerchantAbortPayRefundStatus =
-  (): Codec<MerchantAbortPayRefundStatus> =>
-    buildCodecForUnion<MerchantAbortPayRefundStatus>()
-      .discriminateOn("type")
-      .alternative("success", codecForMerchantAbortPayRefundSuccessStatus())
-      .alternative("failure", codecForMerchantAbortPayRefundFailureStatus())
-      .build("MerchantAbortPayRefundStatus");
-
-export const codecForAbortResponse = (): Codec<AbortResponse> =>
-  buildCodecForObject<AbortResponse>()
-    .property("refunds", codecForList(codecForMerchantAbortPayRefundStatus()))
-    .build("AbortResponse");
-
 export type MerchantAbortPayRefundStatus =
   | MerchantAbortPayRefundSuccessStatus
   | MerchantAbortPayRefundFailureStatus;
@@ -2413,6 +2305,12 @@ export interface ExchangeWireAccount {
   // a TALER_MasterWireDetailsPS
   // with purpose TALER_SIGNATURE_MASTER_WIRE_DETAILS.
   master_sig: EddsaSignatureString;
+
+  // Display label wallets should use to show this
+  // bank account.
+  // Since protocol **v19**.
+  bank_label?: string;
+  priority?: number;
 }
 
 export const codecForExchangeWireAccount = (): Codec<ExchangeWireAccount> =>
@@ -2422,6 +2320,8 @@ export const codecForExchangeWireAccount = (): Codec<ExchangeWireAccount> =>
     .property("debit_restrictions", codecForList(codecForAny()))
     .property("master_sig", codecForString())
     .property("payto_uri", codecForString())
+    .property("bank_label", codecOptional(codecForString()))
+    .property("priority", codecOptional(codecForNumber()))
     .build("WireAccount");
 
 export type Integer = number;
@@ -2459,3 +2359,58 @@ export const codecForBankConversionInfoConfig =
         codecForCurrencySpecificiation(),
       )
       .build("BankConversionInfoConfig");
+
+export interface DenominationExpiredMessage {
+  // Taler error code.  Note that beyond
+  // expiration this message format is also
+  // used if the key is not yet valid, or
+  // has been revoked.
+  code: number;
+
+  // Signature by the exchange over a
+  // TALER_DenominationExpiredAffirmationPS.
+  // Must have purpose TALER_SIGNATURE_EXCHANGE_AFFIRM_DENOM_EXPIRED.
+  exchange_sig: EddsaSignatureString;
+
+  // Public key of the exchange used to create
+  // the 'exchange_sig.
+  exchange_pub: EddsaPublicKeyString;
+
+  // Hash of the denomination public key that is unknown.
+  h_denom_pub: HashCodeString;
+
+  // When was the signature created.
+  timestamp: TalerProtocolTimestamp;
+
+  // What kind of operation was requested that now
+  // failed?
+  oper: string;
+}
+
+export const codecForDenominationExpiredMessage = () =>
+  buildCodecForObject<DenominationExpiredMessage>()
+    .property("code", codecForNumber())
+    .property("exchange_sig", codecForString())
+    .property("exchange_pub", codecForString())
+    .property("h_denom_pub", codecForString())
+    .property("timestamp", codecForTimestamp)
+    .property("oper", codecForString())
+    .build("DenominationExpiredMessage");
+
+export interface CoinHistoryResponse {
+  // Current balance of the coin.
+  balance: AmountString;
+
+  // Hash of the coin's denomination.
+  h_denom_pub: HashCodeString;
+
+  // Transaction history for the coin.
+  history: any[];
+}
+
+export const codecForCoinHistoryResponse = () =>
+  buildCodecForObject<CoinHistoryResponse>()
+    .property("balance", codecForAmountString())
+    .property("h_denom_pub", codecForString())
+    .property("history", codecForAny())
+    .build("CoinHistoryResponse");
