@@ -82,7 +82,7 @@ export class ConfigValue<T> {
     private optionName: string,
     private value: string | undefined,
     private converter: (x: string) => T,
-  ) { }
+  ) {}
 
   required(): T {
     if (this.value == undefined) {
@@ -114,7 +114,7 @@ export class ConfigValue<T> {
   }
 
   getValue(): string | undefined {
-    return this.value
+    return this.value;
   }
 }
 
@@ -249,7 +249,7 @@ function globMatch(pattern: string, str: string): boolean {
   /* Backtrack position in pattern */
   let patBt = -1;
 
-  for (; ;) {
+  for (;;) {
     if (pattern[patPos] === "*") {
       strBt = strPos;
       patBt = patPos++;
