@@ -25,9 +25,10 @@
  */
 import { Codec, Context, DecodingError, renderContext } from "./codec.js";
 import { canonicalizeBaseUrl } from "./helpers.js";
+import { opFixedSuccess, opKnownTalerFailure } from "./operation.js";
+import { TalerErrorCode } from "./taler-error-codes.js";
 import { AmountString } from "./taler-types.js";
 import { URL, URLSearchParams } from "./url.js";
-
 /**
  * A parsed taler URI.
  */
@@ -40,14 +41,15 @@ export type TalerUri =
   | BackupRestoreUri
   | RefundUriResult
   | WithdrawUriResult
-  | WithdrawExchangeUri;
+  | WithdrawExchangeUri
+  | AddExchangeUri;
 
 declare const __action_str: unique symbol;
-export type TalerActionString = string & { [__action_str]: true };
+export type TalerUriString = string & { [__action_str]: true };
 
-export function codecForTalerActionString(): Codec<TalerActionString> {
+export function codecForTalerUriString(): Codec<TalerUriString> {
   return {
-    decode(x: any, c?: Context): TalerActionString {
+    decode(x: any, c?: Context): TalerUriString {
       if (typeof x !== "string") {
         throw new DecodingError(
           `expected string at ${renderContext(c)} but got ${typeof x}`,
@@ -55,10 +57,10 @@ export function codecForTalerActionString(): Codec<TalerActionString> {
       }
       if (parseTalerUri(x) === undefined) {
         throw new DecodingError(
-          `invalid taler action at ${renderContext(c)} but got "${x}"`,
+          `invalid taler URI at ${renderContext(c)} but got "${x}"`,
         );
       }
-      return x as TalerActionString;
+      return x as TalerUriString;
     },
   };
 }
@@ -126,19 +128,26 @@ export interface WithdrawExchangeUri {
   amount?: AmountString;
 }
 
+export interface AddExchangeUri {
+  type: TalerUriAction.AddExchange;
+  exchangeBaseUrl: string;
+}
+
 /**
  * Parse a taler[+http]://withdraw URI.
  * Return undefined if not passed a valid URI.
  */
-export function parseWithdrawUri(s: string): WithdrawUriResult | undefined {
-  const pi = parseProtoInfo(s, "withdraw");
-  if (!pi) {
-    return undefined;
+export function parseWithdrawUriWithError(s: string) {
+  const pi = parseProtoInfoWithError(s, "withdraw");
+  if (pi.type === "fail") {
+    return pi;
   }
-  const parts = pi.rest.split("/");
+  const parts = pi.body.rest.split("/");
 
   if (parts.length < 2) {
-    return undefined;
+    return opKnownTalerFailure(TalerErrorCode.WALLET_TALER_URI_MALFORMED, {
+      code: TalerErrorCode.WALLET_TALER_URI_MALFORMED,
+    });
   }
 
   const host = parts[0].toLowerCase();
@@ -153,11 +162,71 @@ export function parseWithdrawUri(s: string): WithdrawUriResult | undefined {
   const withdrawId = parts[parts.length - 1];
   const p = [host, ...pathSegments].join("/");
 
-  return {
+  const result: WithdrawUriResult = {
     type: TalerUriAction.Withdraw,
-    bankIntegrationApiBaseUrl: canonicalizeBaseUrl(`${pi.innerProto}://${p}/`),
+    bankIntegrationApiBaseUrl: canonicalizeBaseUrl(
+      `${pi.body.innerProto}://${p}/`,
+    ),
     withdrawalOperationId: withdrawId,
   };
+  return opFixedSuccess(result);
+}
+
+/**
+ *
+ * @deprecated use parseWithdrawUriWithError
+ */
+export function parseWithdrawUri(s: string): WithdrawUriResult | undefined {
+  const r = parseWithdrawUriWithError(s);
+  if (r.type === "fail") return undefined;
+  return r.body;
+}
+
+/**
+ * Parse a taler[+http]://withdraw URI.
+ * Return undefined if not passed a valid URI.
+ */
+export function parseAddExchangeUriWithError(s: string) {
+  const pi = parseProtoInfoWithError(s, "add-exchange");
+  if (pi.type === "fail") {
+    return pi;
+  }
+  const parts = pi.body.rest.split("/");
+
+  if (parts.length < 2) {
+    return opKnownTalerFailure(TalerErrorCode.WALLET_TALER_URI_MALFORMED, {
+      code: TalerErrorCode.WALLET_TALER_URI_MALFORMED,
+    });
+  }
+
+  const host = parts[0].toLowerCase();
+  const pathSegments = parts.slice(1, parts.length - 1);
+  /**
+   * The statement below does not tolerate a slash-ended URI.
+   * This results in (1) the withdrawalId being passed as the
+   * empty string, and (2) the bankIntegrationApi ending with the
+   * actual withdrawal operation ID.  That can be fixed by
+   * trimming the parts-list.  FIXME
+   */
+  const p = [host, ...pathSegments].join("/");
+
+  const result: AddExchangeUri = {
+    type: TalerUriAction.AddExchange,
+    exchangeBaseUrl: canonicalizeBaseUrl(
+      `${pi.body.innerProto}://${p}/`,
+    ),
+  };
+  return opFixedSuccess(result);
+}
+
+/**
+ *
+ * @deprecated use parseWithdrawUriWithError
+ */
+export function parseAddExchangeUri(s: string): AddExchangeUri | undefined {
+  const r = parseAddExchangeUriWithError(s);
+  if (r.type === "fail") return undefined;
+  return r.body;
 }
 
 /**
@@ -187,6 +256,7 @@ export enum TalerUriAction {
   Restore = "restore",
   DevExperiment = "dev-experiment",
   WithdrawExchange = "withdraw-exchange",
+  AddExchange = "add-exchange",
 }
 
 interface TalerUriProtoInfo {
@@ -215,6 +285,34 @@ function parseProtoInfo(
   }
 }
 
+function parseProtoInfoWithError(s: string, action: string) {
+  if (
+    !s.toLowerCase().startsWith("taler://") &&
+    !s.toLowerCase().startsWith("taler+http://")
+  ) {
+    return opKnownTalerFailure(TalerErrorCode.WALLET_TALER_URI_MALFORMED, {
+      code: TalerErrorCode.WALLET_TALER_URI_MALFORMED,
+    });
+  }
+  const pfxPlain = `taler://${action}/`;
+  const pfxHttp = `taler+http://${action}/`;
+  if (s.toLowerCase().startsWith(pfxPlain)) {
+    return opFixedSuccess({
+      innerProto: "https",
+      rest: s.substring(pfxPlain.length),
+    });
+  } else if (s.toLowerCase().startsWith(pfxHttp)) {
+    return opFixedSuccess({
+      innerProto: "http",
+      rest: s.substring(pfxHttp.length),
+    });
+  } else {
+    return opKnownTalerFailure(TalerErrorCode.WALLET_TALER_URI_MALFORMED, {
+      code: TalerErrorCode.WALLET_TALER_URI_MALFORMED,
+    });
+  }
+}
+
 type Parser = (s: string) => TalerUri | undefined;
 const parsers: { [A in TalerUriAction]: Parser } = {
   [TalerUriAction.Pay]: parsePayUri,
@@ -226,6 +324,7 @@ const parsers: { [A in TalerUriAction]: Parser } = {
   [TalerUriAction.Withdraw]: parseWithdrawUri,
   [TalerUriAction.DevExperiment]: parseDevExperimentUri,
   [TalerUriAction.WithdrawExchange]: parseWithdrawExchangeUri,
+  [TalerUriAction.AddExchange]: parseAddExchangeUri,
 };
 
 export function parseTalerUri(string: string): TalerUri | undefined {
@@ -269,6 +368,9 @@ export function stringifyTalerUri(uri: TalerUri): string {
     case TalerUriAction.WithdrawExchange: {
       return stringifyWithdrawExchange(uri);
     }
+    case TalerUriAction.AddExchange: {
+      return stringifyAddExchange(uri);
+    }
   }
 }
 
@@ -548,6 +650,13 @@ export function stringifyWithdrawExchange({
   return `${proto}://withdraw-exchange/${path}${exchangePub ?? ""}${query}`;
 }
 
+export function stringifyAddExchange({
+  exchangeBaseUrl,
+}: Omit<AddExchangeUri, "type">): string {
+  const { proto, path } = getUrlInfo(exchangeBaseUrl);
+  return `${proto}://add-exchange/${path}`;
+}
+
 export function stringifyDevExperimentUri({
   devExperimentId,
 }: Omit<DevExperimentUri, "type">): string {
