@@ -77,8 +77,13 @@ export interface TransactionsRequest {
    * Sort order of the transaction items.
    * By default, items are sorted ascending by their
    * main timestamp.
+   *
+   * ascending: ascending by timestamp, but pending transactions first
+   * descending: ascending by timestamp, but pending transactions first
+   * stable-ascending: ascending by timestamp, with pending transactions amidst other transactions
+   *    (stable in the sense of: pending transactions don't jump around)
    */
-  sort?: "ascending" | "descending";
+  sort?: "ascending" | "descending" | "stable-ascending";
 
   /**
    * If true, include all refreshes in the transactions list.
@@ -217,7 +222,8 @@ export type Transaction =
   | TransactionPeerPushCredit
   | TransactionPeerPushDebit
   | TransactionInternalWithdrawal
-  | TransactionRecoup;
+  | TransactionRecoup
+  | TransactionDenomLoss;
 
 export enum TransactionType {
   Withdrawal = "withdrawal",
@@ -232,6 +238,7 @@ export enum TransactionType {
   PeerPullDebit = "peer-pull-debit",
   PeerPullCredit = "peer-pull-credit",
   Recoup = "recoup",
+  DenomLoss = "denom-loss",
 }
 
 export enum WithdrawalType {
@@ -293,6 +300,22 @@ interface WithdrawalDetailsForTalerBankIntegrationApi {
   exchangeCreditAccountDetails?: WithdrawalExchangeAccountDetails[];
 }
 
+export enum DenomLossEventType {
+  DenomExpired = "denom-expired",
+  DenomVanished = "denom-vanished",
+  DenomUnoffered = "denom-unoffered",
+}
+
+/**
+ * A transaction to indicate financial loss due to denominations
+ * that became unusable for deposits.
+ */
+export interface TransactionDenomLoss extends TransactionCommon {
+  type: TransactionType.DenomLoss;
+  lossEventType: DenomLossEventType;
+  exchangeBaseUrl: string;
+}
+
 /**
  * A withdrawal transaction (either bank-integrated or manual).
  */
@@ -747,6 +770,7 @@ export const codecForTransactionsRequest = (): Codec<TransactionsRequest> =>
         codecForEither(
           codecForConstString("ascending"),
           codecForConstString("descending"),
+          codecForConstString("stable-ascending"),
         ),
       ),
     )
