@@ -462,7 +462,7 @@ export interface GetCurrencySpecificationResponse {
 
 export interface BuiltinExchange {
   exchangeBaseUrl: string;
-  currencyHint?: string;
+  currencyHint: string;
 }
 
 export interface PartialWalletRunConfig {
@@ -578,6 +578,11 @@ export enum CoinStatus {
    */
   Fresh = "fresh",
 
+  /**
+   * Coin was lost as the denomination is not usable anymore.
+   */
+  DenomLoss = "denom-loss",
+
   /**
    * Fresh, but currently marked as "suspended", thus won't be used
    * for spending.  Used for testing.
@@ -884,6 +889,11 @@ export interface PaymentInsufficientBalanceDetails {
       balanceReceiverAcceptable: AmountString;
       balanceReceiverDepositable: AmountString;
       maxEffectiveSpendAmount: AmountString;
+      /**
+       * Exchange doesn't have global fees configured for the relevant year,
+       * p2p payments aren't possible.
+       */
+      missingGlobalFees: boolean;
     };
   };
 }
@@ -1387,14 +1397,25 @@ export interface ShortExchangeListItem {
 export interface ExchangeListItem {
   exchangeBaseUrl: string;
   masterPub: string | undefined;
-  currency: string | undefined;
+  currency: string;
   paytoUris: string[];
   tosStatus: ExchangeTosStatus;
   exchangeEntryStatus: ExchangeEntryStatus;
   exchangeUpdateStatus: ExchangeUpdateStatus;
   ageRestrictionOptions: number[];
 
-  scopeInfo: ScopeInfo | undefined;
+  /**
+   * P2P payments are disabled with this exchange
+   * (e.g. because no global fees are configured).
+   */
+  peerPaymentsDisabled: boolean;
+
+  /**
+   * Set to true if this exchange doesn't charge any fees.
+   */
+  noFees: boolean;
+
+  scopeInfo: ScopeInfo;
 
   lastUpdateTimestamp: TalerPreciseTimestamp | undefined;
 
@@ -1462,7 +1483,7 @@ export const codecForExchangeFullDetails = (): Codec<ExchangeFullDetails> =>
 
 export const codecForExchangeListItem = (): Codec<ExchangeListItem> =>
   buildCodecForObject<ExchangeListItem>()
-    .property("currency", codecOptional(codecForString()))
+    .property("currency", codecForString())
     .property("exchangeBaseUrl", codecForString())
     .property("masterPub", codecOptional(codecForString()))
     .property("paytoUris", codecForList(codecForString()))
@@ -1473,6 +1494,8 @@ export const codecForExchangeListItem = (): Codec<ExchangeListItem> =>
     .property("scopeInfo", codecForScopeInfo())
     .property("lastUpdateErrorInfo", codecForAny())
     .property("lastUpdateTimestamp", codecOptional(codecForPreciseTimestamp))
+    .property("noFees", codecForBoolean())
+    .property("peerPaymentsDisabled", codecForBoolean())
     .build("ExchangeListItem");
 
 export const codecForExchangesListResponse = (): Codec<ExchangesListResponse> =>
@@ -1548,16 +1571,25 @@ export interface WithdrawalDetailsForAmount {
   scopeInfo: ScopeInfo;
 }
 
+export interface DenomSelItem {
+  denomPubHash: string;
+  count: number;
+  /**
+   * Number of denoms/planchets to skip, because
+   * a re-denomination effectively deleted them.
+   */
+  skip?: number;
+}
+
 /**
  * Selected denominations withn some extra info.
  */
 export interface DenomSelectionState {
   totalCoinValue: AmountString;
   totalWithdrawCost: AmountString;
-  selectedDenoms: {
-    denomPubHash: string;
-    count: number;
-  }[];
+  selectedDenoms: DenomSelItem[];
+  earliestDepositExpiration: TalerProtocolTimestamp;
+  hasDenomWithAgeRestriction: boolean;
 }
 
 /**
@@ -1590,16 +1622,6 @@ export interface ExchangeWithdrawalDetails {
    */
   earliestDepositExpiration: TalerProtocolTimestamp;
 
-  /**
-   * Number of currently offered denominations.
-   */
-  numOfferedDenoms: number;
-
-  /**
-   * Public keys of trusted auditors for the currency we're withdrawing.
-   */
-  trustedAuditorPubs: string[];
-
   /**
    * Result of checking the wallet's version
    * against the exchange's version.
@@ -1835,6 +1857,19 @@ export interface GetWithdrawalDetailsForAmountRequest {
   exchangeBaseUrl: string;
   amount: AmountString;
   restrictAge?: number;
+
+  /**
+   * ID provided by the client to cancel the request.
+   *
+   * If the same request is made again with the same clientCancellationId,
+   * all previous requests are cancelled.
+   *
+   * The cancelled request will receive an error response with
+   * an error code that indicates the cancellation.
+   *
+   * The cancellation is best-effort, responses might still arrive.
+   */
+  clientCancellationId?: string;
 }
 
 export interface AcceptBankIntegratedWithdrawalRequest {
@@ -1859,6 +1894,7 @@ export const codecForGetWithdrawalDetailsForAmountRequest =
       .property("exchangeBaseUrl", codecForString())
       .property("amount", codecForAmountString())
       .property("restrictAge", codecOptional(codecForNumber()))
+      .property("clientCancellationId", codecOptional(codecForString()))
       .build("GetWithdrawalDetailsForAmountRequest");
 
 export interface AcceptExchangeTosRequest {
@@ -1960,13 +1996,16 @@ export const codecForAbortProposalRequest = (): Codec<AbortProposalRequest> =>
     .build("AbortProposalRequest");
 
 export interface GetContractTermsDetailsRequest {
-  proposalId: string;
+  // @deprecated use transaction id
+  proposalId?: string;
+  transactionId?: string;
 }
 
 export const codecForGetContractTermsDetails =
   (): Codec<GetContractTermsDetailsRequest> =>
     buildCodecForObject<GetContractTermsDetailsRequest>()
-      .property("proposalId", codecForString())
+      .property("proposalId", codecOptional(codecForString()))
+      .property("transactionId", codecOptional(codecForString()))
       .build("GetContractTermsDetails");
 
 export interface PreparePayRequest {
@@ -2150,13 +2189,24 @@ export const codecForSetCoinSuspendedRequest =
       .property("suspended", codecForBoolean())
       .build("SetCoinSuspendedRequest");
 
+export interface RefreshCoinSpec {
+  coinPub: string;
+  amount?: AmountString;
+}
+
+export const codecForRefreshCoinSpec = (): Codec<RefreshCoinSpec> =>
+  buildCodecForObject<RefreshCoinSpec>()
+    .property("amount", codecForAmountString())
+    .property("coinPub", codecForString())
+    .build("ForceRefreshRequest");
+
 export interface ForceRefreshRequest {
-  coinPubList: string[];
+  refreshCoinSpecs: RefreshCoinSpec[];
 }
 
 export const codecForForceRefreshRequest = (): Codec<ForceRefreshRequest> =>
   buildCodecForObject<ForceRefreshRequest>()
-    .property("coinPubList", codecForList(codecForString()))
+    .property("refreshCoinSpecs", codecForList(codecForRefreshCoinSpec()))
     .build("ForceRefreshRequest");
 
 export interface PrepareRefundRequest {
@@ -2632,8 +2682,16 @@ export interface TestPayResult {
 }
 
 export interface SelectedCoin {
+  denomPubHash: string;
   coinPub: string;
   contribution: AmountString;
+  exchangeBaseUrl: string;
+}
+
+export interface SelectedProspectiveCoin {
+  denomPubHash: string;
+  contribution: AmountString;
+  exchangeBaseUrl: string;
 }
 
 /**
@@ -2654,6 +2712,20 @@ export interface PayCoinSelection {
   customerDepositFees: AmountString;
 }
 
+export interface ProspectivePayCoinSelection {
+  prospectiveCoins: SelectedProspectiveCoin[];
+
+  /**
+   * How much of the wire fees is the customer paying?
+   */
+  customerWireFees: AmountString;
+
+  /**
+   * How much of the deposit fees is the customer paying?
+   */
+  customerDepositFees: AmountString;
+}
+
 export interface CheckPeerPushDebitRequest {
   /**
    * Preferred exchange to use for the p2p payment.
@@ -2766,14 +2838,7 @@ export const codecForCheckPeerPullPaymentRequest =
       .build("PreparePeerPullDebitRequest");
 
 export interface ConfirmPeerPushCreditRequest {
-  /**
-   * Transparent identifier of the incoming peer push payment.
-   *
-   * @deprecated specify transactionId instead!
-   */
-  peerPushCreditId?: string;
-
-  transactionId?: string;
+  transactionId: string;
 }
 export interface AcceptPeerPushPaymentResponse {
   transactionId: TransactionIdStr;
@@ -2786,19 +2851,11 @@ export interface AcceptPeerPullPaymentResponse {
 export const codecForConfirmPeerPushPaymentRequest =
   (): Codec<ConfirmPeerPushCreditRequest> =>
     buildCodecForObject<ConfirmPeerPushCreditRequest>()
-      .property("peerPushCreditId", codecOptional(codecForString()))
-      .property("transactionId", codecOptional(codecForString()))
+      .property("transactionId", codecForString())
       .build("ConfirmPeerPushCreditRequest");
 
 export interface ConfirmPeerPullDebitRequest {
-  /**
-   * Transparent identifier of the incoming peer pull payment.
-   *
-   * @deprecated use transactionId instead
-   */
-  peerPullDebitId?: string;
-
-  transactionId?: TransactionIdStr;
+  transactionId: TransactionIdStr;
 }
 
 export interface ApplyDevExperimentRequest {
@@ -2814,8 +2871,7 @@ export const codecForApplyDevExperiment =
 export const codecForAcceptPeerPullPaymentRequest =
   (): Codec<ConfirmPeerPullDebitRequest> =>
     buildCodecForObject<ConfirmPeerPullDebitRequest>()
-      .property("peerPullDebitId", codecOptional(codecForString()))
-      .property("transactionId", codecOptional(codecForTransactionIdStr()))
+      .property("transactionId", codecForTransactionIdStr())
       .build("ConfirmPeerPullDebitRequest");
 
 export interface CheckPeerPullCreditRequest {
@@ -2963,7 +3019,6 @@ export interface WalletContractData {
   summary: string;
   summaryI18n: { [lang_tag: string]: string } | undefined;
   autoRefund: TalerProtocolDuration | undefined;
-  wireFeeAmortization: number;
   payDeadline: TalerProtocolTimestamp;
   refundDeadline: TalerProtocolTimestamp;
   allowedExchanges: AllowedExchangeInfo[];
@@ -3019,9 +3074,7 @@ export interface WithdrawalExchangeAccountDetails {
    * Transfer amount. Might be in a different currency than the requested
    * amount for withdrawal.
    *
-   * Redundant with the amount in paytoUri, just included to avoid parsing.
-   *
-   * Only included if this account does a currency conversion.
+   * Absent if this is a conversion account and the conversion failed.
    */
   transferAmount?: AmountString;
 
@@ -3038,6 +3091,16 @@ export interface WithdrawalExchangeAccountDetails {
    */
   creditRestrictions?: AccountRestriction[];
 
+  /**
+   * Label given to the account or the account's bank by the exchange.
+   */
+  bankLabel?: string;
+
+  /*
+   * Display priority assigned to this bank account by the exchange.
+   */
+  priority?: number;
+
   /**
    * Error that happened when attempting to request the conversion rate.
    */
