@@ -1908,15 +1908,22 @@ function parseURL(
 }
 
 export class URLImpl {
-  constructor(url: string, base?: string) {
+  //Include URL type for "url" and "base" params.
+  constructor(url: string | URL, base?: string | URL) {
     let parsedBase = null;
     if (base !== undefined) {
+      if (base instanceof URL) {
+        base = base.href;
+      }
       parsedBase = basicURLParse(base);
       if (parsedBase === null) {
         throw new TypeError(`Invalid base URL: ${base}`);
       }
     }
 
+    if (url instanceof URL) {
+      url = url.href;
+    }
     const parsedURL = basicURLParse(url, { baseURL: parsedBase });
     if (parsedURL === null) {
       throw new TypeError(`Invalid URL: ${url}`);
