@@ -1057,13 +1057,14 @@ peerCli
 
 peerCli
   .subcommand("confirmIncomingPayPull", "confirm-pull-debit")
-  .requiredArgument("peerPullDebitId", clk.STRING)
+  .requiredArgument("transactionId", clk.STRING)
   .action(async (args) => {
     await withWallet(args, async (wallet) => {
       const resp = await wallet.client.call(
         WalletApiOperation.ConfirmPeerPullDebit,
         {
-          peerPullDebitId: args.confirmIncomingPayPull.peerPullDebitId,
+          transactionId: args.confirmIncomingPayPull
+            .transactionId as TransactionIdStr,
         },
       );
       console.log(JSON.stringify(resp, undefined, 2));
@@ -1535,7 +1536,11 @@ advancedCli
   .action(async (args) => {
     await withWallet(args, async (wallet) => {
       await wallet.client.call(WalletApiOperation.ForceRefresh, {
-        coinPubList: [args.refresh.coinPub],
+        refreshCoinSpecs: [
+          {
+            coinPub: args.refresh.coinPub,
+          },
+        ],
       });
     });
   });
