@@ -67,8 +67,10 @@ import {
 import { ExchangeRestrictionSpec, findMatchingWire } from "./coinSelection.js";
 import {
   DepositOperationStatus,
+  ExchangeEntryDbRecordStatus,
   OPERATION_STATUS_ACTIVE_FIRST,
   OPERATION_STATUS_ACTIVE_LAST,
+  PeerPushDebitStatus,
   RefreshGroupRecord,
   RefreshOperationStatus,
   WalletDbReadOnlyTransaction,
@@ -294,6 +296,7 @@ export async function getBalancesInsideTransaction(
       "withdrawalGroups",
       "globalCurrencyAuditors",
       "globalCurrencyExchanges",
+      "peerPushDebit",
     ]
   >,
 ): Promise<BalancesResponse> {
@@ -304,6 +307,18 @@ export async function getBalancesInsideTransaction(
     OPERATION_STATUS_ACTIVE_LAST,
   );
 
+  await tx.exchanges.iter().forEachAsync(async (ex) => {
+    if (
+      ex.entryStatus === ExchangeEntryDbRecordStatus.Used ||
+      ex.tosAcceptedTimestamp != null
+    ) {
+      const det = await getExchangeWireDetailsInTx(tx, ex.baseUrl);
+      if (det) {
+        await balanceStore.addZero(det.currency, ex.baseUrl);
+      }
+    }
+  });
+
   await tx.coinAvailability.iter().forEachAsync(async (ca) => {
     const count = ca.visibleCoinCount ?? 0;
     await balanceStore.addZero(ca.currency, ca.exchangeBaseUrl);
@@ -386,6 +401,27 @@ export async function getBalancesInsideTransaction(
       );
     });
 
+  await tx.peerPushDebit.indexes.byStatus
+    .iter(keyRangeActive)
+    .forEachAsync(async (ppdRecord) => {
+      switch (ppdRecord.status) {
+        case PeerPushDebitStatus.AbortingDeletePurse:
+        case PeerPushDebitStatus.SuspendedAbortingDeletePurse:
+        case PeerPushDebitStatus.PendingReady:
+        case PeerPushDebitStatus.SuspendedReady:
+        case PeerPushDebitStatus.PendingCreatePurse:
+        case PeerPushDebitStatus.SuspendedCreatePurse: {
+          const currency = Amounts.currencyOf(ppdRecord.amount);
+          await balanceStore.addPendingOutgoing(
+            currency,
+            ppdRecord.exchangeBaseUrl,
+            ppdRecord.totalCost,
+          );
+          break;
+        }
+      }
+    });
+
   await tx.depositGroups.indexes.byStatus
     .iter(keyRangeActive)
     .forEachAsync(async (dgRecord) => {
@@ -447,6 +483,7 @@ export async function getBalances(
       "purchases",
       "refreshGroups",
       "withdrawalGroups",
+      "peerPushDebit",
     ],
     async (tx) => {
       return getBalancesInsideTransaction(wex, tx);
