@@ -36,6 +36,7 @@ import {
   checkLogicInvariant,
   CoinStatus,
   DenominationInfo,
+  ExchangeGlobalFees,
   ForcedCoinSel,
   InternationalizedString,
   j2s,
@@ -43,7 +44,9 @@ import {
   parsePaytoUri,
   PayCoinSelection,
   PaymentInsufficientBalanceDetails,
+  ProspectivePayCoinSelection,
   SelectedCoin,
+  SelectedProspectiveCoin,
   strcmp,
   TalerProtocolTimestamp,
 } from "@gnu-taler/taler-util";
@@ -96,7 +99,6 @@ export interface CoinSelectionTally {
 function tallyFees(
   tally: CoinSelectionTally,
   wireFeesPerExchange: Record<string, AmountJson>,
-  wireFeeAmortization: number,
   exchangeBaseUrl: string,
   feeDeposit: AmountJson,
 ): void {
@@ -107,7 +109,7 @@ function tallyFees(
       wireFeesPerExchange[exchangeBaseUrl] ?? Amounts.zeroOfCurrency(currency);
     // The remaining, amortized amount needs to be paid by the
     // wallet or covered by the deposit fee allowance.
-    let wfRemaining = Amounts.divide(wf, wireFeeAmortization);
+    let wfRemaining = wf;
     // This is the amount forgiven via the deposit fee allowance.
     const wfDepositForgiven = Amounts.min(
       tally.amountDepositFeeLimitRemaining,
@@ -157,8 +159,99 @@ export type SelectPayCoinsResult =
       type: "failure";
       insufficientBalanceDetails: PaymentInsufficientBalanceDetails;
     }
+  | { type: "prospective"; result: ProspectivePayCoinSelection }
   | { type: "success"; coinSel: PayCoinSelection };
 
+async function internalSelectPayCoins(
+  wex: WalletExecutionContext,
+  tx: WalletDbReadOnlyTransaction<
+    [
+      "coinAvailability",
+      "denominations",
+      "refreshGroups",
+      "exchanges",
+      "exchangeDetails",
+      "coins",
+    ]
+  >,
+  req: SelectPayCoinRequestNg,
+  includePendingCoins: boolean,
+): Promise<
+  | { sel: SelResult; coinRes: SelectedCoin[]; tally: CoinSelectionTally }
+  | undefined
+> {
+  const { contractTermsAmount, depositFeeLimit } = req;
+  const [candidateDenoms, wireFeesPerExchange] = await selectPayCandidates(
+    wex,
+    tx,
+    {
+      restrictExchanges: req.restrictExchanges,
+      instructedAmount: req.contractTermsAmount,
+      restrictWireMethod: req.restrictWireMethod,
+      depositPaytoUri: req.depositPaytoUri,
+      requiredMinimumAge: req.requiredMinimumAge,
+      includePendingCoins,
+    },
+  );
+
+  if (logger.shouldLogTrace()) {
+    logger.trace(
+      `instructed amount: ${Amounts.stringify(req.contractTermsAmount)}`,
+    );
+    logger.trace(`wire fees per exchange: ${j2s(wireFeesPerExchange)}`);
+    logger.trace(`candidates: ${j2s(candidateDenoms)}`);
+  }
+
+  const coinRes: SelectedCoin[] = [];
+  const currency = contractTermsAmount.currency;
+
+  let tally: CoinSelectionTally = {
+    amountPayRemaining: contractTermsAmount,
+    amountDepositFeeLimitRemaining: depositFeeLimit,
+    customerDepositFees: Amounts.zeroOfCurrency(currency),
+    customerWireFees: Amounts.zeroOfCurrency(currency),
+    wireFeeCoveredForExchange: new Set(),
+    lastDepositFee: Amounts.zeroOfCurrency(currency),
+  };
+
+  await maybeRepairCoinSelection(
+    wex,
+    tx,
+    req.prevPayCoins ?? [],
+    coinRes,
+    tally,
+    {
+      wireFeesPerExchange: wireFeesPerExchange,
+    },
+  );
+
+  let selectedDenom: SelResult | undefined;
+  if (req.forcedSelection) {
+    selectedDenom = selectForced(req, candidateDenoms);
+  } else {
+    // FIXME:  Here, we should select coins in a smarter way.
+    // Instead of always spending the next-largest coin,
+    // we should try to find the smallest coin that covers the
+    // amount.
+    selectedDenom = selectGreedy(
+      {
+        wireFeesPerExchange: wireFeesPerExchange,
+      },
+      candidateDenoms,
+      tally,
+    );
+  }
+
+  if (!selectedDenom) {
+    return undefined;
+  }
+  return {
+    sel: selectedDenom,
+    coinRes,
+    tally,
+  };
+}
+
 /**
  * Select coins to spend under the merchant's constraints.
  *
@@ -170,8 +263,6 @@ export async function selectPayCoins(
   wex: WalletExecutionContext,
   req: SelectPayCoinRequestNg,
 ): Promise<SelectPayCoinsResult> {
-  const { contractTermsAmount, depositFeeLimit } = req;
-
   if (logger.shouldLogTrace()) {
     logger.trace(`selecting coins for ${j2s(req)}`);
   }
@@ -186,63 +277,42 @@ export async function selectPayCoins(
       "coins",
     ],
     async (tx) => {
-      const [candidateDenoms, wireFeesPerExchange] = await selectPayCandidates(
-        wex,
-        tx,
-        {
-          restrictExchanges: req.restrictExchanges,
-          instructedAmount: req.contractTermsAmount,
-          restrictWireMethod: req.restrictWireMethod,
-          depositPaytoUri: req.depositPaytoUri,
-          requiredMinimumAge: req.requiredMinimumAge,
-        },
-      );
+      const materialAvSel = await internalSelectPayCoins(wex, tx, req, false);
 
-      logger.trace(`wire fees per exchange: ${j2s(wireFeesPerExchange)}`);
-
-      const coinRes: SelectedCoin[] = [];
-      const currency = contractTermsAmount.currency;
-
-      let tally: CoinSelectionTally = {
-        amountPayRemaining: contractTermsAmount,
-        amountDepositFeeLimitRemaining: depositFeeLimit,
-        customerDepositFees: Amounts.zeroOfCurrency(currency),
-        customerWireFees: Amounts.zeroOfCurrency(currency),
-        wireFeeCoveredForExchange: new Set(),
-        lastDepositFee: Amounts.zeroOfCurrency(currency),
-      };
-
-      await maybeRepairCoinSelection(
-        wex,
-        tx,
-        req.prevPayCoins ?? [],
-        coinRes,
-        tally,
-        {
-          wireFeeAmortization: req.wireFeeAmortization,
-          wireFeesPerExchange: wireFeesPerExchange,
-        },
-      );
-
-      let selectedDenom: SelResult | undefined;
-      if (req.forcedSelection) {
-        selectedDenom = selectForced(req, candidateDenoms);
-      } else {
-        // FIXME:  Here, we should select coins in a smarter way.
-        // Instead of always spending the next-largest coin,
-        // we should try to find the smallest coin that covers the
-        // amount.
-        selectedDenom = selectGreedy(
-          {
-            wireFeeAmortization: req.wireFeeAmortization,
-            wireFeesPerExchange: wireFeesPerExchange,
-          },
-          candidateDenoms,
-          tally,
+      if (!materialAvSel) {
+        const prospectiveAvSel = await internalSelectPayCoins(
+          wex,
+          tx,
+          req,
+          true,
         );
-      }
 
-      if (!selectedDenom) {
+        if (prospectiveAvSel) {
+          const prospectiveCoins: SelectedProspectiveCoin[] = [];
+          for (const avKey of Object.keys(prospectiveAvSel.sel)) {
+            const mySel = prospectiveAvSel.sel[avKey];
+            for (const contrib of mySel.contributions) {
+              prospectiveCoins.push({
+                denomPubHash: mySel.denomPubHash,
+                contribution: Amounts.stringify(contrib),
+                exchangeBaseUrl: mySel.exchangeBaseUrl,
+              });
+            }
+          }
+          return {
+            type: "prospective",
+            result: {
+              prospectiveCoins,
+              customerDepositFees: Amounts.stringify(
+                prospectiveAvSel.tally.customerDepositFees,
+              ),
+              customerWireFees: Amounts.stringify(
+                prospectiveAvSel.tally.customerWireFees,
+              ),
+            },
+          } satisfies SelectPayCoinsResult;
+        }
+
         return {
           type: "failure",
           insufficientBalanceDetails: await reportInsufficientBalanceDetails(
@@ -261,9 +331,9 @@ export async function selectPayCoins(
 
       const coinSel = await assembleSelectPayCoinsSuccessResult(
         tx,
-        selectedDenom,
-        coinRes,
-        tally,
+        materialAvSel.sel,
+        materialAvSel.coinRes,
+        materialAvSel.tally,
       );
 
       if (logger.shouldLogTrace()) {
@@ -285,7 +355,6 @@ async function maybeRepairCoinSelection(
   coinRes: SelectedCoin[],
   tally: CoinSelectionTally,
   feeInfo: {
-    wireFeeAmortization: number;
     wireFeesPerExchange: Record<string, AmountJson>;
   },
 ): Promise<void> {
@@ -307,7 +376,6 @@ async function maybeRepairCoinSelection(
     tallyFees(
       tally,
       feeInfo.wireFeesPerExchange,
-      feeInfo.wireFeeAmortization,
       coin.exchangeBaseUrl,
       Amounts.parseOrThrow(denom.feeDeposit),
     );
@@ -317,12 +385,18 @@ async function maybeRepairCoinSelection(
     ).amount;
 
     coinRes.push({
+      exchangeBaseUrl: coin.exchangeBaseUrl,
+      denomPubHash: coin.denomPubHash,
       coinPub: prev.coinPub,
       contribution: Amounts.stringify(prev.contribution),
     });
   }
 }
 
+/**
+ * Returns undefined if the success response could not be assembled,
+ * as not enough coins are actually available.
+ */
 async function assembleSelectPayCoinsSuccessResult(
   tx: WalletDbReadOnlyTransaction<["coins"]>,
   finalSel: SelResult,
@@ -352,8 +426,10 @@ async function assembleSelectPayCoinsSuccessResult(
 
     for (let i = 0; i < selInfo.contributions.length; i++) {
       coinRes.push({
+        denomPubHash: coins[i].denomPubHash,
         coinPub: coins[i].coinPub,
         contribution: Amounts.stringify(selInfo.contributions[i]),
+        exchangeBaseUrl: coins[i].exchangeBaseUrl,
       });
     }
   }
@@ -400,6 +476,16 @@ export async function reportInsufficientBalanceDetails(
     if (!exch.detailsPointer) {
       continue;
     }
+    let missingGlobalFees = false;
+    const exchWire = await getExchangeWireDetailsInTx(tx, exch.baseUrl);
+    if (!exchWire) {
+      missingGlobalFees = true;
+    } else {
+      const globalFees = getGlobalFees(exchWire);
+      if (!globalFees) {
+        missingGlobalFees = true;
+      }
+    }
     const exchDet = await getPaymentBalanceDetailsInTx(wex, tx, {
       restrictExchanges: {
         exchanges: [
@@ -431,6 +517,7 @@ export async function reportInsufficientBalanceDetails(
       maxEffectiveSpendAmount: Amounts.stringify(
         exchDet.maxEffectiveSpendAmount,
       ),
+      missingGlobalFees,
     };
   }
 
@@ -484,7 +571,6 @@ export function testing_selectGreedy(
 }
 
 export interface SelectGreedyRequest {
-  wireFeeAmortization: number;
   wireFeesPerExchange: Record<string, AmountJson>;
 }
 
@@ -493,7 +579,6 @@ function selectGreedy(
   candidateDenoms: AvailableDenom[],
   tally: CoinSelectionTally,
 ): SelResult | undefined {
-  const { wireFeeAmortization } = req;
   const selectedDenom: SelResult = {};
   for (const denom of candidateDenoms) {
     const contributions: AmountJson[] = [];
@@ -513,7 +598,6 @@ function selectGreedy(
       tallyFees(
         tally,
         req.wireFeesPerExchange,
-        wireFeeAmortization,
         denom.exchangeBaseUrl,
         Amounts.parseOrThrow(denom.feeDeposit),
       );
@@ -626,7 +710,6 @@ export interface SelectPayCoinRequestNg {
   restrictWireMethod: string;
   contractTermsAmount: AmountJson;
   depositFeeLimit: AmountJson;
-  wireFeeAmortization: number;
   prevPayCoins?: PreviousPayCoins;
   requiredMinimumAge?: number;
   forcedSelection?: ForcedCoinSel;
@@ -727,6 +810,13 @@ interface SelectPayCandidatesRequest {
   depositPaytoUri?: string;
   restrictExchanges: ExchangeRestrictionSpec | undefined;
   requiredMinimumAge?: number;
+
+  /**
+   * If set to true, the coin selection will also use coins that are not
+   * materially available yet, but that are expected to become available
+   * as the output of a refresh operation.
+   */
+  includePendingCoins: boolean;
 }
 
 async function selectPayCandidates(
@@ -738,6 +828,8 @@ async function selectPayCandidates(
 ): Promise<[AvailableDenom[], Record<string, AmountJson>]> {
   // FIXME: Use the existing helper (from balance.ts) to
   // get acceptable exchanges.
+  logger.shouldLogTrace() &&
+    logger.trace(`selecting available coin candidates for ${j2s(req)}`);
   const denoms: AvailableDenom[] = [];
   const exchanges = await tx.exchanges.iter().toArray();
   const wfPerExchange: Record<string, AmountJson> = {};
@@ -748,6 +840,8 @@ async function selectPayCandidates(
     );
     // 1. exchange has same currency
     if (exchangeDetails?.currency !== req.instructedAmount.currency) {
+      logger.shouldLogTrace() &&
+        logger.trace(`skipping ${exchange.baseUrl} due to currency mismatch`);
       continue;
     }
 
@@ -759,6 +853,11 @@ async function selectPayCandidates(
         exchangeDetails,
       );
       if (!wire) {
+        if (logger.shouldLogTrace()) {
+          logger.trace(
+            `skipping ${exchange.baseUrl} due to missing wire info mismatch`,
+          );
+        }
         continue;
       }
       wfPerExchange[exchange.baseUrl] = wire.wireFee;
@@ -770,6 +869,9 @@ async function selectPayCandidates(
       req.restrictExchanges,
     );
     if (!accepted) {
+      if (logger.shouldLogTrace()) {
+        logger.trace(`skipping ${exchange.baseUrl} due to unacceptability`);
+      }
       continue;
     }
 
@@ -788,6 +890,14 @@ async function selectPayCandidates(
         ),
       );
 
+    if (logger.shouldLogTrace()) {
+      logger.trace(
+        `exchange ${exchange.baseUrl} has ${myExchangeCoins.length} candidate records`,
+      );
+    }
+
+    let numUsable = 0;
+
     // 5. save denoms with how many coins are available
     // FIXME: Check that the individual denomination is audited!
     // FIXME: Should we exclude denominations that are
@@ -798,15 +908,31 @@ async function selectPayCandidates(
         coinAvail.denomPubHash,
       ]);
       checkDbInvariant(!!denom);
-      if (denom.isRevoked || !denom.isOffered) {
+      if (denom.isRevoked) {
+        logger.trace("denom is revoked");
         continue;
       }
+      if (!denom.isOffered) {
+        logger.trace("denom is unoffered");
+        continue;
+      }
+      numUsable++;
+      let numAvailable = coinAvail.freshCoinCount ?? 0;
+      if (req.includePendingCoins) {
+        numAvailable += coinAvail.pendingRefreshOutputCount ?? 0;
+      }
       denoms.push({
         ...DenominationRecord.toDenomInfo(denom),
-        numAvailable: coinAvail.freshCoinCount ?? 0,
+        numAvailable,
         maxAge: coinAvail.maxAge,
       });
     }
+
+    if (logger.shouldLogTrace()) {
+      logger.trace(
+        `exchange ${exchange.baseUrl} has ${numUsable} candidate records with usable denominations`,
+      );
+    }
   }
   // Sort by available amount (descending),  deposit fee (ascending) and
   // denomPub (ascending) if deposit fee is the same
@@ -836,8 +962,23 @@ export interface PeerCoinSelectionDetails {
   maxExpirationDate: TalerProtocolTimestamp;
 }
 
+export interface ProspectivePeerCoinSelectionDetails {
+  exchangeBaseUrl: string;
+
+  prospectiveCoins: SelectedProspectiveCoin[];
+
+  /**
+   * How much of the deposit fees is the customer paying?
+   */
+  depositFees: AmountJson;
+
+  maxExpirationDate: TalerProtocolTimestamp;
+}
+
 export type SelectPeerCoinsResult =
   | { type: "success"; result: PeerCoinSelectionDetails }
+  // Successful, but using coins that are not materially available yet.
+  | { type: "prospective"; result: ProspectivePeerCoinSelectionDetails }
   | {
       type: "failure";
       insufficientBalanceDetails: PaymentInsufficientBalanceDetails;
@@ -900,6 +1041,93 @@ export function emptyTallyForPeerPayment(
   };
 }
 
+function getGlobalFees(
+  wireDetails: ExchangeWireDetails,
+): ExchangeGlobalFees | undefined {
+  const now = AbsoluteTime.now();
+  for (let gf of wireDetails.globalFees) {
+    const isActive = AbsoluteTime.isBetween(
+      now,
+      AbsoluteTime.fromProtocolTimestamp(gf.startDate),
+      AbsoluteTime.fromProtocolTimestamp(gf.endDate),
+    );
+    if (!isActive) {
+      continue;
+    }
+    return gf;
+  }
+  return undefined;
+}
+
+async function internalSelectPeerCoins(
+  wex: WalletExecutionContext,
+  tx: WalletDbReadOnlyTransaction<
+    [
+      "exchanges",
+      "contractTerms",
+      "coins",
+      "coinAvailability",
+      "denominations",
+      "refreshGroups",
+      "exchangeDetails",
+    ]
+  >,
+  req: PeerCoinSelectionRequest,
+  exch: ExchangeWireDetails,
+  includePendingCoins: boolean,
+): Promise<
+  | { sel: SelResult; tally: CoinSelectionTally; resCoins: SelectedCoin[] }
+  | undefined
+> {
+  const candidatesRes = await selectPayCandidates(wex, tx, {
+    instructedAmount: req.instructedAmount,
+    restrictExchanges: {
+      auditors: [],
+      exchanges: [
+        {
+          exchangeBaseUrl: exch.exchangeBaseUrl,
+          exchangePub: exch.masterPublicKey,
+        },
+      ],
+    },
+    restrictWireMethod: undefined,
+    includePendingCoins,
+  });
+  const candidates = candidatesRes[0];
+  if (logger.shouldLogTrace()) {
+    logger.trace(`peer payment candidate coins: ${j2s(candidates)}`);
+  }
+  const tally = emptyTallyForPeerPayment(req.instructedAmount);
+  const resCoins: SelectedCoin[] = [];
+
+  await maybeRepairCoinSelection(wex, tx, req.repair ?? [], resCoins, tally, {
+    wireFeesPerExchange: {},
+  });
+
+  if (logger.shouldLogTrace()) {
+    logger.trace(`candidates: ${j2s(candidates)}`);
+    logger.trace(`instructedAmount: ${j2s(req.instructedAmount)}`);
+    logger.trace(`tally: ${j2s(tally)}`);
+  }
+
+  const selRes = selectGreedy(
+    {
+      wireFeesPerExchange: {},
+    },
+    candidates,
+    tally,
+  );
+  if (!selRes) {
+    return undefined;
+  }
+
+  return {
+    sel: selRes,
+    tally,
+    resCoins,
+  };
+}
+
 export async function selectPeerCoins(
   wex: WalletExecutionContext,
   req: PeerCoinSelectionRequest,
@@ -928,65 +1156,71 @@ export async function selectPeerCoins(
         if (exch.detailsPointer?.currency !== currency) {
           continue;
         }
-        const candidatesRes = await selectPayCandidates(wex, tx, {
-          instructedAmount,
-          restrictExchanges: {
-            auditors: [],
-            exchanges: [
-              {
-                exchangeBaseUrl: exch.baseUrl,
-                exchangePub: exch.detailsPointer.masterPublicKey,
-              },
-            ],
-          },
-          restrictWireMethod: undefined,
-        });
-        const candidates = candidatesRes[0];
-        if (logger.shouldLogTrace()) {
-          logger.trace(`peer payment candidate coins: ${j2s(candidates)}`);
+        const exchWire = await getExchangeWireDetailsInTx(tx, exch.baseUrl);
+        if (!exchWire) {
+          continue;
+        }
+        const globalFees = getGlobalFees(exchWire);
+        if (!globalFees) {
+          continue;
         }
-        const tally = emptyTallyForPeerPayment(req.instructedAmount);
-        const resCoins: SelectedCoin[] = [];
 
-        await maybeRepairCoinSelection(
+        const avRes = await internalSelectPeerCoins(
           wex,
           tx,
-          req.repair ?? [],
-          resCoins,
-          tally,
-          {
-            wireFeeAmortization: 1,
-            wireFeesPerExchange: {},
-          },
+          req,
+          exchWire,
+          false,
         );
 
-        if (logger.shouldLogTrace()) {
-          logger.trace(`candidates: ${j2s(candidates)}`);
-          logger.trace(`instructedAmount: ${j2s(instructedAmount)}`);
-          logger.trace(`tally: ${j2s(tally)}`);
-        }
-
-        const selectedDenom = selectGreedy(
-          {
-            wireFeeAmortization: 1,
-            wireFeesPerExchange: {},
-          },
-          candidates,
-          tally,
-        );
-
-        if (selectedDenom) {
+        if (!avRes) {
+          // Try to see if we can do a prospective selection
+          const prospectiveAvRes = await internalSelectPeerCoins(
+            wex,
+            tx,
+            req,
+            exchWire,
+            true,
+          );
+          if (prospectiveAvRes) {
+            const prospectiveCoins: SelectedProspectiveCoin[] = [];
+            for (const avKey of Object.keys(prospectiveAvRes.sel)) {
+              const mySel = prospectiveAvRes.sel[avKey];
+              for (const contrib of mySel.contributions) {
+                prospectiveCoins.push({
+                  denomPubHash: mySel.denomPubHash,
+                  contribution: Amounts.stringify(contrib),
+                  exchangeBaseUrl: mySel.exchangeBaseUrl,
+                });
+              }
+            }
+            const maxExpirationDate = await computeCoinSelMaxExpirationDate(
+              wex,
+              tx,
+              prospectiveAvRes.sel,
+            );
+            return {
+              type: "prospective",
+              result: {
+                prospectiveCoins,
+                depositFees: prospectiveAvRes.tally.customerDepositFees,
+                exchangeBaseUrl: exch.baseUrl,
+                maxExpirationDate,
+              },
+            };
+          }
+        } else if (avRes) {
           const r = await assembleSelectPayCoinsSuccessResult(
             tx,
-            selectedDenom,
-            resCoins,
-            tally,
+            avRes.sel,
+            avRes.resCoins,
+            avRes.tally,
           );
 
           const maxExpirationDate = await computeCoinSelMaxExpirationDate(
             wex,
             tx,
-            selectedDenom,
+            avRes.sel,
           );
 
           return {
