@@ -61,11 +61,7 @@ import {
   timestampPreciseToDb,
 } from "./db.js";
 import { createRefreshGroup } from "./refresh.js";
-import {
-  InternalWalletState,
-  WalletExecutionContext,
-  getDenomInfo,
-} from "./wallet.js";
+import { WalletExecutionContext, getDenomInfo } from "./wallet.js";
 
 const logger = new Logger("operations/common.ts");
 
@@ -149,7 +145,13 @@ export async function makeCoinAvailable(
 export async function spendCoins(
   wex: WalletExecutionContext,
   tx: WalletDbReadWriteTransaction<
-    ["coins", "coinAvailability", "refreshGroups", "denominations"]
+    [
+      "coins",
+      "coinAvailability",
+      "refreshGroups",
+      "refreshSessions",
+      "denominations",
+    ]
   >,
   csi: CoinsSpendInfo,
 ): Promise<void> {
@@ -279,6 +281,8 @@ export function getExchangeUpdateStatusFromRecord(
       return ExchangeUpdateStatus.ReadyUpdate;
     case ExchangeEntryDbUpdateStatus.Suspended:
       return ExchangeUpdateStatus.Suspended;
+    default:
+      assertUnreachable(r.updateStatus);
   }
 }
 
@@ -292,6 +296,8 @@ export function getExchangeEntryStatusFromRecord(
       return ExchangeEntryStatus.Preset;
     case ExchangeEntryDbRecordStatus.Used:
       return ExchangeEntryStatus.Used;
+    default:
+      assertUnreachable(r.entryStatus);
   }
 }
 
@@ -484,25 +490,28 @@ function updateTimeout(
   r.nextRetry = timestampPreciseToDb(TalerPreciseTimestamp.fromMilliseconds(t));
 }
 
-export namespace DbRetryInfo {
-  export function getDuration(
-    r: DbRetryInfo | undefined,
-    p: RetryPolicy = defaultRetryPolicy,
-  ): Duration {
-    if (!r) {
-      // If we don't have any retry info, run immediately.
-      return { d_ms: 0 };
-    }
-    if (p.backoffDelta.d_ms === "forever") {
-      return { d_ms: "forever" };
-    }
-    const t = p.backoffDelta.d_ms * Math.pow(p.backoffBase, r.retryCounter);
-    return {
-      d_ms:
-        p.maxTimeout.d_ms === "forever" ? t : Math.min(p.maxTimeout.d_ms, t),
-    };
+export function computeDbBackoff(retryCounter: number): DbPreciseTimestamp {
+  const now = AbsoluteTime.now();
+  if (now.t_ms === "never") {
+    throw Error("assertion failed");
+  }
+  const p = defaultRetryPolicy;
+  if (p.backoffDelta.d_ms === "forever") {
+    throw Error("assertion failed");
   }
 
+  const nextIncrement =
+    p.backoffDelta.d_ms * Math.pow(p.backoffBase, retryCounter);
+
+  const t =
+    now.t_ms +
+    (p.maxTimeout.d_ms === "forever"
+      ? nextIncrement
+      : Math.min(p.maxTimeout.d_ms, nextIncrement));
+  return timestampPreciseToDb(TalerPreciseTimestamp.fromMilliseconds(t));
+}
+
+export namespace DbRetryInfo {
   export function reset(p: RetryPolicy = defaultRetryPolicy): DbRetryInfo {
     const now = TalerPreciseTimestamp.now();
     const info: DbRetryInfo = {
@@ -696,11 +705,32 @@ export namespace TaskIdentifiers {
 /**
  * Result of a transaction transition.
  */
-export enum TransitionResult {
+export enum TransitionResultType {
   Transition = 1,
   Stay = 2,
+  Delete = 3,
 }
 
+export type TransitionResult<R> =
+  | { type: TransitionResultType.Stay }
+  | { type: TransitionResultType.Transition; rec: R }
+  | { type: TransitionResultType.Delete };
+
+export const TransitionResult = {
+  stay<T>(): TransitionResult<T> {
+    return { type: TransitionResultType.Stay };
+  },
+  delete<T>(): TransitionResult<T> {
+    return { type: TransitionResultType.Delete };
+  },
+  transition<T>(rec: T): TransitionResult<T> {
+    return {
+      type: TransitionResultType.Transition,
+      rec,
+    };
+  },
+};
+
 /**
  * Transaction context.
  * Uniform interface to all transactions.
