@@ -1134,7 +1134,7 @@ export const nativeCryptoR: TalerCryptoInterfaceR = {
         depositInfo.ageCommitmentProof.commitment,
       );
       hAgeCommitment = decodeCrock(ach);
-      if (depositInfo.requiredMinimumAge != null) {
+      if (depositInfo.requiredMinimumAge) {
         minimumAgeSig = encodeCrock(
           AgeRestriction.commitmentAttest(
             depositInfo.ageCommitmentProof,
@@ -1184,7 +1184,7 @@ export const nativeCryptoR: TalerCryptoInterfaceR = {
         },
       };
 
-      if (depositInfo.requiredMinimumAge != null) {
+      if (depositInfo.requiredMinimumAge) {
         // These are only required by the merchant
         s.minimum_age_sig = minimumAgeSig;
         s.age_commitment =
