@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2021-2022 Taler Systems S.A.
+ (C) 2021-2024 Taler Systems S.A.
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -39,6 +39,7 @@ import {
   CoinPublicKeyString,
   CoinRefreshRequest,
   CoinStatus,
+  DenomLossEventType,
   DenomSelectionState,
   DenominationInfo,
   DenominationPubKey,
@@ -53,6 +54,7 @@ import {
   TalerPreciseTimestamp,
   TalerProtocolDuration,
   TalerProtocolTimestamp,
+  Transaction,
   TransactionIdStr,
   UnblindedSignature,
   WireInfo,
@@ -148,7 +150,7 @@ export const CURRENT_DB_CONFIG_KEY = "currentMainDbName";
  * backwards-compatible way or object stores and indices
  * are added.
  */
-export const WALLET_DB_MINOR_VERSION = 6;
+export const WALLET_DB_MINOR_VERSION = 10;
 
 declare const symDbProtocolTimestamp: unique symbol;
 
@@ -502,12 +504,6 @@ export interface DenominationRecord {
    * on the denomination.
    */
   exchangeMasterPub: string;
-
-  /**
-   * Latest list issue date of the "/keys" response
-   * that includes this denomination.
-   */
-  listIssueDate: DbProtocolTimestamp;
 }
 
 export namespace DenominationRecord {
@@ -678,6 +674,8 @@ export interface ExchangeEntryRecord {
    */
   nextUpdateStamp: DbPreciseTimestamp;
 
+  updateRetryCounter?: number;
+
   lastKeysEtag: string | undefined;
 
   /**
@@ -693,12 +691,23 @@ export interface ExchangeEntryRecord {
    * receiving P2P payments.
    */
   currentMergeReserveRowId?: number;
+
+  /**
+   * Defaults to false.
+   */
+  peerPaymentsDisabled?: boolean;
+
+  /**
+   * Defaults to false.
+   */
+  noFees?: boolean;
 }
 
 export enum PlanchetStatus {
   Pending = 0x0100_0000,
   KycRequired = 0x0100_0001,
   WithdrawalDone = 0x0500_000,
+  AbortedReplaced = 0x0503_0001,
 }
 
 /**
@@ -1151,7 +1160,7 @@ export enum PurchaseStatus {
   SuspendedQueryingAutoRefund = 0x0110_0004,
 
   PendingAcceptRefund = 0x0100_0005,
-  SuspendedPendingAcceptRefund = 0x0100_0005,
+  SuspendedPendingAcceptRefund = 0x0110_0005,
 
   /**
    * Proposal downloaded, but the user needs to accept/reject it.
@@ -1219,9 +1228,15 @@ export interface DbCoinSelection {
 }
 
 export interface PurchasePayInfo {
-  payCoinSelection: DbCoinSelection;
+  /**
+   * Undefined if payment is blocked by a pending refund.
+   */
+  payCoinSelection?: DbCoinSelection;
+  /**
+   * Undefined if payment is blocked by a pending refund.
+   */
+  payCoinSelectionUid?: string;
   totalPayCost: AmountString;
-  payCoinSelectionUid: string;
 }
 
 /**
@@ -1360,7 +1375,6 @@ export type ConfigRecord =
       value: WalletBackupConfState;
     }
   | { key: ConfigRecordKey.CurrencyDefaultsApplied; value: boolean }
-  | { key: ConfigRecordKey.DevMode; value: boolean }
   | { key: ConfigRecordKey.TestLoopTx; value: number }
   | { key: ConfigRecordKey.LastInitInfo; value: DbProtocolTimestamp };
 
@@ -1780,9 +1794,9 @@ export interface DepositGroupRecord {
 
   contractTermsHash: string;
 
-  payCoinSelection: DbCoinSelection;
+  payCoinSelection?: DbCoinSelection;
 
-  payCoinSelectionUid: string;
+  payCoinSelectionUid?: string;
 
   totalPayCost: AmountString;
 
@@ -1797,7 +1811,7 @@ export interface DepositGroupRecord {
 
   operationStatus: DepositOperationStatus;
 
-  statusPerCoin: DepositElementStatus[];
+  statusPerCoin?: DepositElementStatus[];
 
   infoPerExchange?: Record<string, DepositInfoPerExchange>;
 
@@ -1879,7 +1893,7 @@ export interface PeerPushDebitRecord {
 
   totalCost: AmountString;
 
-  coinSel: DbPeerPushPaymentCoinSelection;
+  coinSel?: DbPeerPushPaymentCoinSelection;
 
   contractTermsHash: HashCodeString;
 
@@ -2191,6 +2205,11 @@ export interface CoinAvailabilityRecord {
    * a final state.
    */
   visibleCoinCount: number;
+
+  /**
+   * Number of coins that we expect to obtain via a pending refresh.
+   */
+  pendingRefreshOutputCount?: number;
 }
 
 export interface ContractTermsRecord {
@@ -2332,11 +2351,80 @@ export interface GlobalCurrencyExchangeRecord {
   exchangeMasterPub: string;
 }
 
+/**
+ * Primary key: transactionItem.transactionId
+ */
+export interface TransactionRecord {
+  /**
+   * Transaction item returned to the client.
+   */
+  transactionItem: Transaction;
+
+  /**
+   * Exchanges involved in the transaction.
+   */
+  exchanges: string[];
+
+  currency: string;
+}
+
+export enum DenomLossStatus {
+  /**
+   * Done indicates that the loss happened.
+   */
+  Done = 0x0500_0000,
+
+  /**
+   * Aborted in the sense that the loss was reversed.
+   */
+  Aborted = 0x0503_0001,
+}
+
+export interface DenomLossEventRecord {
+  denomLossEventId: string;
+  currency: string;
+  denomPubHashes: string[];
+  status: DenomLossStatus;
+  timestampCreated: DbPreciseTimestamp;
+  amount: string;
+  eventType: DenomLossEventType;
+  exchangeBaseUrl: string;
+}
+
 /**
  * Schema definition for the IndexedDB
  * wallet database.
  */
 export const WalletStoresV1 = {
+  denomLossEvents: describeStoreV2({
+    recordCodec: passthroughCodec<DenomLossEventRecord>(),
+    storeName: "denomLossEvents",
+    keyPath: "denomLossEventId",
+    versionAdded: 9,
+    indexes: {
+      byCurrency: describeIndex("byCurrency", "currency", {
+        versionAdded: 9,
+      }),
+      byStatus: describeIndex("byStatus", "status", {
+        versionAdded: 10,
+      }),
+    },
+  }),
+  transactions: describeStoreV2({
+    recordCodec: passthroughCodec<TransactionRecord>(),
+    storeName: "transactions",
+    keyPath: "transactionItem.transactionId",
+    versionAdded: 7,
+    indexes: {
+      byCurrency: describeIndex("byCurrency", "currency", {
+        versionAdded: 7,
+      }),
+      byExchange: describeIndex("byExchange", "exchanges", {
+        versionAdded: 7,
+        multiEntry: true,
+      }),
+    },
+  }),
   globalCurrencyAuditors: describeStoreV2({
     recordCodec: passthroughCodec<GlobalCurrencyAuditorRecord>(),
     storeName: "globalCurrencyAuditors",
@@ -2382,6 +2470,9 @@ export const WalletStoresV1 = {
         "maxAge",
         "freshCoinCount",
       ]),
+      byExchangeBaseUrl: describeIndex("byExchangeBaseUrl", "exchangeBaseUrl", {
+        versionAdded: 8,
+      }),
     },
   ),
   coins: describeStore(
@@ -2726,22 +2817,22 @@ export const WalletStoresV1 = {
   ),
 };
 
-export type WalletDbReadWriteTransaction<
-  StoresArr extends Array<StoreNames<typeof WalletStoresV1>>,
-> = DbReadWriteTransaction<typeof WalletStoresV1, StoresArr>;
+export type WalletDbStoresArr = Array<StoreNames<typeof WalletStoresV1>>;
 
-export type WalletDbReadOnlyTransaction<
-  StoresArr extends Array<StoreNames<typeof WalletStoresV1>>,
-> = DbReadOnlyTransaction<typeof WalletStoresV1, StoresArr>;
+export type WalletDbReadWriteTransaction<StoresArr extends WalletDbStoresArr> =
+  DbReadWriteTransaction<typeof WalletStoresV1, StoresArr>;
+
+export type WalletDbReadOnlyTransaction<StoresArr extends WalletDbStoresArr> =
+  DbReadOnlyTransaction<typeof WalletStoresV1, StoresArr>;
 
 export type WalletDbAllStoresReadOnlyTransaction<> = DbReadOnlyTransaction<
   typeof WalletStoresV1,
-  Array<StoreNames<typeof WalletStoresV1>>
+  WalletDbStoresArr
 >;
 
 export type WalletDbAllStoresReadWriteTransaction<> = DbReadWriteTransaction<
   typeof WalletStoresV1,
-  Array<StoreNames<typeof WalletStoresV1>>
+  WalletDbStoresArr
 >;
 
 /**
@@ -2971,7 +3062,7 @@ export async function applyFixups(
   db: DbAccess<typeof WalletStoresV1>,
 ): Promise<void> {
   logger.trace("applying fixups");
-  await db.runAllStoresReadWriteTx(async (tx) => {
+  await db.runAllStoresReadWriteTx({}, async (tx) => {
     for (const fixupInstruction of walletDbFixups) {
       logger.trace(`checking fixup ${fixupInstruction.name}`);
       const fixupRecord = await tx.fixups.get(fixupInstruction.name);
