@@ -33,15 +33,16 @@ import {
   DepositGroupFees,
   Duration,
   ExchangeBatchDepositRequest,
+  ExchangeHandle,
   ExchangeRefundRequest,
   HttpStatusCode,
   Logger,
   MerchantContractTerms,
   NotificationType,
-  PayCoinSelection,
   PrepareDepositRequest,
   PrepareDepositResponse,
   RefreshReason,
+  SelectedProspectiveCoin,
   TalerError,
   TalerErrorCode,
   TalerPreciseTimestamp,
@@ -109,7 +110,6 @@ import {
   parseTransactionIdentifier,
 } from "./transactions.js";
 import { WalletExecutionContext, getDenomInfo } from "./wallet.js";
-import { getCandidateWithdrawalDenomsTx } from "./withdraw.js";
 
 /**
  * Logger.
@@ -412,16 +412,28 @@ async function refundDepositGroup(
   wex: WalletExecutionContext,
   depositGroup: DepositGroupRecord,
 ): Promise<TaskRunResult> {
-  const newTxPerCoin = [...depositGroup.statusPerCoin];
+  const statusPerCoin = depositGroup.statusPerCoin;
+  const payCoinSelection = depositGroup.payCoinSelection;
+  if (!statusPerCoin) {
+    throw Error(
+      "unable to refund deposit group without coin selection (status missing)",
+    );
+  }
+  if (!payCoinSelection) {
+    throw Error(
+      "unable to refund deposit group without coin selection (selection missing)",
+    );
+  }
+  const newTxPerCoin = [...statusPerCoin];
   logger.info(`status per coin: ${j2s(depositGroup.statusPerCoin)}`);
-  for (let i = 0; i < depositGroup.statusPerCoin.length; i++) {
-    const st = depositGroup.statusPerCoin[i];
+  for (let i = 0; i < statusPerCoin.length; i++) {
+    const st = statusPerCoin[i];
     switch (st) {
       case DepositElementStatus.RefundFailed:
       case DepositElementStatus.RefundSuccess:
         break;
       default: {
-        const coinPub = depositGroup.payCoinSelection.coinPubs[i];
+        const coinPub = payCoinSelection.coinPubs[i];
         const coinExchange = await wex.db.runReadOnlyTx(
           ["coins"],
           async (tx) => {
@@ -430,7 +442,7 @@ async function refundDepositGroup(
             return coinRecord.exchangeBaseUrl;
           },
         );
-        const refundAmount = depositGroup.payCoinSelection.coinContributions[i];
+        const refundAmount = payCoinSelection.coinContributions[i];
         // We use a constant refund transaction ID, since there can
         // only be one refund.
         const rtid = 1;
@@ -488,6 +500,7 @@ async function refundDepositGroup(
     [
       "depositGroups",
       "refreshGroups",
+      "refreshSessions",
       "coins",
       "denominations",
       "coinAvailability",
@@ -501,8 +514,8 @@ async function refundDepositGroup(
       const refreshCoins: CoinRefreshRequest[] = [];
       for (let i = 0; i < newTxPerCoin.length; i++) {
         refreshCoins.push({
-          amount: depositGroup.payCoinSelection.coinContributions[i],
-          coinPub: depositGroup.payCoinSelection.coinPubs[i],
+          amount: payCoinSelection.coinContributions[i],
+          coinPub: payCoinSelection.coinPubs[i],
         });
       }
       let refreshRes: CreateRefreshGroupResult | undefined = undefined;
@@ -738,9 +751,21 @@ async function processDepositGroupPendingTrack(
   wex: WalletExecutionContext,
   depositGroup: DepositGroupRecord,
 ): Promise<TaskRunResult> {
+  const statusPerCoin = depositGroup.statusPerCoin;
+  const payCoinSelection = depositGroup.payCoinSelection;
+  if (!statusPerCoin) {
+    throw Error(
+      "unable to refund deposit group without coin selection (status missing)",
+    );
+  }
+  if (!payCoinSelection) {
+    throw Error(
+      "unable to refund deposit group without coin selection (selection missing)",
+    );
+  }
   const { depositGroupId } = depositGroup;
-  for (let i = 0; i < depositGroup.statusPerCoin.length; i++) {
-    const coinPub = depositGroup.payCoinSelection.coinPubs[i];
+  for (let i = 0; i < statusPerCoin.length; i++) {
+    const coinPub = payCoinSelection.coinPubs[i];
     // FIXME: Make the URL part of the coin selection?
     const exchangeBaseUrl = await wex.db.runReadWriteTx(
       ["coins"],
@@ -759,7 +784,7 @@ async function processDepositGroupPendingTrack(
         }
       | undefined;
 
-    if (depositGroup.statusPerCoin[i] !== DepositElementStatus.Wired) {
+    if (statusPerCoin[i] !== DepositElementStatus.Wired) {
       const track = await trackDeposit(
         wex,
         depositGroup,
@@ -824,6 +849,9 @@ async function processDepositGroupPendingTrack(
         if (!dg) {
           return;
         }
+        if (!dg.statusPerCoin) {
+          return;
+        }
         if (updatedTxStatus !== undefined) {
           dg.statusPerCoin[i] = updatedTxStatus;
         }
@@ -856,9 +884,12 @@ async function processDepositGroupPendingTrack(
       if (!dg) {
         return undefined;
       }
+      if (!dg.statusPerCoin) {
+        return undefined;
+      }
       const oldTxState = computeDepositTransactionStatus(dg);
-      for (let i = 0; i < depositGroup.statusPerCoin.length; i++) {
-        if (depositGroup.statusPerCoin[i] !== DepositElementStatus.Wired) {
+      for (let i = 0; i < dg.statusPerCoin.length; i++) {
+        if (dg.statusPerCoin[i] !== DepositElementStatus.Wired) {
           allWired = false;
           break;
         }
@@ -922,6 +953,86 @@ async function processDepositGroupPendingDeposit(
   // Check for cancellation before expensive operations.
   cancellationToken?.throwIfCancelled();
 
+  if (!depositGroup.payCoinSelection) {
+    logger.info("missing coin selection for deposit group, selecting now");
+    // FIXME: Consider doing the coin selection inside the txn
+    const payCoinSel = await selectPayCoins(wex, {
+      restrictExchanges: {
+        auditors: [],
+        exchanges: contractData.allowedExchanges,
+      },
+      restrictWireMethod: contractData.wireMethod,
+      contractTermsAmount: Amounts.parseOrThrow(contractData.amount),
+      depositFeeLimit: Amounts.parseOrThrow(contractData.maxDepositFee),
+      prevPayCoins: [],
+    });
+
+    switch (payCoinSel.type) {
+      case "success":
+        logger.info("coin selection success");
+        break;
+      case "failure":
+        logger.info("coin selection failure");
+        throw TalerError.fromDetail(
+          TalerErrorCode.WALLET_DEPOSIT_GROUP_INSUFFICIENT_BALANCE,
+          {
+            insufficientBalanceDetails: payCoinSel.insufficientBalanceDetails,
+          },
+        );
+      case "prospective":
+        logger.info("coin selection prospective");
+        throw Error("insufficient balance (waiting on pending refresh)");
+      default:
+        assertUnreachable(payCoinSel);
+    }
+
+    const transitionDone = await wex.db.runReadWriteTx(
+      [
+        "depositGroups",
+        "coins",
+        "coinAvailability",
+        "refreshGroups",
+        "refreshSessions",
+        "denominations",
+      ],
+      async (tx) => {
+        const dg = await tx.depositGroups.get(depositGroupId);
+        if (!dg) {
+          return false;
+        }
+        if (dg.statusPerCoin) {
+          return false;
+        }
+        dg.payCoinSelection = {
+          coinContributions: payCoinSel.coinSel.coins.map(
+            (x) => x.contribution,
+          ),
+          coinPubs: payCoinSel.coinSel.coins.map((x) => x.coinPub),
+        };
+        dg.payCoinSelectionUid = encodeCrock(getRandomBytes(32));
+        dg.statusPerCoin = payCoinSel.coinSel.coins.map(
+          () => DepositElementStatus.DepositPending,
+        );
+        await tx.depositGroups.put(dg);
+        await spendCoins(wex, tx, {
+          allocationId: transactionId,
+          coinPubs: dg.payCoinSelection.coinPubs,
+          contributions: dg.payCoinSelection.coinContributions.map((x) =>
+            Amounts.parseOrThrow(x),
+          ),
+          refreshReason: RefreshReason.PayDeposit,
+        });
+        return true;
+      },
+    );
+
+    if (transitionDone) {
+      return TaskRunResult.progress();
+    } else {
+      return TaskRunResult.backoff();
+    }
+  }
+
   // FIXME: Cache these!
   const depositPermissions = await generateDepositPermissions(
     wex,
@@ -988,6 +1099,9 @@ async function processDepositGroupPendingDeposit(
       if (!dg) {
         return;
       }
+      if (!dg.statusPerCoin) {
+        return;
+      }
       for (const batchIndex of batchIndexes) {
         const coinStatus = dg.statusPerCoin[batchIndex];
         switch (coinStatus) {
@@ -1153,11 +1267,8 @@ async function trackDeposit(
 /**
  * Check if creating a deposit group is possible and calculate
  * the associated fees.
- *
- * FIXME: This should be renamed to checkDepositGroup,
- * as it doesn't prepare anything
  */
-export async function prepareDepositGroup(
+export async function checkDepositGroup(
   wex: WalletExecutionContext,
   req: PrepareDepositRequest,
 ): Promise<PrepareDepositResponse> {
@@ -1166,8 +1277,9 @@ export async function prepareDepositGroup(
     throw Error("invalid payto URI");
   }
   const amount = Amounts.parseOrThrow(req.amount);
+  const currency = Amounts.currencyOf(amount);
 
-  const exchangeInfos: { url: string; master_pub: string }[] = [];
+  const exchangeInfos: ExchangeHandle[] = [];
 
   await wex.db.runReadOnlyTx(["exchangeDetails", "exchanges"], async (tx) => {
     const allExchanges = await tx.exchanges.iter().toArray();
@@ -1189,7 +1301,6 @@ export async function prepareDepositGroup(
     exchanges: exchangeInfos,
     amount: req.amount,
     max_fee: Amounts.stringify(amount),
-    max_wire_fee: Amounts.stringify(amount),
     wire_method: p.targetType,
     timestamp: nowRounded,
     merchant_base_url: "",
@@ -1226,32 +1337,42 @@ export async function prepareDepositGroup(
     restrictWireMethod: contractData.wireMethod,
     contractTermsAmount: Amounts.parseOrThrow(contractData.amount),
     depositFeeLimit: Amounts.parseOrThrow(contractData.maxDepositFee),
-    wireFeeAmortization: contractData.wireFeeAmortization ?? 1,
     prevPayCoins: [],
   });
 
-  if (payCoinSel.type !== "success") {
-    throw TalerError.fromDetail(
-      TalerErrorCode.WALLET_DEPOSIT_GROUP_INSUFFICIENT_BALANCE,
-      {
-        insufficientBalanceDetails: payCoinSel.insufficientBalanceDetails,
-      },
-    );
+  let selCoins: SelectedProspectiveCoin[] | undefined = undefined;
+
+  switch (payCoinSel.type) {
+    case "failure":
+      throw TalerError.fromDetail(
+        TalerErrorCode.WALLET_DEPOSIT_GROUP_INSUFFICIENT_BALANCE,
+        {
+          insufficientBalanceDetails: payCoinSel.insufficientBalanceDetails,
+        },
+      );
+    case "prospective":
+      selCoins = payCoinSel.result.prospectiveCoins;
+      break;
+    case "success":
+      selCoins = payCoinSel.coinSel.coins;
+      break;
+    default:
+      assertUnreachable(payCoinSel);
   }
 
-  const totalDepositCost = await getTotalPaymentCost(wex, payCoinSel.coinSel);
+  const totalDepositCost = await getTotalPaymentCost(wex, currency, selCoins);
 
   const effectiveDepositAmount = await getCounterpartyEffectiveDepositAmount(
     wex,
     p.targetType,
-    payCoinSel.coinSel,
+    selCoins,
   );
 
   const fees = await getTotalFeesForDepositAmount(
     wex,
     p.targetType,
     amount,
-    payCoinSel.coinSel,
+    selCoins,
   );
 
   return {
@@ -1279,6 +1400,7 @@ export async function createDepositGroup(
   }
 
   const amount = Amounts.parseOrThrow(req.amount);
+  const currency = amount.currency;
 
   const exchangeInfos: { url: string; master_pub: string }[] = [];
 
@@ -1309,7 +1431,6 @@ export async function createDepositGroup(
     exchanges: exchangeInfos,
     amount: req.amount,
     max_fee: Amounts.stringify(amount),
-    max_wire_fee: Amounts.stringify(amount),
     wire_method: p.targetType,
     timestamp: nowRounded,
     merchant_base_url: "",
@@ -1346,20 +1467,30 @@ export async function createDepositGroup(
     restrictWireMethod: contractData.wireMethod,
     contractTermsAmount: Amounts.parseOrThrow(contractData.amount),
     depositFeeLimit: Amounts.parseOrThrow(contractData.maxDepositFee),
-    wireFeeAmortization: contractData.wireFeeAmortization ?? 1,
     prevPayCoins: [],
   });
 
-  if (payCoinSel.type !== "success") {
-    throw TalerError.fromDetail(
-      TalerErrorCode.WALLET_DEPOSIT_GROUP_INSUFFICIENT_BALANCE,
-      {
-        insufficientBalanceDetails: payCoinSel.insufficientBalanceDetails,
-      },
-    );
+  let coins: SelectedProspectiveCoin[] | undefined = undefined;
+
+  switch (payCoinSel.type) {
+    case "success":
+      coins = payCoinSel.coinSel.coins;
+      break;
+    case "failure":
+      throw TalerError.fromDetail(
+        TalerErrorCode.WALLET_DEPOSIT_GROUP_INSUFFICIENT_BALANCE,
+        {
+          insufficientBalanceDetails: payCoinSel.insufficientBalanceDetails,
+        },
+      );
+    case "prospective":
+      coins = payCoinSel.result.prospectiveCoins;
+      break;
+    default:
+      assertUnreachable(payCoinSel);
   }
 
-  const totalDepositCost = await getTotalPaymentCost(wex, payCoinSel.coinSel);
+  const totalDepositCost = await getTotalPaymentCost(wex, currency, coins);
 
   let depositGroupId: string;
   if (req.transactionId) {
@@ -1374,34 +1505,23 @@ export async function createDepositGroup(
 
   const infoPerExchange: Record<string, DepositInfoPerExchange> = {};
 
-  await wex.db.runReadOnlyTx(["coins"], async (tx) => {
-    for (let i = 0; i < payCoinSel.coinSel.coins.length; i++) {
-      const coin = await tx.coins.get(payCoinSel.coinSel.coins[i].coinPub);
-      if (!coin) {
-        logger.error("coin not found anymore");
-        continue;
-      }
-      let depPerExchange = infoPerExchange[coin.exchangeBaseUrl];
-      if (!depPerExchange) {
-        infoPerExchange[coin.exchangeBaseUrl] = depPerExchange = {
-          amountEffective: Amounts.stringify(
-            Amounts.zeroOfAmount(totalDepositCost),
-          ),
-        };
-      }
-      const contrib = payCoinSel.coinSel.coins[i].contribution;
-      depPerExchange.amountEffective = Amounts.stringify(
-        Amounts.add(depPerExchange.amountEffective, contrib).amount,
-      );
+  for (let i = 0; i < coins.length; i++) {
+    let depPerExchange = infoPerExchange[coins[i].exchangeBaseUrl];
+    if (!depPerExchange) {
+      infoPerExchange[coins[i].exchangeBaseUrl] = depPerExchange = {
+        amountEffective: Amounts.stringify(
+          Amounts.zeroOfAmount(totalDepositCost),
+        ),
+      };
     }
-  });
+    const contrib = coins[i].contribution;
+    depPerExchange.amountEffective = Amounts.stringify(
+      Amounts.add(depPerExchange.amountEffective, contrib).amount,
+    );
+  }
 
   const counterpartyEffectiveDepositAmount =
-    await getCounterpartyEffectiveDepositAmount(
-      wex,
-      p.targetType,
-      payCoinSel.coinSel,
-    );
+    await getCounterpartyEffectiveDepositAmount(wex, p.targetType, coins);
 
   const depositGroup: DepositGroupRecord = {
     contractTermsHash,
@@ -1414,14 +1534,9 @@ export async function createDepositGroup(
       AbsoluteTime.toPreciseTimestamp(now),
     ),
     timestampFinished: undefined,
-    statusPerCoin: payCoinSel.coinSel.coins.map(
-      () => DepositElementStatus.DepositPending,
-    ),
-    payCoinSelection: {
-      coinContributions: payCoinSel.coinSel.coins.map((x) => x.contribution),
-      coinPubs: payCoinSel.coinSel.coins.map((x) => x.coinPub),
-    },
-    payCoinSelectionUid: encodeCrock(getRandomBytes(32)),
+    statusPerCoin: undefined,
+    payCoinSelection: undefined,
+    payCoinSelectionUid: undefined,
     merchantPriv: merchantPair.priv,
     merchantPub: merchantPair.pub,
     totalPayCost: Amounts.stringify(totalDepositCost),
@@ -1439,6 +1554,17 @@ export async function createDepositGroup(
     infoPerExchange,
   };
 
+  if (payCoinSel.type === "success") {
+    depositGroup.payCoinSelection = {
+      coinContributions: payCoinSel.coinSel.coins.map((x) => x.contribution),
+      coinPubs: payCoinSel.coinSel.coins.map((x) => x.coinPub),
+    };
+    depositGroup.payCoinSelectionUid = encodeCrock(getRandomBytes(32));
+    depositGroup.statusPerCoin = payCoinSel.coinSel.coins.map(
+      () => DepositElementStatus.DepositPending,
+    );
+  }
+
   const ctx = new DepositTransactionContext(wex, depositGroupId);
   const transactionId = ctx.transactionId;
 
@@ -1449,18 +1575,21 @@ export async function createDepositGroup(
       "recoupGroups",
       "denominations",
       "refreshGroups",
+      "refreshSessions",
       "coinAvailability",
       "contractTerms",
     ],
     async (tx) => {
-      await spendCoins(wex, tx, {
-        allocationId: transactionId,
-        coinPubs: payCoinSel.coinSel.coins.map((x) => x.coinPub),
-        contributions: payCoinSel.coinSel.coins.map((x) =>
-          Amounts.parseOrThrow(x.contribution),
-        ),
-        refreshReason: RefreshReason.PayDeposit,
-      });
+      if (depositGroup.payCoinSelection) {
+        await spendCoins(wex, tx, {
+          allocationId: transactionId,
+          coinPubs: depositGroup.payCoinSelection.coinPubs,
+          contributions: depositGroup.payCoinSelection.coinContributions.map(
+            (x) => Amounts.parseOrThrow(x),
+          ),
+          refreshReason: RefreshReason.PayDeposit,
+        });
+      }
       await tx.depositGroups.put(depositGroup);
       await tx.contractTerms.put({
         contractTermsRaw: contractTerms,
@@ -1499,7 +1628,7 @@ export async function createDepositGroup(
 export async function getCounterpartyEffectiveDepositAmount(
   wex: WalletExecutionContext,
   wireType: string,
-  pcs: PayCoinSelection,
+  pcs: SelectedProspectiveCoin[],
 ): Promise<AmountJson> {
   const amt: AmountJson[] = [];
   const fees: AmountJson[] = [];
@@ -1508,23 +1637,19 @@ export async function getCounterpartyEffectiveDepositAmount(
   await wex.db.runReadOnlyTx(
     ["coins", "denominations", "exchangeDetails", "exchanges"],
     async (tx) => {
-      for (let i = 0; i < pcs.coins.length; i++) {
-        const coin = await tx.coins.get(pcs.coins[i].coinPub);
-        if (!coin) {
-          throw Error("can't calculate deposit amount, coin not found");
-        }
+      for (let i = 0; i < pcs.length; i++) {
         const denom = await getDenomInfo(
           wex,
           tx,
-          coin.exchangeBaseUrl,
-          coin.denomPubHash,
+          pcs[i].exchangeBaseUrl,
+          pcs[i].denomPubHash,
         );
         if (!denom) {
           throw Error("can't find denomination to calculate deposit amount");
         }
-        amt.push(Amounts.parseOrThrow(pcs.coins[i].contribution));
+        amt.push(Amounts.parseOrThrow(pcs[i].contribution));
         fees.push(Amounts.parseOrThrow(denom.feeDeposit));
-        exchangeSet.add(coin.exchangeBaseUrl);
+        exchangeSet.add(pcs[i].exchangeBaseUrl);
       }
 
       for (const exchangeUrl of exchangeSet.values()) {
@@ -1563,7 +1688,7 @@ async function getTotalFeesForDepositAmount(
   wex: WalletExecutionContext,
   wireType: string,
   total: AmountJson,
-  pcs: PayCoinSelection,
+  pcs: SelectedProspectiveCoin[],
 ): Promise<DepositGroupFees> {
   const wireFee: AmountJson[] = [];
   const coinFee: AmountJson[] = [];
@@ -1574,38 +1699,24 @@ async function getTotalFeesForDepositAmount(
   await wex.db.runReadOnlyTx(
     ["coins", "denominations", "exchanges", "exchangeDetails"],
     async (tx) => {
-      for (let i = 0; i < pcs.coins.length; i++) {
-        const coin = await tx.coins.get(pcs.coins[i].coinPub);
-        if (!coin) {
-          throw Error("can't calculate deposit amount, coin not found");
-        }
+      for (let i = 0; i < pcs.length; i++) {
         const denom = await getDenomInfo(
           wex,
           tx,
-          coin.exchangeBaseUrl,
-          coin.denomPubHash,
+          pcs[i].exchangeBaseUrl,
+          pcs[i].denomPubHash,
         );
         if (!denom) {
           throw Error("can't find denomination to calculate deposit amount");
         }
         coinFee.push(Amounts.parseOrThrow(denom.feeDeposit));
-        exchangeSet.add(coin.exchangeBaseUrl);
-
-        const allDenoms = await getCandidateWithdrawalDenomsTx(
+        exchangeSet.add(pcs[i].exchangeBaseUrl);
+        const amountLeft = Amounts.sub(denom.value, pcs[i].contribution).amount;
+        const refreshCost = await getTotalRefreshCost(
           wex,
           tx,
-          coin.exchangeBaseUrl,
-          currency,
-        );
-        const amountLeft = Amounts.sub(
-          denom.value,
-          pcs.coins[i].contribution,
-        ).amount;
-        const refreshCost = getTotalRefreshCost(
-          allDenoms,
           denom,
           amountLeft,
-          wex.ws.config.testing.denomselAllowLate,
         );
         refreshFee.push(refreshCost);
       }
