@@ -26,6 +26,7 @@
 import {
   AbsoluteTime,
   AgeRestriction,
+  Amount,
   Amounts,
   AsyncFlag,
   CancellationToken,
@@ -33,6 +34,7 @@ import {
   CoinStatus,
   DeleteExchangeRequest,
   DenomKeyType,
+  DenomLossEventType,
   DenomOperationMap,
   DenominationInfo,
   DenominationPubKey,
@@ -65,6 +67,10 @@ import {
   TalerPreciseTimestamp,
   TalerProtocolDuration,
   TalerProtocolTimestamp,
+  TransactionIdStr,
+  TransactionMajorState,
+  TransactionState,
+  TransactionType,
   URL,
   WalletNotification,
   WireFee,
@@ -77,6 +83,7 @@ import {
   codecForExchangeKeysJson,
   durationMul,
   encodeCrock,
+  getRandomBytes,
   hashDenomPub,
   j2s,
   makeErrorDetail,
@@ -90,9 +97,12 @@ import {
 } from "@gnu-taler/taler-util/http";
 import {
   PendingTaskType,
+  TaskIdStr,
   TaskIdentifiers,
   TaskRunResult,
   TaskRunResultType,
+  TransactionContext,
+  computeDbBackoff,
   constructTaskIdentifier,
   getAutoRefreshExecuteThreshold,
   getExchangeEntryStatusFromRecord,
@@ -101,6 +111,8 @@ import {
   getExchangeUpdateStatusFromRecord,
 } from "./common.js";
 import {
+  DenomLossEventRecord,
+  DenomLossStatus,
   DenominationRecord,
   DenominationVerificationStatus,
   ExchangeDetailsRecord,
@@ -126,6 +138,10 @@ import {
 import { DbReadOnlyTransaction } from "./query.js";
 import { createRecoupGroup } from "./recoup.js";
 import { createRefreshGroup } from "./refresh.js";
+import {
+  constructTransactionIdentifier,
+  notifyTransition,
+} from "./transactions.js";
 import { WALLET_EXCHANGE_PROTOCOL_VERSION } from "./versions.js";
 import { InternalWalletState, WalletExecutionContext } from "./wallet.js";
 
@@ -296,8 +312,8 @@ async function makeExchangeListItem(
 ): Promise<ExchangeListItem> {
   const lastUpdateErrorInfo: OperationErrorInfo | undefined = lastError
     ? {
-        error: lastError,
-      }
+      error: lastError,
+    }
     : undefined;
 
   let scopeInfo: ScopeInfo | undefined = undefined;
@@ -309,7 +325,9 @@ async function makeExchangeListItem(
   return {
     exchangeBaseUrl: r.baseUrl,
     masterPub: exchangeDetails?.masterPublicKey,
-    currency: exchangeDetails?.currency ?? r.presetCurrencyHint,
+    noFees: r.noFees ?? false,
+    peerPaymentsDisabled: r.peerPaymentsDisabled ?? false,
+    currency: exchangeDetails?.currency ?? r.presetCurrencyHint ?? "UNKNOWN",
     exchangeUpdateStatus: getExchangeUpdateStatusFromRecord(r),
     exchangeEntryStatus: getExchangeEntryStatusFromRecord(r),
     tosStatus: getExchangeTosStatusFromRecord(r),
@@ -319,7 +337,11 @@ async function makeExchangeListItem(
     paytoUris: exchangeDetails?.wireInfo.accounts.map((x) => x.payto_uri) ?? [],
     lastUpdateTimestamp: timestampOptionalPreciseFromDb(r.lastUpdate),
     lastUpdateErrorInfo,
-    scopeInfo,
+    scopeInfo: scopeInfo ?? {
+      type: ScopeType.Exchange,
+      currency: "UNKNOWN",
+      url: r.baseUrl,
+    },
   };
 }
 
@@ -403,6 +425,7 @@ export async function acceptExchangeTermsOfService(
         );
         await tx.exchanges.put(exch);
         const newExchangeState = getExchangeState(exch);
+        wex.ws.exchangeCache.clear();
         return {
           type: NotificationType.ExchangeStateTransition,
           exchangeBaseUrl,
@@ -435,6 +458,7 @@ export async function forgetExchangeTermsOfService(
         exch.tosAcceptedTimestamp = undefined;
         await tx.exchanges.put(exch);
         const newExchangeState = getExchangeState(exch);
+        wex.ws.exchangeCache.clear();
         return {
           type: NotificationType.ExchangeStateTransition,
           exchangeBaseUrl,
@@ -791,9 +815,6 @@ async function downloadExchangeKeysInfo(
             stampStart: timestampProtocolToDb(denomIn.stamp_start),
             verificationStatus: DenominationVerificationStatus.Unverified,
             masterSig: denomIn.master_sig,
-            listIssueDate: timestampProtocolToDb(
-              exchangeKeysJsonUnchecked.list_issue_date,
-            ),
             fees: {
               feeDeposit: Amounts.stringify(denomGroup.fee_deposit),
               feeRefresh: Amounts.stringify(denomGroup.fee_refresh),
@@ -894,18 +915,20 @@ async function startUpdateExchangeEntry(
   const canonBaseUrl = canonicalizeBaseUrl(exchangeBaseUrl);
 
   logger.info(
-    `starting update of exchange entry ${canonBaseUrl}, forced=${
-      options.forceUpdate ?? false
+    `starting update of exchange entry ${canonBaseUrl}, forced=${options.forceUpdate ?? false
     }`,
   );
 
   const { notification } = await wex.db.runReadWriteTx(
     ["exchanges", "exchangeDetails"],
     async (tx) => {
+      wex.ws.exchangeCache.clear();
       return provideExchangeRecordInTx(wex.ws, tx, exchangeBaseUrl);
     },
   );
 
+  logger.trace("created exchange record");
+
   if (notification) {
     wex.ws.notify(notification);
   }
@@ -951,6 +974,7 @@ async function startUpdateExchangeEntry(
             r.cachebreakNextUpdate = options.forceUpdate;
             break;
         }
+        wex.ws.exchangeCache.clear();
         await tx.exchanges.put(r);
         const newExchangeState = getExchangeState(r);
         // Reset retries for updating the exchange entry.
@@ -999,6 +1023,9 @@ async function internalWaitReadyExchange(
     exchangeBaseUrl: canonUrl,
   });
   while (true) {
+    if (wex.cancellationToken.isCancelled) {
+      throw Error("cancelled");
+    }
     logger.info(`waiting for ready exchange ${canonUrl}`);
     const { exchange, exchangeDetails, retryInfo, scopeInfo } =
       await wex.db.runReadOnlyTx(
@@ -1042,6 +1069,14 @@ async function internalWaitReadyExchange(
           ready = true;
         }
         break;
+      case ExchangeEntryDbUpdateStatus.UnavailableUpdate:
+        throw TalerError.fromDetail(
+          TalerErrorCode.WALLET_EXCHANGE_UNAVAILABLE,
+          {
+            exchangeBaseUrl: canonUrl,
+            innerError: retryInfo?.lastError,
+          },
+        );
       default: {
         if (retryInfo) {
           throw TalerError.fromDetail(
@@ -1122,24 +1157,35 @@ export async function fetchFreshExchange(
 ): Promise<ReadyExchangeSummary> {
   const canonUrl = canonicalizeBaseUrl(baseUrl);
 
+  if (!options.forceUpdate) {
+    const cachedResp = wex.ws.exchangeCache.get(canonUrl);
+    if (cachedResp) {
+      return cachedResp;
+    }
+  } else {
+    wex.ws.exchangeCache.clear();
+  }
+
   wex.taskScheduler.ensureRunning();
 
   await startUpdateExchangeEntry(wex, canonUrl, {
     forceUpdate: options.forceUpdate,
   });
 
-  return waitReadyExchange(wex, canonUrl, options);
+  const resp = await waitReadyExchange(wex, canonUrl, options);
+  wex.ws.exchangeCache.put(canonUrl, resp);
+  return resp;
 }
 
 async function waitReadyExchange(
   wex: WalletExecutionContext,
   canonUrl: string,
   options: {
-    cancellationToken?: CancellationToken;
     forceUpdate?: boolean;
     expectedMasterPub?: string;
   } = {},
 ): Promise<ReadyExchangeSummary> {
+  logger.trace(`waiting for exchange ${canonUrl} to become ready`);
   // FIXME: We should use Symbol.dispose magic here for cleanup!
 
   const exchangeNotifFlag = new AsyncFlag();
@@ -1155,6 +1201,11 @@ async function waitReadyExchange(
     }
   });
 
+  const unregisterOnCancelled = wex.cancellationToken.onCancelled(() => {
+    cancelNotif();
+    exchangeNotifFlag.raise();
+  });
+
   try {
     const res = await internalWaitReadyExchange(
       wex,
@@ -1165,10 +1216,66 @@ async function waitReadyExchange(
     logger.info("done waiting for ready exchange");
     return res;
   } finally {
+    unregisterOnCancelled();
     cancelNotif();
   }
 }
 
+function checkPeerPaymentsDisabled(
+  keysInfo: ExchangeKeysDownloadResult,
+): boolean {
+  const now = AbsoluteTime.now();
+  for (let gf of keysInfo.globalFees) {
+    const isActive = AbsoluteTime.isBetween(
+      now,
+      AbsoluteTime.fromProtocolTimestamp(gf.start_date),
+      AbsoluteTime.fromProtocolTimestamp(gf.end_date),
+    );
+    if (!isActive) {
+      continue;
+    }
+    return false;
+  }
+  // No global fees, we can't do p2p payments!
+  return true;
+}
+
+function checkNoFees(keysInfo: ExchangeKeysDownloadResult): boolean {
+  for (const gf of keysInfo.globalFees) {
+    if (!Amounts.isZero(gf.account_fee)) {
+      return false;
+    }
+    if (!Amounts.isZero(gf.history_fee)) {
+      return false;
+    }
+    if (!Amounts.isZero(gf.purse_fee)) {
+      return false;
+    }
+  }
+  for (const denom of keysInfo.currentDenominations) {
+    if (!Amounts.isZero(denom.fees.feeWithdraw)) {
+      return false;
+    }
+    if (!Amounts.isZero(denom.fees.feeDeposit)) {
+      return false;
+    }
+    if (!Amounts.isZero(denom.fees.feeRefund)) {
+      return false;
+    }
+    if (!Amounts.isZero(denom.fees.feeRefresh)) {
+      return false;
+    }
+  }
+  for (const wft of Object.values(keysInfo.wireFees)) {
+    for (const wf of wft) {
+      if (!Amounts.isZero(wf.wire_fee)) {
+        return false;
+      }
+    }
+  }
+  return true;
+}
+
 /**
  * Update an exchange entry in the wallet's database
  * by fetching the /keys and /wire information.
@@ -1205,9 +1312,11 @@ export async function updateExchangeFromUrlHandler(
       return TaskRunResult.finished();
     case ExchangeEntryDbUpdateStatus.InitialUpdate:
     case ExchangeEntryDbUpdateStatus.ReadyUpdate:
-    case ExchangeEntryDbUpdateStatus.UnavailableUpdate:
       updateRequestedExplicitly = true;
       break;
+    case ExchangeEntryDbUpdateStatus.UnavailableUpdate:
+      // Only retry when scheduled to respect backoff
+      break;
     case ExchangeEntryDbUpdateStatus.Ready:
       break;
     default:
@@ -1255,6 +1364,7 @@ export async function updateExchangeFromUrlHandler(
     }
 
     if (!(updateNecessary || refreshCheckNecessary)) {
+      logger.trace("update not necessary, running again later");
       return TaskRunResult.runAgainAt(
         AbsoluteTime.min(nextUpdateStamp, nextRefreshCheckStamp),
       );
@@ -1296,6 +1406,7 @@ export async function updateExchangeFromUrlHandler(
     keysInfo.globalFees,
     keysInfo.masterPublicKey,
   );
+
   if (keysInfo.baseUrl != exchangeBaseUrl) {
     logger.warn("exchange base URL mismatch");
     const errorDetail: TalerErrorDetail = makeErrorDetail(
@@ -1323,12 +1434,8 @@ export async function updateExchangeFromUrlHandler(
     ["text/plain"],
   );
 
-  let recoupGroupId: string | undefined;
-
   logger.trace("updating exchange info in database");
 
-  let detailsPointerChanged = false;
-
   let ageMask = 0;
   for (const x of keysInfo.currentDenominations) {
     if (
@@ -1339,6 +1446,8 @@ export async function updateExchangeFromUrlHandler(
       break;
     }
   }
+  let noFees = checkNoFees(keysInfo);
+  let peerPaymentsDisabled = checkPeerPaymentsDisabled(keysInfo);
 
   const updated = await wex.db.runReadWriteTx(
     [
@@ -1349,6 +1458,8 @@ export async function updateExchangeFromUrlHandler(
       "coins",
       "refreshGroups",
       "recoupGroups",
+      "coinAvailability",
+      "denomLossEvents",
     ],
     async (tx) => {
       const r = await tx.exchanges.get(exchangeBaseUrl);
@@ -1356,20 +1467,49 @@ export async function updateExchangeFromUrlHandler(
         logger.warn(`exchange ${exchangeBaseUrl} no longer present`);
         return;
       }
+
+      wex.ws.refreshCostCache.clear();
+      wex.ws.exchangeCache.clear();
+      wex.ws.denomInfoCache.clear();
+
       const oldExchangeState = getExchangeState(r);
       const existingDetails = await getExchangeRecordsInternal(tx, r.baseUrl);
+      let detailsPointerChanged = false;
       if (!existingDetails) {
         detailsPointerChanged = true;
       }
+      let detailsIncompatible = false;
       if (existingDetails) {
         if (existingDetails.masterPublicKey !== keysInfo.masterPublicKey) {
+          detailsIncompatible = true;
           detailsPointerChanged = true;
         }
         if (existingDetails.currency !== keysInfo.currency) {
+          detailsIncompatible = true;
           detailsPointerChanged = true;
         }
         // FIXME: We need to do some consistency checks!
       }
+      if (detailsIncompatible) {
+        logger.warn(
+          `exchange ${r.baseUrl} has incompatible data in /keys, not updating`,
+        );
+        // We don't support this gracefully right now.
+        // See https://bugs.taler.net/n/8576
+        r.updateStatus = ExchangeEntryDbUpdateStatus.UnavailableUpdate;
+        r.updateRetryCounter = (r.updateRetryCounter ?? 0) + 1;
+        r.nextUpdateStamp = computeDbBackoff(r.updateRetryCounter);
+        r.nextRefreshCheckStamp = timestampPreciseToDb(
+          AbsoluteTime.toPreciseTimestamp(AbsoluteTime.never()),
+        );
+        r.cachebreakNextUpdate = true;
+        await tx.exchanges.put(r);
+        return {
+          oldExchangeState,
+          newExchangeState: getExchangeState(r),
+        };
+      }
+      r.updateRetryCounter = 0;
       const newDetails: ExchangeDetailsRecord = {
         auditors: keysInfo.auditors,
         currency: keysInfo.currency,
@@ -1381,6 +1521,8 @@ export async function updateExchangeFromUrlHandler(
         wireInfo,
         ageMask,
       };
+      r.noFees = noFees;
+      r.peerPaymentsDisabled = peerPaymentsDisabled;
       r.tosCurrentEtag = tosDownload.tosEtag;
       if (existingDetails?.rowId) {
         newDetails.rowId = existingDetails.rowId;
@@ -1402,6 +1544,7 @@ export async function updateExchangeFromUrlHandler(
           updateClock: timestampPreciseToDb(TalerPreciseTimestamp.now()),
         };
       }
+
       r.updateStatus = ExchangeEntryDbUpdateStatus.Ready;
       r.cachebreakNextUpdate = false;
       await tx.exchanges.put(r);
@@ -1421,15 +1564,23 @@ export async function updateExchangeFromUrlHandler(
         });
       }
 
+      // In the future: Filter out old denominations by index
+      const allOldDenoms =
+        await tx.denominations.indexes.byExchangeBaseUrl.getAll(
+          exchangeBaseUrl,
+        );
+      const oldDenomByDph = new Map<string, DenominationRecord>();
+      for (const denom of allOldDenoms) {
+        oldDenomByDph.set(denom.denomPubHash, denom);
+      }
+
       logger.trace("updating denominations in database");
       const currentDenomSet = new Set<string>(
         keysInfo.currentDenominations.map((x) => x.denomPubHash),
       );
+
       for (const currentDenom of keysInfo.currentDenominations) {
-        const oldDenom = await tx.denominations.get([
-          exchangeBaseUrl,
-          currentDenom.denomPubHash,
-        ]);
+        const oldDenom = oldDenomByDph.get(currentDenom.denomPubHash);
         if (oldDenom) {
           // FIXME: Do consistency check, report to auditor if necessary.
           // See https://bugs.taler.net/n/8594
@@ -1449,70 +1600,37 @@ export async function updateExchangeFromUrlHandler(
 
       // Update list issue date for all denominations,
       // and mark non-offered denominations as such.
-      await tx.denominations.indexes.byExchangeBaseUrl
-        .iter(r.baseUrl)
-        .forEachAsync(async (x) => {
-          if (!currentDenomSet.has(x.denomPubHash)) {
-            // FIXME: Here, an auditor report should be created, unless
-            // the denomination is really legally expired.
-            if (x.isOffered) {
-              x.isOffered = false;
-              logger.info(
-                `setting denomination ${x.denomPubHash} to offered=false`,
-              );
-            }
-          } else {
-            x.listIssueDate = timestampProtocolToDb(keysInfo.listIssueDate);
-            if (!x.isOffered) {
-              x.isOffered = true;
-              logger.info(
-                `setting denomination ${x.denomPubHash} to offered=true`,
-              );
-            }
+      for (const x of allOldDenoms) {
+        if (!currentDenomSet.has(x.denomPubHash)) {
+          // FIXME: Here, an auditor report should be created, unless
+          // the denomination is really legally expired.
+          if (x.isOffered) {
+            x.isOffered = false;
+            logger.info(
+              `setting denomination ${x.denomPubHash} to offered=false`,
+            );
           }
-          await tx.denominations.put(x);
-        });
+        } else {
+          if (!x.isOffered) {
+            x.isOffered = true;
+            logger.info(
+              `setting denomination ${x.denomPubHash} to offered=true`,
+            );
+          }
+        }
+        await tx.denominations.put(x);
+      }
 
       logger.trace("done updating denominations in database");
 
-      // Handle recoup
-      const recoupDenomList = keysInfo.recoup;
-      const newlyRevokedCoinPubs: string[] = [];
-      logger.trace("recoup list from exchange", recoupDenomList);
-      for (const recoupInfo of recoupDenomList) {
-        const oldDenom = await tx.denominations.get([
-          r.baseUrl,
-          recoupInfo.h_denom_pub,
-        ]);
-        if (!oldDenom) {
-          // We never even knew about the revoked denomination, all good.
-          continue;
-        }
-        if (oldDenom.isRevoked) {
-          // We already marked the denomination as revoked,
-          // this implies we revoked all coins
-          logger.trace("denom already revoked");
-          continue;
-        }
-        logger.info("revoking denom", recoupInfo.h_denom_pub);
-        oldDenom.isRevoked = true;
-        await tx.denominations.put(oldDenom);
-        const affectedCoins = await tx.coins.indexes.byDenomPubHash
-          .iter(recoupInfo.h_denom_pub)
-          .toArray();
-        for (const ac of affectedCoins) {
-          newlyRevokedCoinPubs.push(ac.coinPub);
-        }
-      }
-      if (newlyRevokedCoinPubs.length != 0) {
-        logger.info("recouping coins", newlyRevokedCoinPubs);
-        recoupGroupId = await createRecoupGroup(
-          wex,
-          tx,
-          exchangeBaseUrl,
-          newlyRevokedCoinPubs,
-        );
-      }
+      const denomLossResult = await handleDenomLoss(
+        wex,
+        tx,
+        newDetails.currency,
+        exchangeBaseUrl,
+      );
+
+      await handleRecoup(wex, tx, exchangeBaseUrl, keysInfo.recoup);
 
       const newExchangeState = getExchangeState(r);
 
@@ -1521,24 +1639,21 @@ export async function updateExchangeFromUrlHandler(
         exchangeDetails: newDetails,
         oldExchangeState,
         newExchangeState,
+        denomLossResult,
       };
     },
   );
 
-  if (recoupGroupId) {
-    const recoupTaskId = constructTaskIdentifier({
-      tag: PendingTaskType.Recoup,
-      recoupGroupId,
-    });
-    // Asynchronously start recoup.  This doesn't need to finish
-    // for the exchange update to be considered finished.
-    wex.taskScheduler.startShepherdTask(recoupTaskId);
-  }
-
   if (!updated) {
     throw Error("something went wrong with updating the exchange");
   }
 
+  if (updated.denomLossResult) {
+    for (const notif of updated.denomLossResult.notifications) {
+      wex.ws.notify(notif);
+    }
+  }
+
   logger.trace("done updating exchange info in database");
 
   logger.trace(`doing auto-refresh check for '${exchangeBaseUrl}'`);
@@ -1556,6 +1671,7 @@ export async function updateExchangeFromUrlHandler(
         "denominations",
         "coinAvailability",
         "refreshGroups",
+        "refreshSessions",
         "exchanges",
       ],
       async (tx) => {
@@ -1615,6 +1731,7 @@ export async function updateExchangeFromUrlHandler(
         exchange.nextRefreshCheckStamp = timestampPreciseToDb(
           AbsoluteTime.toPreciseTimestamp(minCheckThreshold),
         );
+        wex.ws.exchangeCache.clear();
         await tx.exchanges.put(exchange);
       },
     );
@@ -1632,6 +1749,299 @@ export async function updateExchangeFromUrlHandler(
   return TaskRunResult.progress();
 }
 
+interface DenomLossResult {
+  notifications: WalletNotification[];
+}
+
+async function handleDenomLoss(
+  wex: WalletExecutionContext,
+  tx: WalletDbReadWriteTransaction<
+    ["coinAvailability", "denominations", "denomLossEvents", "coins"]
+  >,
+  currency: string,
+  exchangeBaseUrl: string,
+): Promise<DenomLossResult> {
+  const coinAvailabilityRecs =
+    await tx.coinAvailability.indexes.byExchangeBaseUrl.getAll(exchangeBaseUrl);
+  const denomsVanished: string[] = [];
+  const denomsUnoffered: string[] = [];
+  const denomsExpired: string[] = [];
+  let amountVanished = Amount.zeroOfCurrency(currency);
+  let amountExpired = Amount.zeroOfCurrency(currency);
+  let amountUnoffered = Amount.zeroOfCurrency(currency);
+
+  const result: DenomLossResult = {
+    notifications: [],
+  };
+
+  for (const coinAv of coinAvailabilityRecs) {
+    if (coinAv.freshCoinCount <= 0) {
+      continue;
+    }
+    const n = coinAv.freshCoinCount;
+    const denom = await tx.denominations.get([
+      coinAv.exchangeBaseUrl,
+      coinAv.denomPubHash,
+    ]);
+    const timestampExpireDeposit = !denom
+      ? undefined
+      : timestampAbsoluteFromDb(denom.stampExpireDeposit);
+    if (!denom) {
+      // Remove availability
+      coinAv.freshCoinCount = 0;
+      coinAv.visibleCoinCount = 0;
+      await tx.coinAvailability.put(coinAv);
+      denomsVanished.push(coinAv.denomPubHash);
+      const total = Amount.from(coinAv.value).mult(n);
+      amountVanished = amountVanished.add(total);
+    } else if (!denom.isOffered) {
+      // Remove availability
+      coinAv.freshCoinCount = 0;
+      coinAv.visibleCoinCount = 0;
+      await tx.coinAvailability.put(coinAv);
+      denomsUnoffered.push(coinAv.denomPubHash);
+      const total = Amount.from(coinAv.value).mult(n);
+      amountUnoffered = amountUnoffered.add(total);
+    } else if (
+      timestampExpireDeposit &&
+      AbsoluteTime.isExpired(timestampExpireDeposit)
+    ) {
+      // Remove availability
+      coinAv.freshCoinCount = 0;
+      coinAv.visibleCoinCount = 0;
+      await tx.coinAvailability.put(coinAv);
+      denomsExpired.push(coinAv.denomPubHash);
+      const total = Amount.from(coinAv.value).mult(n);
+      amountExpired = amountExpired.add(total);
+    } else {
+      // Denomination is still fine!
+      continue;
+    }
+
+    logger.warn(`denomination ${coinAv.denomPubHash} is a loss`);
+
+    const coins = await tx.coins.indexes.byDenomPubHash.getAll(
+      coinAv.denomPubHash,
+    );
+    for (const coin of coins) {
+      switch (coin.status) {
+        case CoinStatus.Fresh:
+        case CoinStatus.FreshSuspended: {
+          coin.status = CoinStatus.DenomLoss;
+          await tx.coins.put(coin);
+          break;
+        }
+      }
+    }
+  }
+
+  if (denomsVanished.length > 0) {
+    const denomLossEventId = encodeCrock(getRandomBytes(32));
+    await tx.denomLossEvents.add({
+      denomLossEventId,
+      amount: amountVanished.toString(),
+      currency,
+      exchangeBaseUrl,
+      denomPubHashes: denomsVanished,
+      eventType: DenomLossEventType.DenomVanished,
+      status: DenomLossStatus.Done,
+      timestampCreated: timestampPreciseToDb(TalerPreciseTimestamp.now()),
+    });
+    const transactionId = constructTransactionIdentifier({
+      tag: TransactionType.DenomLoss,
+      denomLossEventId,
+    });
+    result.notifications.push({
+      type: NotificationType.TransactionStateTransition,
+      transactionId,
+      oldTxState: {
+        major: TransactionMajorState.None,
+      },
+      newTxState: {
+        major: TransactionMajorState.Done,
+      },
+    });
+    result.notifications.push({
+      type: NotificationType.BalanceChange,
+      hintTransactionId: transactionId,
+    });
+  }
+
+  if (denomsUnoffered.length > 0) {
+    const denomLossEventId = encodeCrock(getRandomBytes(32));
+    await tx.denomLossEvents.add({
+      denomLossEventId,
+      amount: amountUnoffered.toString(),
+      currency,
+      exchangeBaseUrl,
+      denomPubHashes: denomsUnoffered,
+      eventType: DenomLossEventType.DenomUnoffered,
+      status: DenomLossStatus.Done,
+      timestampCreated: timestampPreciseToDb(TalerPreciseTimestamp.now()),
+    });
+    const transactionId = constructTransactionIdentifier({
+      tag: TransactionType.DenomLoss,
+      denomLossEventId,
+    });
+    result.notifications.push({
+      type: NotificationType.TransactionStateTransition,
+      transactionId,
+      oldTxState: {
+        major: TransactionMajorState.None,
+      },
+      newTxState: {
+        major: TransactionMajorState.Done,
+      },
+    });
+    result.notifications.push({
+      type: NotificationType.BalanceChange,
+      hintTransactionId: transactionId,
+    });
+  }
+
+  if (denomsExpired.length > 0) {
+    const denomLossEventId = encodeCrock(getRandomBytes(32));
+    await tx.denomLossEvents.add({
+      denomLossEventId,
+      amount: amountExpired.toString(),
+      currency,
+      exchangeBaseUrl,
+      denomPubHashes: denomsUnoffered,
+      eventType: DenomLossEventType.DenomExpired,
+      status: DenomLossStatus.Done,
+      timestampCreated: timestampPreciseToDb(TalerPreciseTimestamp.now()),
+    });
+    const transactionId = constructTransactionIdentifier({
+      tag: TransactionType.DenomLoss,
+      denomLossEventId,
+    });
+    result.notifications.push({
+      type: NotificationType.TransactionStateTransition,
+      transactionId,
+      oldTxState: {
+        major: TransactionMajorState.None,
+      },
+      newTxState: {
+        major: TransactionMajorState.Done,
+      },
+    });
+    result.notifications.push({
+      type: NotificationType.BalanceChange,
+      hintTransactionId: transactionId,
+    });
+  }
+
+  return result;
+}
+
+export function computeDenomLossTransactionStatus(
+  rec: DenomLossEventRecord,
+): TransactionState {
+  switch (rec.status) {
+    case DenomLossStatus.Aborted:
+      return {
+        major: TransactionMajorState.Aborted,
+      };
+    case DenomLossStatus.Done:
+      return {
+        major: TransactionMajorState.Done,
+      };
+  }
+}
+
+export class DenomLossTransactionContext implements TransactionContext {
+  get taskId(): TaskIdStr | undefined {
+    return undefined;
+  }
+  transactionId: TransactionIdStr;
+
+  abortTransaction(): Promise<void> {
+    throw new Error("Method not implemented.");
+  }
+  suspendTransaction(): Promise<void> {
+    throw new Error("Method not implemented.");
+  }
+  resumeTransaction(): Promise<void> {
+    throw new Error("Method not implemented.");
+  }
+  failTransaction(): Promise<void> {
+    throw new Error("Method not implemented.");
+  }
+  async deleteTransaction(): Promise<void> {
+    const transitionInfo = await this.wex.db.runReadWriteTx(
+      ["denomLossEvents"],
+      async (tx) => {
+        const rec = await tx.denomLossEvents.get(this.denomLossEventId);
+        if (rec) {
+          const oldTxState = computeDenomLossTransactionStatus(rec);
+          await tx.denomLossEvents.delete(this.denomLossEventId);
+          return {
+            oldTxState,
+            newTxState: {
+              major: TransactionMajorState.Deleted,
+            },
+          };
+        }
+        return undefined;
+      },
+    );
+    notifyTransition(this.wex, this.transactionId, transitionInfo);
+  }
+
+  constructor(
+    private wex: WalletExecutionContext,
+    public denomLossEventId: string,
+  ) {
+    this.transactionId = constructTransactionIdentifier({
+      tag: TransactionType.DenomLoss,
+      denomLossEventId,
+    });
+  }
+}
+
+async function handleRecoup(
+  wex: WalletExecutionContext,
+  tx: WalletDbReadWriteTransaction<
+    ["denominations", "coins", "recoupGroups", "refreshGroups"]
+  >,
+  exchangeBaseUrl: string,
+  recoup: Recoup[],
+): Promise<void> {
+  // Handle recoup
+  const recoupDenomList = recoup;
+  const newlyRevokedCoinPubs: string[] = [];
+  logger.trace("recoup list from exchange", recoupDenomList);
+  for (const recoupInfo of recoupDenomList) {
+    const oldDenom = await tx.denominations.get([
+      exchangeBaseUrl,
+      recoupInfo.h_denom_pub,
+    ]);
+    if (!oldDenom) {
+      // We never even knew about the revoked denomination, all good.
+      continue;
+    }
+    if (oldDenom.isRevoked) {
+      // We already marked the denomination as revoked,
+      // this implies we revoked all coins
+      logger.trace("denom already revoked");
+      continue;
+    }
+    logger.info("revoking denom", recoupInfo.h_denom_pub);
+    oldDenom.isRevoked = true;
+    await tx.denominations.put(oldDenom);
+    const affectedCoins = await tx.coins.indexes.byDenomPubHash.getAll(
+      recoupInfo.h_denom_pub,
+    );
+    for (const ac of affectedCoins) {
+      newlyRevokedCoinPubs.push(ac.coinPub);
+    }
+  }
+  if (newlyRevokedCoinPubs.length != 0) {
+    logger.info("recouping coins", newlyRevokedCoinPubs);
+    await createRecoupGroup(wex, tx, exchangeBaseUrl, newlyRevokedCoinPubs);
+  }
+}
+
 function getAutoRefreshExecuteThresholdForDenom(
   d: DenominationRecord,
 ): AbsoluteTime {
@@ -1716,6 +2126,7 @@ export async function getExchangeTos(
     const updateExchangeEntry = await tx.exchanges.get(exchangeBaseUrl);
     if (updateExchangeEntry) {
       updateExchangeEntry.tosCurrentEtag = tosDownload.tosEtag;
+      wex.ws.exchangeCache.clear();
       await tx.exchanges.put(updateExchangeEntry);
     }
   });
@@ -2018,6 +2429,90 @@ async function internalGetExchangeResources(
   };
 }
 
+/**
+ * Purge information in the database associated with the exchange.
+ *
+ * Deletes information specific to the exchange and withdrawals,
+ * but keeps some transactions (payments, p2p, refreshes) around.
+ */
+async function purgeExchange(
+  tx: WalletDbReadWriteTransaction<
+    [
+      "exchanges",
+      "exchangeDetails",
+      "transactions",
+      "coinAvailability",
+      "coins",
+      "denominations",
+      "exchangeSignKeys",
+      "withdrawalGroups",
+      "planchets",
+    ]
+  >,
+  exchangeBaseUrl: string,
+): Promise<void> {
+  const detRecs = await tx.exchangeDetails.indexes.byExchangeBaseUrl.getAll();
+  for (const r of detRecs) {
+    if (r.rowId == null) {
+      // Should never happen, as rowId is the primary key.
+      continue;
+    }
+    await tx.exchangeDetails.delete(r.rowId);
+    const signkeyRecs =
+      await tx.exchangeSignKeys.indexes.byExchangeDetailsRowId.getAll(r.rowId);
+    for (const rec of signkeyRecs) {
+      await tx.exchangeSignKeys.delete([r.rowId, rec.signkeyPub]);
+    }
+  }
+  // FIXME: Also remove records related to transactions?
+  await tx.exchanges.delete(exchangeBaseUrl);
+
+  {
+    const coinAvailabilityRecs =
+      await tx.coinAvailability.indexes.byExchangeBaseUrl.getAll(
+        exchangeBaseUrl,
+      );
+    for (const rec of coinAvailabilityRecs) {
+      await tx.coinAvailability.delete([
+        exchangeBaseUrl,
+        rec.denomPubHash,
+        rec.maxAge,
+      ]);
+    }
+  }
+
+  {
+    const coinRecs = await tx.coins.indexes.byBaseUrl.getAll(exchangeBaseUrl);
+    for (const rec of coinRecs) {
+      await tx.coins.delete(rec.coinPub);
+    }
+  }
+
+  {
+    const denomRecs =
+      await tx.denominations.indexes.byExchangeBaseUrl.getAll(exchangeBaseUrl);
+    for (const rec of denomRecs) {
+      await tx.denominations.delete(rec.denomPubHash);
+    }
+  }
+
+  {
+    const withdrawalGroupRecs =
+      await tx.withdrawalGroups.indexes.byExchangeBaseUrl.getAll(
+        exchangeBaseUrl,
+      );
+    for (const wg of withdrawalGroupRecs) {
+      await tx.withdrawalGroups.delete(wg.withdrawalGroupId);
+      const planchets = await tx.planchets.indexes.byGroup.getAll(
+        wg.withdrawalGroupId,
+      );
+      for (const p of planchets) {
+        await tx.planchets.delete(p.coinPub);
+      }
+    }
+  }
+}
+
 export async function deleteExchange(
   wex: WalletExecutionContext,
   req: DeleteExchangeRequest,
@@ -2025,7 +2520,17 @@ export async function deleteExchange(
   let inUse: boolean = false;
   const exchangeBaseUrl = canonicalizeBaseUrl(req.exchangeBaseUrl);
   await wex.db.runReadWriteTx(
-    ["exchanges", "coins", "withdrawalGroups", "exchangeDetails"],
+    [
+      "exchanges",
+      "exchangeDetails",
+      "transactions",
+      "coinAvailability",
+      "coins",
+      "denominations",
+      "exchangeSignKeys",
+      "withdrawalGroups",
+      "planchets",
+    ],
     async (tx) => {
       const exchangeRec = await tx.exchanges.get(exchangeBaseUrl);
       if (!exchangeRec) {
@@ -2038,17 +2543,8 @@ export async function deleteExchange(
         inUse = true;
         return;
       }
-      const detRecs =
-        await tx.exchangeDetails.indexes.byExchangeBaseUrl.getAll();
-      for (const r of detRecs) {
-        if (r.rowId == null) {
-          // Should never happen, as rowId is the primary key.
-          continue;
-        }
-        await tx.exchangeDetails.delete(r.rowId);
-      }
-      // FIXME: Also remove records related to transactions?
-      await tx.exchanges.delete(exchangeBaseUrl);
+      await purgeExchange(tx, exchangeBaseUrl);
+      wex.ws.exchangeCache.clear();
     },
   );
 
