@@ -84,6 +84,7 @@ import { WalletExecutionContext } from "./wallet.js";
 import {
   getExchangeWithdrawalInfo,
   internalCreateWithdrawalGroup,
+  waitWithdrawalFinal,
 } from "./withdraw.js";
 
 const logger = new Logger("pay-peer-pull-credit.ts");
@@ -584,6 +585,7 @@ async function handlePeerPullCreditWithdrawing(
   if (!pullIni.withdrawalGroupId) {
     throw Error("invalid db state (withdrawing, but no withdrawal group ID");
   }
+  await waitWithdrawalFinal(wex, pullIni.withdrawalGroupId);
   const transactionId = constructTransactionIdentifier({
     tag: TransactionType.PeerPullCredit,
     pursePub: pullIni.pursePub,
@@ -1071,15 +1073,15 @@ export async function initiatePeerPullPayment(
 
   const ctx = new PeerPullCreditTransactionContext(wex, pursePair.pub);
 
+  notifyTransition(wex, ctx.transactionId, transitionInfo);
+  wex.taskScheduler.startShepherdTask(ctx.taskId);
+
   // The pending-incoming balance has changed.
   wex.ws.notify({
     type: NotificationType.BalanceChange,
     hintTransactionId: ctx.transactionId,
   });
 
-  notifyTransition(wex, ctx.transactionId, transitionInfo);
-  wex.taskScheduler.startShepherdTask(ctx.taskId);
-
   return {
     talerUri: stringifyTalerUri({
       type: TalerUriAction.PayPull,
