@@ -37,6 +37,7 @@ import {
   PreparePeerPullDebitRequest,
   PreparePeerPullDebitResponse,
   RefreshReason,
+  SelectedProspectiveCoin,
   TalerError,
   TalerErrorCode,
   TalerPreciseTimestamp,
@@ -71,7 +72,7 @@ import {
   TaskRunResult,
   TaskRunResultType,
   TransactionContext,
-  TransitionResult,
+  TransitionResultType,
   constructTaskIdentifier,
   spendCoins,
 } from "./common.js";
@@ -192,17 +193,17 @@ export class PeerPullDebitTransactionContext implements TransactionContext {
       switch (pi.status) {
         case PeerPullDebitRecordStatus.SuspendedDeposit:
           pi.status = PeerPullDebitRecordStatus.PendingDeposit;
-          return TransitionResult.Transition;
+          return TransitionResultType.Transition;
         case PeerPullDebitRecordStatus.SuspendedAbortingRefresh:
           pi.status = PeerPullDebitRecordStatus.AbortingRefresh;
-          return TransitionResult.Transition;
+          return TransitionResultType.Transition;
         case PeerPullDebitRecordStatus.Aborted:
         case PeerPullDebitRecordStatus.AbortingRefresh:
         case PeerPullDebitRecordStatus.Failed:
         case PeerPullDebitRecordStatus.DialogProposed:
         case PeerPullDebitRecordStatus.Done:
         case PeerPullDebitRecordStatus.PendingDeposit:
-          return TransitionResult.Stay;
+          return TransitionResultType.Stay;
       }
     });
     this.wex.taskScheduler.startShepherdTask(this.taskId);
@@ -218,9 +219,9 @@ export class PeerPullDebitTransactionContext implements TransactionContext {
         case PeerPullDebitRecordStatus.SuspendedAbortingRefresh:
           // FIXME: Should we also abort the corresponding refresh session?!
           pi.status = PeerPullDebitRecordStatus.Failed;
-          return TransitionResult.Transition;
+          return TransitionResultType.Transition;
         default:
-          return TransitionResult.Stay;
+          return TransitionResultType.Stay;
       }
     });
     this.wex.taskScheduler.stopShepherdTask(this.taskId);
@@ -234,6 +235,7 @@ export class PeerPullDebitTransactionContext implements TransactionContext {
           "coinAvailability",
           "denominations",
           "refreshGroups",
+          "refreshSessions",
           "coins",
           "coinAvailability",
         ],
@@ -244,7 +246,7 @@ export class PeerPullDebitTransactionContext implements TransactionContext {
           case PeerPullDebitRecordStatus.PendingDeposit:
             break;
           default:
-            return TransitionResult.Stay;
+            return TransitionResultType.Stay;
         }
         const currency = Amounts.currencyOf(pi.totalCostEstimated);
         const coinPubs: CoinRefreshRequest[] = [];
@@ -271,13 +273,13 @@ export class PeerPullDebitTransactionContext implements TransactionContext {
 
         pi.status = PeerPullDebitRecordStatus.AbortingRefresh;
         pi.abortRefreshGroupId = refresh.refreshGroupId;
-        return TransitionResult.Transition;
+        return TransitionResultType.Transition;
       },
     );
   }
 
   async transition(
-    f: (rec: PeerPullPaymentIncomingRecord) => Promise<TransitionResult>,
+    f: (rec: PeerPullPaymentIncomingRecord) => Promise<TransitionResultType>,
   ): Promise<void> {
     return this.transitionExtra(
       {
@@ -297,7 +299,7 @@ export class PeerPullDebitTransactionContext implements TransactionContext {
         typeof WalletStoresV1,
         ["peerPullDebit", ...StoreNameArray]
       >,
-    ) => Promise<TransitionResult>,
+    ) => Promise<TransitionResultType>,
   ): Promise<void> {
     const wex = this.wex;
     const extraStores = opts.extraStores ?? [];
@@ -311,7 +313,7 @@ export class PeerPullDebitTransactionContext implements TransactionContext {
         const oldTxState = computePeerPullDebitTransactionState(pi);
         const res = await f(pi, tx);
         switch (res) {
-          case TransitionResult.Transition: {
+          case TransitionResultType.Transition: {
             await tx.peerPullDebit.put(pi);
             const newTxState = computePeerPullDebitTransactionState(pi);
             return {
@@ -369,13 +371,25 @@ async function handlePurseCreationConflict(
     }
   }
 
-  const coinSelRes = await selectPeerCoins(ws, { instructedAmount, repair });
+  const coinSelRes = await selectPeerCoins(ws, {
+    instructedAmount,
+    repair,
+  });
 
-  if (coinSelRes.type == "failure") {
-    // FIXME: Details!
-    throw Error(
-      "insufficient balance to re-select coins to repair double spending",
-    );
+  switch (coinSelRes.type) {
+    case "failure":
+      // FIXME: Details!
+      throw Error(
+        "insufficient balance to re-select coins to repair double spending",
+      );
+    case "prospective":
+      throw Error(
+        "insufficient balance to re-select coins to repair double spending (blocked on refresh)",
+      );
+    case "success":
+      break;
+    default:
+      assertUnreachable(coinSelRes);
   }
 
   const totalAmount = await getTotalPeerPaymentCost(
@@ -414,8 +428,88 @@ async function processPeerPullDebitPendingDeposit(
   const pursePub = peerPullInc.pursePub;
 
   const coinSel = peerPullInc.coinSel;
+
   if (!coinSel) {
-    throw Error("invalid state, no coins selected");
+    const instructedAmount = Amounts.parseOrThrow(peerPullInc.amount);
+
+    const coinSelRes = await selectPeerCoins(wex, {
+      instructedAmount,
+    });
+    if (logger.shouldLogTrace()) {
+      logger.trace(`selected p2p coins (pull): ${j2s(coinSelRes)}`);
+    }
+
+    let coins: SelectedProspectiveCoin[] | undefined = undefined;
+
+    switch (coinSelRes.type) {
+      case "failure":
+        throw TalerError.fromDetail(
+          TalerErrorCode.WALLET_PEER_PUSH_PAYMENT_INSUFFICIENT_BALANCE,
+          {
+            insufficientBalanceDetails: coinSelRes.insufficientBalanceDetails,
+          },
+        );
+      case "prospective":
+        throw Error("insufficient balance (locked behind refresh)");
+      case "success":
+        coins = coinSelRes.result.coins;
+        break;
+      default:
+        assertUnreachable(coinSelRes);
+    }
+
+    const peerPullDebitId = peerPullInc.peerPullDebitId;
+    const totalAmount = await getTotalPeerPaymentCost(wex, coins);
+
+    // FIXME: Missing notification here!
+
+    const transitionDone = await wex.db.runReadWriteTx(
+      [
+        "exchanges",
+        "coins",
+        "denominations",
+        "refreshGroups",
+        "refreshSessions",
+        "peerPullDebit",
+        "coinAvailability",
+      ],
+      async (tx) => {
+        const pi = await tx.peerPullDebit.get(peerPullDebitId);
+        if (!pi) {
+          return false;
+        }
+        if (pi.status !== PeerPullDebitRecordStatus.PendingDeposit) {
+          return false;
+        }
+        if (pi.coinSel) {
+          return false;
+        }
+        await spendCoins(wex, tx, {
+          // allocationId: `txn:peer-pull-debit:${req.peerPullDebitId}`,
+          allocationId: constructTransactionIdentifier({
+            tag: TransactionType.PeerPullDebit,
+            peerPullDebitId,
+          }),
+          coinPubs: coinSelRes.result.coins.map((x) => x.coinPub),
+          contributions: coinSelRes.result.coins.map((x) =>
+            Amounts.parseOrThrow(x.contribution),
+          ),
+          refreshReason: RefreshReason.PayPeerPull,
+        });
+        pi.coinSel = {
+          coinPubs: coinSelRes.result.coins.map((x) => x.coinPub),
+          contributions: coinSelRes.result.coins.map((x) => x.contribution),
+          totalCost: Amounts.stringify(totalAmount),
+        };
+        await tx.peerPullDebit.put(pi);
+        return true;
+      },
+    );
+    if (transitionDone) {
+      return TaskRunResult.progress();
+    } else {
+      return TaskRunResult.backoff();
+    }
   }
 
   const coins = await queryCoinInfosForSelection(wex, coinSel);
@@ -460,10 +554,10 @@ async function processPeerPullDebitPendingDeposit(
 
       await ctx.transition(async (r) => {
         if (r.status !== PeerPullDebitRecordStatus.PendingDeposit) {
-          return TransitionResult.Stay;
+          return TransitionResultType.Stay;
         }
         r.status = PeerPullDebitRecordStatus.Done;
-        return TransitionResult.Transition;
+        return TransitionResultType.Transition;
       });
       return TaskRunResult.finished();
     }
@@ -561,18 +655,11 @@ export async function confirmPeerPullDebit(
   req: ConfirmPeerPullDebitRequest,
 ): Promise<AcceptPeerPullPaymentResponse> {
   let peerPullDebitId: string;
-
-  if (req.transactionId) {
-    const parsedTx = parseTransactionIdentifier(req.transactionId);
-    if (!parsedTx || parsedTx.tag !== TransactionType.PeerPullDebit) {
-      throw Error("invalid peer-pull-debit transaction identifier");
-    }
-    peerPullDebitId = parsedTx.peerPullDebitId;
-  } else if (req.peerPullDebitId) {
-    peerPullDebitId = req.peerPullDebitId;
-  } else {
-    throw Error("invalid request, transactionId or peerPullDebitId required");
+  const parsedTx = parseTransactionIdentifier(req.transactionId);
+  if (!parsedTx || parsedTx.tag !== TransactionType.PeerPullDebit) {
+    throw Error("invalid peer-pull-debit transaction identifier");
   }
+  peerPullDebitId = parsedTx.peerPullDebitId;
 
   const peerPullInc = await wex.db.runReadOnlyTx(
     ["peerPullDebit"],
@@ -583,32 +670,42 @@ export async function confirmPeerPullDebit(
 
   if (!peerPullInc) {
     throw Error(
-      `can't accept unknown incoming p2p pull payment (${req.peerPullDebitId})`,
+      `can't accept unknown incoming p2p pull payment (${req.transactionId})`,
     );
   }
 
   const instructedAmount = Amounts.parseOrThrow(peerPullInc.amount);
 
-  const coinSelRes = await selectPeerCoins(wex, { instructedAmount });
+  const coinSelRes = await selectPeerCoins(wex, {
+    instructedAmount,
+  });
   if (logger.shouldLogTrace()) {
     logger.trace(`selected p2p coins (pull): ${j2s(coinSelRes)}`);
   }
 
-  if (coinSelRes.type !== "success") {
-    throw TalerError.fromDetail(
-      TalerErrorCode.WALLET_PEER_PUSH_PAYMENT_INSUFFICIENT_BALANCE,
-      {
-        insufficientBalanceDetails: coinSelRes.insufficientBalanceDetails,
-      },
-    );
+  let coins: SelectedProspectiveCoin[] | undefined = undefined;
+
+  switch (coinSelRes.type) {
+    case "failure":
+      throw TalerError.fromDetail(
+        TalerErrorCode.WALLET_PEER_PUSH_PAYMENT_INSUFFICIENT_BALANCE,
+        {
+          insufficientBalanceDetails: coinSelRes.insufficientBalanceDetails,
+        },
+      );
+    case "prospective":
+      coins = coinSelRes.result.prospectiveCoins;
+      break;
+    case "success":
+      coins = coinSelRes.result.coins;
+      break;
+    default:
+      assertUnreachable(coinSelRes);
   }
 
-  const sel = coinSelRes.result;
+  const totalAmount = await getTotalPeerPaymentCost(wex, coins);
 
-  const totalAmount = await getTotalPeerPaymentCost(
-    wex,
-    coinSelRes.result.coins,
-  );
+  // FIXME: Missing notification here!
 
   await wex.db.runReadWriteTx(
     [
@@ -616,35 +713,38 @@ export async function confirmPeerPullDebit(
       "coins",
       "denominations",
       "refreshGroups",
+      "refreshSessions",
       "peerPullDebit",
       "coinAvailability",
     ],
     async (tx) => {
-      await spendCoins(wex, tx, {
-        // allocationId: `txn:peer-pull-debit:${req.peerPullDebitId}`,
-        allocationId: constructTransactionIdentifier({
-          tag: TransactionType.PeerPullDebit,
-          peerPullDebitId,
-        }),
-        coinPubs: sel.coins.map((x) => x.coinPub),
-        contributions: sel.coins.map((x) =>
-          Amounts.parseOrThrow(x.contribution),
-        ),
-        refreshReason: RefreshReason.PayPeerPull,
-      });
-
       const pi = await tx.peerPullDebit.get(peerPullDebitId);
       if (!pi) {
         throw Error();
       }
-      if (pi.status === PeerPullDebitRecordStatus.DialogProposed) {
-        pi.status = PeerPullDebitRecordStatus.PendingDeposit;
+      if (pi.status !== PeerPullDebitRecordStatus.DialogProposed) {
+        return;
+      }
+      if (coinSelRes.type == "success") {
+        await spendCoins(wex, tx, {
+          // allocationId: `txn:peer-pull-debit:${req.peerPullDebitId}`,
+          allocationId: constructTransactionIdentifier({
+            tag: TransactionType.PeerPullDebit,
+            peerPullDebitId,
+          }),
+          coinPubs: coinSelRes.result.coins.map((x) => x.coinPub),
+          contributions: coinSelRes.result.coins.map((x) =>
+            Amounts.parseOrThrow(x.contribution),
+          ),
+          refreshReason: RefreshReason.PayPeerPull,
+        });
         pi.coinSel = {
-          coinPubs: sel.coins.map((x) => x.coinPub),
-          contributions: sel.coins.map((x) => x.contribution),
+          coinPubs: coinSelRes.result.coins.map((x) => x.coinPub),
+          contributions: coinSelRes.result.coins.map((x) => x.contribution),
           totalCost: Amounts.stringify(totalAmount),
         };
       }
+      pi.status = PeerPullDebitRecordStatus.PendingDeposit;
       await tx.peerPullDebit.put(pi);
     },
   );
@@ -763,24 +863,34 @@ export async function preparePeerPullDebit(
 
   const instructedAmount = Amounts.parseOrThrow(contractTerms.amount);
 
-  const coinSelRes = await selectPeerCoins(wex, { instructedAmount });
+  const coinSelRes = await selectPeerCoins(wex, {
+    instructedAmount,
+  });
   if (logger.shouldLogTrace()) {
     logger.trace(`selected p2p coins (pull): ${j2s(coinSelRes)}`);
   }
 
-  if (coinSelRes.type !== "success") {
-    throw TalerError.fromDetail(
-      TalerErrorCode.WALLET_PEER_PUSH_PAYMENT_INSUFFICIENT_BALANCE,
-      {
-        insufficientBalanceDetails: coinSelRes.insufficientBalanceDetails,
-      },
-    );
+  let coins: SelectedProspectiveCoin[] | undefined = undefined;
+
+  switch (coinSelRes.type) {
+    case "failure":
+      throw TalerError.fromDetail(
+        TalerErrorCode.WALLET_PEER_PUSH_PAYMENT_INSUFFICIENT_BALANCE,
+        {
+          insufficientBalanceDetails: coinSelRes.insufficientBalanceDetails,
+        },
+      );
+    case "prospective":
+      coins = coinSelRes.result.prospectiveCoins;
+      break;
+    case "success":
+      coins = coinSelRes.result.coins;
+      break;
+    default:
+      assertUnreachable(coinSelRes);
   }
 
-  const totalAmount = await getTotalPeerPaymentCost(
-    wex,
-    coinSelRes.result.coins,
-  );
+  const totalAmount = await getTotalPeerPaymentCost(wex, coins);
 
   await wex.db.runReadWriteTx(
     ["peerPullDebit", "contractTerms"],
