@@ -88,6 +88,7 @@ import {
   getExchangeWithdrawalInfo,
   internalPerformCreateWithdrawalGroup,
   internalPrepareCreateWithdrawalGroup,
+  waitWithdrawalFinal,
 } from "./withdraw.js";
 
 const logger = new Logger("pay-peer-push-credit.ts");
@@ -789,6 +790,7 @@ async function handlePendingWithdrawing(
   if (!peerInc.withdrawalGroupId) {
     throw Error("invalid db state (withdrawing, but no withdrawal group ID");
   }
+  await waitWithdrawalFinal(wex, peerInc.withdrawalGroupId);
   const transactionId = constructTransactionIdentifier({
     tag: TransactionType.PeerPushCredit,
     peerPushCreditId: peerInc.peerPushCreditId,
@@ -901,20 +903,14 @@ export async function confirmPeerPushCredit(
 ): Promise<AcceptPeerPushPaymentResponse> {
   let peerInc: PeerPushPaymentIncomingRecord | undefined;
   let peerPushCreditId: string;
-  if (req.peerPushCreditId) {
-    peerPushCreditId = req.peerPushCreditId;
-  } else if (req.transactionId) {
-    const parsedTx = parseTransactionIdentifier(req.transactionId);
-    if (!parsedTx) {
-      throw Error("invalid transaction ID");
-    }
-    if (parsedTx.tag !== TransactionType.PeerPushCredit) {
-      throw Error("invalid transaction ID type");
-    }
-    peerPushCreditId = parsedTx.peerPushCreditId;
-  } else {
-    throw Error("no transaction ID (or deprecated peerPushCreditId) provided");
+  const parsedTx = parseTransactionIdentifier(req.transactionId);
+  if (!parsedTx) {
+    throw Error("invalid transaction ID");
   }
+  if (parsedTx.tag !== TransactionType.PeerPushCredit) {
+    throw Error("invalid transaction ID type");
+  }
+  peerPushCreditId = parsedTx.peerPushCreditId;
 
   logger.trace(`confirming peer-push-credit ${peerPushCreditId}`);
 
@@ -934,7 +930,7 @@ export async function confirmPeerPushCredit(
 
   if (!peerInc) {
     throw Error(
-      `can't accept unknown incoming p2p push payment (${req.peerPushCreditId})`,
+      `can't accept unknown incoming p2p push payment (${req.transactionId})`,
     );
   }
 
