@@ -26,6 +26,7 @@ import {
   Logger,
   NotificationType,
   RefreshReason,
+  SelectedProspectiveCoin,
   TalerError,
   TalerErrorCode,
   TalerPreciseTimestamp,
@@ -38,6 +39,7 @@ import {
   TransactionState,
   TransactionType,
   assertUnreachable,
+  checkDbInvariant,
   checkLogicInvariant,
   encodeCrock,
   getRandomBytes,
@@ -72,7 +74,7 @@ import {
   getTotalPeerPaymentCost,
   queryCoinInfosForSelection,
 } from "./pay-peer-common.js";
-import { createRefreshGroup } from "./refresh.js";
+import { createRefreshGroup, waitRefreshFinal } from "./refresh.js";
 import {
   constructTransactionIdentifier,
   notifyTransition,
@@ -343,20 +345,29 @@ export async function checkPeerPushDebit(
   logger.trace(
     `checking peer push debit for ${Amounts.stringify(instructedAmount)}`,
   );
-  const coinSelRes = await selectPeerCoins(wex, { instructedAmount });
-  if (coinSelRes.type === "failure") {
-    throw TalerError.fromDetail(
-      TalerErrorCode.WALLET_PEER_PUSH_PAYMENT_INSUFFICIENT_BALANCE,
-      {
-        insufficientBalanceDetails: coinSelRes.insufficientBalanceDetails,
-      },
-    );
+  const coinSelRes = await selectPeerCoins(wex, {
+    instructedAmount,
+  });
+  let coins: SelectedProspectiveCoin[] | undefined = undefined;
+  switch (coinSelRes.type) {
+    case "failure":
+      throw TalerError.fromDetail(
+        TalerErrorCode.WALLET_PEER_PUSH_PAYMENT_INSUFFICIENT_BALANCE,
+        {
+          insufficientBalanceDetails: coinSelRes.insufficientBalanceDetails,
+        },
+      );
+    case "prospective":
+      coins = coinSelRes.result.prospectiveCoins;
+      break;
+    case "success":
+      coins = coinSelRes.result.coins;
+      break;
+    default:
+      assertUnreachable(coinSelRes);
   }
-  logger.trace(`selected peer coins (len=${coinSelRes.result.coins.length})`);
-  const totalAmount = await getTotalPeerPaymentCost(
-    wex,
-    coinSelRes.result.coins,
-  );
+  logger.trace(`selected peer coins (len=${coins.length})`);
+  const totalAmount = await getTotalPeerPaymentCost(wex, coins);
   logger.trace("computed total peer payment cost");
   return {
     exchangeBaseUrl: coinSelRes.result.exchangeBaseUrl,
@@ -391,6 +402,8 @@ async function handlePurseCreationConflict(
   const instructedAmount = Amounts.parseOrThrow(peerPushInitiation.amount);
   const sel = peerPushInitiation.coinSel;
 
+  checkDbInvariant(!!sel);
+
   const repair: PreviousPayCoins = [];
 
   for (let i = 0; i < sel.coinPubs.length; i++) {
@@ -402,13 +415,22 @@ async function handlePurseCreationConflict(
     }
   }
 
-  const coinSelRes = await selectPeerCoins(wex, { instructedAmount, repair });
+  const coinSelRes = await selectPeerCoins(wex, {
+    instructedAmount,
+    repair,
+  });
 
-  if (coinSelRes.type == "failure") {
-    // FIXME: Details!
-    throw Error(
-      "insufficient balance to re-select coins to repair double spending",
-    );
+  switch (coinSelRes.type) {
+    case "failure":
+    case "prospective":
+      // FIXME: Details!
+      throw Error(
+        "insufficient balance to re-select coins to repair double spending",
+      );
+    case "success":
+      break;
+    default:
+      assertUnreachable(coinSelRes);
   }
 
   await wex.db.runReadWriteTx(["peerPushDebit"], async (tx) => {
@@ -459,6 +481,75 @@ async function processPeerPushDebitCreateReserve(
     );
   }
 
+  if (!peerPushInitiation.coinSel) {
+    const coinSelRes = await selectPeerCoins(wex, {
+      instructedAmount: Amounts.parseOrThrow(peerPushInitiation.amount),
+    });
+
+    switch (coinSelRes.type) {
+      case "failure":
+        throw TalerError.fromDetail(
+          TalerErrorCode.WALLET_PEER_PUSH_PAYMENT_INSUFFICIENT_BALANCE,
+          {
+            insufficientBalanceDetails: coinSelRes.insufficientBalanceDetails,
+          },
+        );
+      case "prospective":
+        throw Error("insufficient funds (blocked on refresh)");
+      case "success":
+        break;
+      default:
+        assertUnreachable(coinSelRes);
+    }
+    const transitionDone = await wex.db.runReadWriteTx(
+      [
+        "exchanges",
+        "contractTerms",
+        "coins",
+        "coinAvailability",
+        "denominations",
+        "refreshGroups",
+        "refreshSessions",
+        "peerPushDebit",
+      ],
+      async (tx) => {
+        const ppi = await tx.peerPushDebit.get(pursePub);
+        if (!ppi) {
+          return false;
+        }
+        if (ppi.coinSel) {
+          return false;
+        }
+
+        ppi.coinSel = {
+          coinPubs: coinSelRes.result.coins.map((x) => x.coinPub),
+          contributions: coinSelRes.result.coins.map((x) => x.contribution),
+        };
+        // FIXME: Instead of directly doing a spendCoin here,
+        // we might want to mark the coins as used and spend them
+        // after we've been able to create the purse.
+        await spendCoins(wex, tx, {
+          allocationId: constructTransactionIdentifier({
+            tag: TransactionType.PeerPushDebit,
+            pursePub,
+          }),
+          coinPubs: coinSelRes.result.coins.map((x) => x.coinPub),
+          contributions: coinSelRes.result.coins.map((x) =>
+            Amounts.parseOrThrow(x.contribution),
+          ),
+          refreshReason: RefreshReason.PayPeerPush,
+        });
+
+        await tx.peerPushDebit.put(ppi);
+        return true;
+      },
+    );
+    if (transitionDone) {
+      return TaskRunResult.progress();
+    }
+    return TaskRunResult.backoff();
+  }
+
   const purseSigResp = await wex.cryptoApi.signPurseCreation({
     hContractTerms,
     mergePub: peerPushInitiation.mergePub,
@@ -588,6 +679,7 @@ async function processPeerPushDebitAbortingDeletePurse(
     [
       "peerPushDebit",
       "refreshGroups",
+      "refreshSessions",
       "denominations",
       "coinAvailability",
       "coins",
@@ -604,6 +696,10 @@ async function processPeerPushDebitAbortingDeletePurse(
       const oldTxState = computePeerPushDebitTransactionState(ppiRec);
       const coinPubs: CoinRefreshRequest[] = [];
 
+      if (!ppiRec.coinSel) {
+        return undefined;
+      }
+
       for (let i = 0; i < ppiRec.coinSel.coinPubs.length; i++) {
         coinPubs.push({
           amount: ppiRec.coinSel.contributions[i],
@@ -682,6 +778,9 @@ async function processPeerPushDebitAbortingRefreshDeleted(
     tag: TransactionType.PeerPushDebit,
     pursePub: peerPushInitiation.pursePub,
   });
+  if (peerPushInitiation.abortRefreshGroupId) {
+    await waitRefreshFinal(wex, peerPushInitiation.abortRefreshGroupId);
+  }
   const transitionInfo = await wex.db.runReadWriteTx(
     ["refreshGroups", "peerPushDebit"],
     async (tx) => {
@@ -818,6 +917,7 @@ async function processPeerPushDebitReady(
       [
         "peerPushDebit",
         "refreshGroups",
+        "refreshSessions",
         "denominations",
         "coinAvailability",
         "coins",
@@ -834,23 +934,26 @@ async function processPeerPushDebitReady(
         const oldTxState = computePeerPushDebitTransactionState(ppiRec);
         const coinPubs: CoinRefreshRequest[] = [];
 
-        for (let i = 0; i < ppiRec.coinSel.coinPubs.length; i++) {
-          coinPubs.push({
-            amount: ppiRec.coinSel.contributions[i],
-            coinPub: ppiRec.coinSel.coinPubs[i],
-          });
-        }
+        if (ppiRec.coinSel) {
+          for (let i = 0; i < ppiRec.coinSel.coinPubs.length; i++) {
+            coinPubs.push({
+              amount: ppiRec.coinSel.contributions[i],
+              coinPub: ppiRec.coinSel.coinPubs[i],
+            });
+          }
 
-        const refresh = await createRefreshGroup(
-          wex,
-          tx,
-          currency,
-          coinPubs,
-          RefreshReason.AbortPeerPushDebit,
-          transactionId,
-        );
+          const refresh = await createRefreshGroup(
+            wex,
+            tx,
+            currency,
+            coinPubs,
+            RefreshReason.AbortPeerPushDebit,
+            transactionId,
+          );
+
+          ppiRec.abortRefreshGroupId = refresh.refreshGroupId;
+        }
         ppiRec.status = PeerPushDebitStatus.AbortingRefreshExpired;
-        ppiRec.abortRefreshGroupId = refresh.refreshGroupId;
         await tx.peerPushDebit.put(ppiRec);
         const newTxState = computePeerPushDebitTransactionState(ppiRec);
         return {
@@ -929,15 +1032,28 @@ export async function initiatePeerPushDebit(
 
   const contractKeyPair = await wex.cryptoApi.createEddsaKeypair({});
 
-  const coinSelRes = await selectPeerCoins(wex, { instructedAmount });
+  const coinSelRes = await selectPeerCoins(wex, {
+    instructedAmount,
+  });
 
-  if (coinSelRes.type !== "success") {
-    throw TalerError.fromDetail(
-      TalerErrorCode.WALLET_PEER_PUSH_PAYMENT_INSUFFICIENT_BALANCE,
-      {
-        insufficientBalanceDetails: coinSelRes.insufficientBalanceDetails,
-      },
-    );
+  let coins: SelectedProspectiveCoin[] | undefined = undefined;
+
+  switch (coinSelRes.type) {
+    case "failure":
+      throw TalerError.fromDetail(
+        TalerErrorCode.WALLET_PEER_PUSH_PAYMENT_INSUFFICIENT_BALANCE,
+        {
+          insufficientBalanceDetails: coinSelRes.insufficientBalanceDetails,
+        },
+      );
+    case "prospective":
+      coins = coinSelRes.result.prospectiveCoins;
+      break;
+    case "success":
+      coins = coinSelRes.result.coins;
+      break;
+    default:
+      assertUnreachable(coinSelRes);
   }
 
   const sel = coinSelRes.result;
@@ -945,10 +1061,7 @@ export async function initiatePeerPushDebit(
   logger.info(`selected p2p coins (push):`);
   logger.trace(`${j2s(coinSelRes)}`);
 
-  const totalAmount = await getTotalPeerPaymentCost(
-    wex,
-    coinSelRes.result.coins,
-  );
+  const totalAmount = await getTotalPeerPaymentCost(wex, coins);
 
   logger.info(`computed total peer payment cost`);
 
@@ -968,24 +1081,10 @@ export async function initiatePeerPushDebit(
       "coinAvailability",
       "denominations",
       "refreshGroups",
+      "refreshSessions",
       "peerPushDebit",
     ],
     async (tx) => {
-      // FIXME: Instead of directly doing a spendCoin here,
-      // we might want to mark the coins as used and spend them
-      // after we've been able to create the purse.
-      await spendCoins(wex, tx, {
-        allocationId: constructTransactionIdentifier({
-          tag: TransactionType.PeerPushDebit,
-          pursePub: pursePair.pub,
-        }),
-        coinPubs: sel.coins.map((x) => x.coinPub),
-        contributions: sel.coins.map((x) =>
-          Amounts.parseOrThrow(x.contribution),
-        ),
-        refreshReason: RefreshReason.PayPeerPush,
-      });
-
       const ppi: PeerPushDebitRecord = {
         amount: Amounts.stringify(instructedAmount),
         contractPriv: contractKeyPair.priv,
@@ -1000,13 +1099,30 @@ export async function initiatePeerPushDebit(
         timestampCreated: timestampPreciseToDb(TalerPreciseTimestamp.now()),
         status: PeerPushDebitStatus.PendingCreatePurse,
         contractEncNonce,
-        coinSel: {
-          coinPubs: sel.coins.map((x) => x.coinPub),
-          contributions: sel.coins.map((x) => x.contribution),
-        },
         totalCost: Amounts.stringify(totalAmount),
       };
 
+      if (coinSelRes.type === "success") {
+        ppi.coinSel = {
+          coinPubs: coinSelRes.result.coins.map((x) => x.coinPub),
+          contributions: coinSelRes.result.coins.map((x) => x.contribution),
+        };
+        // FIXME: Instead of directly doing a spendCoin here,
+        // we might want to mark the coins as used and spend them
+        // after we've been able to create the purse.
+        await spendCoins(wex, tx, {
+          allocationId: constructTransactionIdentifier({
+            tag: TransactionType.PeerPushDebit,
+            pursePub: pursePair.pub,
+          }),
+          coinPubs: coinSelRes.result.coins.map((x) => x.coinPub),
+          contributions: coinSelRes.result.coins.map((x) =>
+            Amounts.parseOrThrow(x.contribution),
+          ),
+          refreshReason: RefreshReason.PayPeerPush,
+        });
+      }
+
       await tx.peerPushDebit.add(ppi);
 
       await tx.contractTerms.put({
