@@ -31,6 +31,7 @@ import {
   IDBKeyRange,
   IDBRequest,
   IDBTransaction,
+  IDBTransactionMode,
   IDBValidKey,
   IDBVersionChangeEvent,
 } from "@gnu-taler/idb-bridge";
@@ -766,26 +767,43 @@ function makeWriteContext(
 
 export interface DbAccess<StoreMap> {
   idbHandle(): IDBDatabase;
+
   runAllStoresReadWriteTx<T>(
+    options: {
+      label?: string;
+    },
     txf: (
       tx: DbReadWriteTransaction<StoreMap, Array<StoreNames<StoreMap>>>,
     ) => Promise<T>,
   ): Promise<T>;
+
   runAllStoresReadOnlyTx<T>(
+    options: {
+      label?: string;
+    },
     txf: (
       tx: DbReadOnlyTransaction<StoreMap, Array<StoreNames<StoreMap>>>,
     ) => Promise<T>,
   ): Promise<T>;
+
   runReadWriteTx<T, StoreNameArray extends Array<StoreNames<StoreMap>>>(
     storeNames: StoreNameArray,
     txf: (tx: DbReadWriteTransaction<StoreMap, StoreNameArray>) => Promise<T>,
   ): Promise<T>;
+
   runReadOnlyTx<T, StoreNameArray extends Array<StoreNames<StoreMap>>>(
     storeNames: StoreNameArray,
     txf: (tx: DbReadOnlyTransaction<StoreMap, StoreNameArray>) => Promise<T>,
   ): Promise<T>;
 }
 
+export interface TriggerSpec {
+  /**
+   * Trigger run after every successful commit, run outside of the transaction.
+   */
+  afterCommit?: (mode: IDBTransactionMode, stores: string[]) => void;
+}
+
 /**
  * Type-safe access to a database with a particular store map.
  *
@@ -795,6 +813,7 @@ export class DbAccessImpl<StoreMap> implements DbAccess<StoreMap> {
   constructor(
     private db: IDBDatabase,
     private stores: StoreMap,
+    private triggers: TriggerSpec = {},
   ) {}
 
   idbHandle(): IDBDatabase {
@@ -802,6 +821,9 @@ export class DbAccessImpl<StoreMap> implements DbAccess<StoreMap> {
   }
 
   runAllStoresReadWriteTx<T>(
+    options: {
+      label?: string;
+    },
     txf: (
       tx: DbReadWriteTransaction<StoreMap, Array<StoreNames<StoreMap>>>,
     ) => Promise<T>,
@@ -820,6 +842,9 @@ export class DbAccessImpl<StoreMap> implements DbAccess<StoreMap> {
   }
 
   runAllStoresReadOnlyTx<T>(
+    options: {
+      label?: string;
+    },
     txf: (
       tx: DbReadOnlyTransaction<StoreMap, Array<StoreNames<StoreMap>>>,
     ) => Promise<T>,
@@ -849,9 +874,14 @@ export class DbAccessImpl<StoreMap> implements DbAccess<StoreMap> {
       strStoreNames.push(swi.storeName);
       accessibleStores[swi.storeName] = swi;
     }
-    const tx = this.db.transaction(strStoreNames, "readwrite");
+    const mode = "readwrite";
+    const tx = this.db.transaction(strStoreNames, mode);
     const writeContext = makeWriteContext(tx, accessibleStores);
-    return runTx(tx, writeContext, txf);
+    const res = runTx(tx, writeContext, txf);
+    if (this.triggers.afterCommit) {
+      this.triggers.afterCommit(mode, strStoreNames);
+    }
+    return res;
   }
 
   runReadOnlyTx<T, StoreNameArray extends Array<StoreNames<StoreMap>>>(
@@ -866,8 +896,17 @@ export class DbAccessImpl<StoreMap> implements DbAccess<StoreMap> {
       strStoreNames.push(swi.storeName);
       accessibleStores[swi.storeName] = swi;
     }
-    const tx = this.db.transaction(strStoreNames, "readonly");
+    const mode = "readonly";
+    const tx = this.db.transaction(strStoreNames, mode);
     const readContext = makeReadContext(tx, accessibleStores);
-    return runTx(tx, readContext, txf);
+    const res = runTx(tx, readContext, txf);
+    if (this.triggers.afterCommit) {
+      this.triggers.afterCommit(mode, strStoreNames);
+    }
+    return res;
   }
+
+  registerPostCommitTrigger(args: {
+    handler: (storeNames: string[]) => void;
+  }): void {}
 }
