@@ -61,7 +61,10 @@ import {
   computeDepositTransactionStatus,
   processDepositGroup,
 } from "./deposits.js";
-import { updateExchangeFromUrlHandler } from "./exchanges.js";
+import {
+  computeDenomLossTransactionStatus,
+  updateExchangeFromUrlHandler,
+} from "./exchanges.js";
 import {
   computePayMerchantTransactionState,
   computeRefundTransactionState,
@@ -286,6 +289,7 @@ export class TaskSchedulerImpl implements TaskScheduler {
 
   async resetTaskRetries(taskId: TaskIdStr): Promise<void> {
     const maybeNotification = await this.ws.db.runAllStoresReadWriteTx(
+      {},
       async (tx) => {
         await tx.operationRetries.delete(taskId);
         return taskToRetryNotification(this.ws, tx, taskId, undefined);
@@ -434,7 +438,7 @@ async function storePendingTaskError(
   e: TalerErrorDetail,
 ): Promise<OperationRetryRecord> {
   logger.info(`storing pending task error for ${pendingTaskId}`);
-  const res = await ws.db.runAllStoresReadWriteTx(async (tx) => {
+  const res = await ws.db.runAllStoresReadWriteTx({}, async (tx) => {
     let retryRecord = await tx.operationRetries.get(pendingTaskId);
     if (!retryRecord) {
       retryRecord = {
@@ -474,7 +478,7 @@ async function storePendingTaskPending(
   ws: InternalWalletState,
   pendingTaskId: string,
 ): Promise<OperationRetryRecord> {
-  const res = await ws.db.runAllStoresReadWriteTx(async (tx) => {
+  const res = await ws.db.runAllStoresReadWriteTx({}, async (tx) => {
     let retryRecord = await tx.operationRetries.get(pendingTaskId);
     let hadError = false;
     if (!retryRecord) {
@@ -635,6 +639,7 @@ async function getTransactionState(
       "peerPushCredit",
       "rewards",
       "refreshGroups",
+      "denomLossEvents",
     ]
   >,
   transactionId: string,
@@ -673,12 +678,13 @@ async function getTransactionState(
       }
       return computeRefundTransactionState(rec);
     }
-    case TransactionType.PeerPullCredit:
+    case TransactionType.PeerPullCredit: {
       const rec = await tx.peerPullCredit.get(parsedTxId.pursePub);
       if (!rec) {
         return undefined;
       }
       return computePeerPullCreditTransactionState(rec);
+    }
     case TransactionType.PeerPullDebit: {
       const rec = await tx.peerPullDebit.get(parsedTxId.peerPullDebitId);
       if (!rec) {
@@ -716,6 +722,13 @@ async function getTransactionState(
     }
     case TransactionType.Recoup:
       throw Error("not yet supported");
+    case TransactionType.DenomLoss: {
+      const rec = await tx.denomLossEvents.get(parsedTxId.denomLossEventId);
+      if (!rec) {
+        return undefined;
+      }
+      return computeDenomLossTransactionStatus(rec);
+    }
     default:
       assertUnreachable(parsedTxId);
   }
@@ -863,6 +876,8 @@ export function listTaskForTransactionId(transactionId: string): TaskIdStr[] {
           withdrawalGroupId: tid.withdrawalGroupId,
         }),
       ];
+    case TransactionType.DenomLoss:
+      return [];
     default:
       assertUnreachable(tid);
   }
