@@ -821,7 +821,7 @@ export async function runIntegrationTest2(
   });
 
   await confirmPeerPullDebit(wex, {
-    peerPullDebitId: peerPullInc.peerPullDebitId,
+    transactionId: peerPullInc.transactionId,
   });
 
   await waitUntilTransactionWithAssociatedRefreshesFinal(
@@ -912,6 +912,6 @@ export async function testPay(
   });
   checkLogicInvariant(!!purchase);
   return {
-    numCoins: purchase.payInfo?.payCoinSelection.coinContributions.length ?? 0,
+    numCoins: purchase.payInfo?.payCoinSelection?.coinContributions.length ?? 0,
   };
 }
