@@ -23,7 +23,6 @@ import {
   Amounts,
   assertUnreachable,
   checkDbInvariant,
-  checkLogicInvariant,
   DepositTransactionTrackingState,
   j2s,
   Logger,
@@ -38,6 +37,7 @@ import {
   TalerErrorCode,
   TalerPreciseTimestamp,
   Transaction,
+  TransactionAction,
   TransactionByIdRequest,
   TransactionIdStr,
   TransactionMajorState,
@@ -59,6 +59,7 @@ import {
   TransactionContext,
 } from "./common.js";
 import {
+  DenomLossEventRecord,
   DepositElementStatus,
   DepositGroupRecord,
   OPERATION_STATUS_ACTIVE_FIRST,
@@ -76,7 +77,6 @@ import {
   RefreshGroupRecord,
   RefreshOperationStatus,
   RefundGroupRecord,
-  RewardRecord,
   timestampPreciseFromDb,
   timestampProtocolFromDb,
   WalletDbReadOnlyTransaction,
@@ -90,6 +90,8 @@ import {
   DepositTransactionContext,
 } from "./deposits.js";
 import {
+  computeDenomLossTransactionStatus,
+  DenomLossTransactionContext,
   ExchangeWireDetails,
   getExchangeWireDetailsInTx,
 } from "./exchanges.js";
@@ -127,12 +129,8 @@ import {
   computeRefreshTransactionState,
   RefreshTransactionContext,
 } from "./refresh.js";
-import {
-  computeRewardTransactionStatus,
-  computeTipTransactionActions,
-  RewardTransactionContext,
-} from "./reward.js";
-import type { InternalWalletState, WalletExecutionContext } from "./wallet.js";
+import { RewardTransactionContext } from "./reward.js";
+import type { WalletExecutionContext } from "./wallet.js";
 import {
   augmentPaytoUrisForWithdrawal,
   computeWithdrawalTransactionActions,
@@ -212,6 +210,7 @@ const txOrder: { [t in TransactionType]: number } = {
   [TransactionType.Refresh]: 10,
   [TransactionType.Recoup]: 11,
   [TransactionType.InternalWithdrawal]: 12,
+  [TransactionType.DenomLoss]: 13,
 };
 
 export async function getTransactionById(
@@ -268,6 +267,19 @@ export async function getTransactionById(
       );
     }
 
+    case TransactionType.DenomLoss: {
+      const rec = await wex.db.runReadOnlyTx(
+        ["denomLossEvents"],
+        async (tx) => {
+          return tx.denomLossEvents.get(parsedTx.denomLossEventId);
+        },
+      );
+      if (!rec) {
+        throw Error("denom loss record not found");
+      }
+      return buildTransactionForDenomLoss(rec);
+    }
+
     case TransactionType.Recoup:
       throw new Error("not yet supported");
 
@@ -322,19 +334,7 @@ export async function getTransactionById(
     }
 
     case TransactionType.Reward: {
-      const tipId = parsedTx.walletRewardId;
-      return await wex.db.runReadWriteTx(
-        ["rewards", "operationRetries"],
-        async (tx) => {
-          const tipRecord = await tx.rewards.get(tipId);
-          if (!tipRecord) throw Error("not found");
-
-          const retries = await tx.operationRetries.get(
-            TaskIdentifiers.forTipPickup(tipRecord),
-          );
-          return buildTransactionForTip(tipRecord, retries);
-        },
-      );
+      throw Error("unsupported operation");
     }
 
     case TransactionType.Deposit: {
@@ -739,7 +739,7 @@ function buildTransactionForBankIntegratedWithdraw(
   };
 }
 
-function isUnsuccessfulTransaction(state: TransactionState): boolean {
+export function isUnsuccessfulTransaction(state: TransactionState): boolean {
   return (
     state.major === TransactionMajorState.Aborted ||
     state.major === TransactionMajorState.Expired ||
@@ -871,15 +871,37 @@ function buildTransactionForRefresh(
   };
 }
 
+function buildTransactionForDenomLoss(rec: DenomLossEventRecord): Transaction {
+  const txState = computeDenomLossTransactionStatus(rec);
+  return {
+    type: TransactionType.DenomLoss,
+    txState,
+    txActions: [TransactionAction.Delete],
+    amountRaw: Amounts.stringify(rec.amount),
+    amountEffective: Amounts.stringify(rec.amount),
+    timestamp: timestampPreciseFromDb(rec.timestampCreated),
+    transactionId: constructTransactionIdentifier({
+      tag: TransactionType.DenomLoss,
+      denomLossEventId: rec.denomLossEventId,
+    }),
+    lossEventType: rec.eventType,
+    exchangeBaseUrl: rec.exchangeBaseUrl,
+  };
+}
+
 function buildTransactionForDeposit(
   dg: DepositGroupRecord,
   ort?: OperationRetryRecord,
 ): Transaction {
   let deposited = true;
-  for (const d of dg.statusPerCoin) {
-    if (d == DepositElementStatus.DepositPending) {
-      deposited = false;
+  if (dg.statusPerCoin) {
+    for (const d of dg.statusPerCoin) {
+      if (d == DepositElementStatus.DepositPending) {
+        deposited = false;
+      }
     }
+  } else {
+    deposited = false;
   }
 
   const trackingState: DepositTransactionTrackingState[] = [];
@@ -893,6 +915,17 @@ function buildTransactionForDeposit(
     });
   }
 
+  let wireTransferProgress = 0;
+  if (dg.statusPerCoin) {
+    wireTransferProgress =
+      (100 *
+        dg.statusPerCoin.reduce(
+          (prev, cur) => prev + (cur === DepositElementStatus.Wired ? 1 : 0),
+          0,
+        )) /
+      dg.statusPerCoin.length;
+  }
+
   const txState = computeDepositTransactionStatus(dg);
   return {
     type: TransactionType.Deposit,
@@ -909,13 +942,7 @@ function buildTransactionForDeposit(
       tag: TransactionType.Deposit,
       depositGroupId: dg.depositGroupId,
     }),
-    wireTransferProgress:
-      (100 *
-        dg.statusPerCoin.reduce(
-          (prev, cur) => prev + (cur === DepositElementStatus.Wired ? 1 : 0),
-          0,
-        )) /
-      dg.statusPerCoin.length,
+    wireTransferProgress,
     depositGroupId: dg.depositGroupId,
     trackingState,
     deposited,
@@ -923,31 +950,6 @@ function buildTransactionForDeposit(
   };
 }
 
-function buildTransactionForTip(
-  tipRecord: RewardRecord,
-  ort?: OperationRetryRecord,
-): Transaction {
-  checkLogicInvariant(!!tipRecord.acceptedTimestamp);
-
-  const txState = computeRewardTransactionStatus(tipRecord);
-  return {
-    type: TransactionType.Reward,
-    txState,
-    txActions: computeTipTransactionActions(tipRecord),
-    amountEffective: isUnsuccessfulTransaction(txState)
-      ? Amounts.stringify(Amounts.zeroOfAmount(tipRecord.rewardAmountEffective))
-      : Amounts.stringify(tipRecord.rewardAmountEffective),
-    amountRaw: Amounts.stringify(tipRecord.rewardAmountRaw),
-    timestamp: timestampPreciseFromDb(tipRecord.acceptedTimestamp),
-    transactionId: constructTransactionIdentifier({
-      tag: TransactionType.Reward,
-      walletRewardId: tipRecord.walletRewardId,
-    }),
-    merchantBaseUrl: tipRecord.merchantBaseUrl,
-    ...(ort?.lastError ? { error: ort.lastError } : {}),
-  };
-}
-
 async function lookupMaybeContractData(
   tx: WalletDbReadOnlyTransaction<["purchases", "contractTerms"]>,
   proposalId: string,
@@ -1116,6 +1118,7 @@ export async function getTransactions(
       "withdrawalGroups",
       "refreshGroups",
       "refundGroups",
+      "denomLossEvents",
     ],
     async (tx) => {
       await iterRecordsForPeerPushDebit(tx, filter, async (pi) => {
@@ -1261,7 +1264,10 @@ export async function getTransactions(
 
         const exchangesInTx: string[] = [];
         const p = await tx.purchases.get(refundGroup.proposalId);
-        if (!p || !p.payInfo) return; //refund with no payment
+        if (!p || !p.payInfo || !p.payInfo.payCoinSelection) {
+          //refund with no payment
+          return;
+        }
 
         // FIXME: This is very slow, should become obsolete with materialized transactions.
         for (const cp of p.payInfo.payCoinSelection.coinPubs) {
@@ -1362,6 +1368,21 @@ export async function getTransactions(
         }
       });
 
+      await iterRecordsForDenomLoss(tx, filter, async (rec) => {
+        const amount = Amounts.parseOrThrow(rec.amount);
+        const exchangesInTx = [rec.exchangeBaseUrl];
+        if (
+          shouldSkipCurrency(
+            transactionsRequest,
+            amount.currency,
+            exchangesInTx,
+          )
+        ) {
+          return;
+        }
+        transactions.push(buildTransactionForDenomLoss(rec));
+      });
+
       await iterRecordsForDeposit(tx, filter, async (dg) => {
         const amount = Amounts.parseOrThrow(dg.amount);
         const exchangesInTx = dg.infoPerExchange
@@ -1392,7 +1413,7 @@ export async function getTransactions(
         }
 
         const exchangesInTx: string[] = [];
-        for (const cp of purchase.payInfo.payCoinSelection.coinPubs) {
+        for (const cp of purchase.payInfo.payCoinSelection?.coinPubs ?? []) {
           const c = await tx.coins.get(cp);
           if (c?.exchangeBaseUrl) {
             exchangesInTx.push(c.exchangeBaseUrl);
@@ -1444,26 +1465,6 @@ export async function getTransactions(
           ),
         );
       });
-
-      //FIXME: remove rewards
-      await iterRecordsForReward(tx, filter, async (tipRecord) => {
-        if (
-          shouldSkipCurrency(
-            transactionsRequest,
-            Amounts.parseOrThrow(tipRecord.rewardAmountRaw).currency,
-            [tipRecord.exchangeBaseUrl],
-          )
-        ) {
-          return;
-        }
-        if (!tipRecord.acceptedTimestamp) {
-          return;
-        }
-        const opId = TaskIdentifiers.forTipPickup(tipRecord);
-        const retryRecord = await tx.operationRetries.get(opId);
-        transactions.push(buildTransactionForTip(tipRecord, retryRecord));
-      });
-      //ends REMOVE REWARDS
     },
   );
 
@@ -1487,9 +1488,6 @@ export async function getTransactions(
     x.txState.major === TransactionMajorState.Aborting ||
     x.txState.major === TransactionMajorState.Dialog;
 
-  const txPending = transactions.filter((x) => isPending(x));
-  const txNotPending = transactions.filter((x) => !isPending(x));
-
   let sortSign: number;
   if (transactionsRequest?.sort == "descending") {
     sortSign = -1;
@@ -1510,6 +1508,14 @@ export async function getTransactions(
     return sortSign * tsCmp;
   };
 
+  if (transactionsRequest?.sort === "stable-ascending") {
+    transactions.sort(txCmp);
+    return { transactions };
+  }
+
+  const txPending = transactions.filter((x) => isPending(x));
+  const txNotPending = transactions.filter((x) => !isPending(x));
+
   txPending.sort(txCmp);
   txNotPending.sort(txCmp);
 
@@ -1528,7 +1534,8 @@ export type ParsedTransactionIdentifier =
   | { tag: TransactionType.Reward; walletRewardId: string }
   | { tag: TransactionType.Withdrawal; withdrawalGroupId: string }
   | { tag: TransactionType.InternalWithdrawal; withdrawalGroupId: string }
-  | { tag: TransactionType.Recoup; recoupGroupId: string };
+  | { tag: TransactionType.Recoup; recoupGroupId: string }
+  | { tag: TransactionType.DenomLoss; denomLossEventId: string };
 
 export function constructTransactionIdentifier(
   pTxId: ParsedTransactionIdentifier,
@@ -1558,6 +1565,8 @@ export function constructTransactionIdentifier(
       return `txn:${pTxId.tag}:${pTxId.withdrawalGroupId}` as TransactionIdStr;
     case TransactionType.Recoup:
       return `txn:${pTxId.tag}:${pTxId.recoupGroupId}` as TransactionIdStr;
+    case TransactionType.DenomLoss:
+      return `txn:${pTxId.tag}:${pTxId.denomLossEventId}` as TransactionIdStr;
     default:
       assertUnreachable(pTxId);
   }
@@ -1617,6 +1626,11 @@ export function parseTransactionIdentifier(
         tag: TransactionType.Withdrawal,
         withdrawalGroupId: rest[0],
       };
+    case TransactionType.DenomLoss:
+      return {
+        tag: TransactionType.DenomLoss,
+        denomLossEventId: rest[0],
+      };
     default:
       return undefined;
   }
@@ -1688,6 +1702,9 @@ function maybeTaskFromTransaction(
         tag: PendingTaskType.Recoup,
         recoupGroupId: parsedTx.recoupGroupId,
       });
+    case TransactionType.DenomLoss:
+      // Nothing to do for denom loss
+      return undefined;
     default:
       assertUnreachable(parsedTx);
   }
@@ -1739,8 +1756,10 @@ async function getContextForTransaction(
     case TransactionType.Reward:
       return new RewardTransactionContext(wex, tx.walletRewardId);
     case TransactionType.Recoup:
+      //return new RecoupTransactionContext(ws, tx.recoupGroupId);
       throw new Error("not yet supported");
-    //return new RecoupTransactionContext(ws, tx.recoupGroupId);
+    case TransactionType.DenomLoss:
+      return new DenomLossTransactionContext(wex, tx.denomLossEventId);
     default:
       assertUnreachable(tx);
   }
@@ -1899,19 +1918,24 @@ async function iterRecordsForDeposit(
   }
 }
 
-async function iterRecordsForReward(
-  tx: WalletDbReadOnlyTransaction<["rewards"]>,
+async function iterRecordsForDenomLoss(
+  tx: WalletDbReadOnlyTransaction<["denomLossEvents"]>,
   filter: TransactionRecordFilter,
-  f: (r: RewardRecord) => Promise<void>,
+  f: (r: DenomLossEventRecord) => Promise<void>,
 ): Promise<void> {
+  let dgs: DenomLossEventRecord[];
   if (filter.onlyState === "nonfinal") {
     const keyRange = GlobalIDB.KeyRange.bound(
       OPERATION_STATUS_ACTIVE_FIRST,
       OPERATION_STATUS_ACTIVE_LAST,
     );
-    await tx.rewards.indexes.byStatus.iter(keyRange).forEachAsync(f);
+    dgs = await tx.denomLossEvents.indexes.byStatus.getAll(keyRange);
   } else {
-    await tx.rewards.indexes.byStatus.iter().forEachAsync(f);
+    dgs = await tx.denomLossEvents.indexes.byStatus.getAll();
+  }
+
+  for (const dg of dgs) {
+    await f(dg);
   }
 }
 
