@@ -24,7 +24,9 @@
  */
 import { IDBFactory } from "@gnu-taler/idb-bridge";
 import {
+  AbsoluteTime,
   ActiveTask,
+  AmountJson,
   AmountString,
   Amounts,
   AsyncCondition,
@@ -35,6 +37,7 @@ import {
   CreateStoredBackupResponse,
   DeleteStoredBackupRequest,
   DenominationInfo,
+  Duration,
   ExchangesShortListResponse,
   GetCurrencySpecificationResponse,
   InitResponse,
@@ -46,6 +49,7 @@ import {
   NotificationType,
   ObservabilityContext,
   ObservabilityEventType,
+  ObservableHttpClientLibrary,
   OpenedPromise,
   PartialWalletRunConfig,
   PrepareWithdrawExchangeRequest,
@@ -67,7 +71,6 @@ import {
   WalletCoreVersion,
   WalletNotification,
   WalletRunConfig,
-  WithdrawalDetailsForAmount,
   checkDbInvariant,
   codecForAbortTransaction,
   codecForAcceptBankIntegratedWithdrawalRequest,
@@ -142,6 +145,7 @@ import {
   openPromise,
   parsePaytoUri,
   parseTalerUri,
+  performanceNow,
   sampleWalletCoreTransactions,
   setDangerousTimetravel,
   validateIban,
@@ -185,12 +189,13 @@ import {
   timestampProtocolToDb,
 } from "./db.js";
 import {
+  checkDepositGroup,
   createDepositGroup,
   generateDepositGroupTxId,
-  prepareDepositGroup,
 } from "./deposits.js";
 import { DevExperimentHttpLib, applyDevExperiment } from "./dev-experiments.js";
 import {
+  ReadyExchangeSummary,
   acceptExchangeTermsOfService,
   addPresetExchangeEntry,
   deleteExchange,
@@ -211,7 +216,6 @@ import {
 } from "./instructedAmountConversion.js";
 import {
   ObservableDbAccess,
-  ObservableHttpClientLibrary,
   ObservableTaskScheduler,
   observeTalerCrypto,
 } from "./observable-wrappers.js";
@@ -274,7 +278,7 @@ import {
   WALLET_BANK_CONVERSION_API_PROTOCOL_VERSION,
   WALLET_BANK_INTEGRATION_PROTOCOL_VERSION,
   WALLET_COREBANK_API_PROTOCOL_VERSION,
-  WALLET_CORE_API_IMPLEMENTATION_VERSION,
+  WALLET_CORE_API_PROTOCOL_VERSION,
   WALLET_EXCHANGE_PROTOCOL_VERSION,
   WALLET_MERCHANT_PROTOCOL_VERSION,
 } from "./versions.js";
@@ -286,7 +290,7 @@ import {
 import {
   acceptWithdrawalFromUri,
   createManualWithdrawal,
-  getExchangeWithdrawalInfo,
+  getWithdrawalDetailsForAmount,
   getWithdrawalDetailsForUri,
 } from "./withdraw.js";
 
@@ -355,15 +359,15 @@ export async function getDenomInfo(
   exchangeBaseUrl: string,
   denomPubHash: string,
 ): Promise<DenominationInfo | undefined> {
-  const key = `${exchangeBaseUrl}:${denomPubHash}`;
-  const cached = wex.ws.lookupDenomCache(key);
+  const cacheKey = `${exchangeBaseUrl}:${denomPubHash}`;
+  const cached = wex.ws.denomInfoCache.get(cacheKey);
   if (cached) {
     return cached;
   }
   const d = await tx.denominations.get([exchangeBaseUrl, denomPubHash]);
   if (d) {
     const denomInfo = DenominationRecord.toDenomInfo(d);
-    wex.ws.putDenomCache(key, denomInfo);
+    wex.ws.denomInfoCache.put(cacheKey, denomInfo);
     return denomInfo;
   }
   return undefined;
@@ -559,7 +563,7 @@ async function createStoredBackup(
   const backup = await exportDb(wex.ws.idb);
   const backupsDb = await openStoredBackupsDatabase(wex.ws.idb);
   const name = `backup-${new Date().getTime()}`;
-  await backupsDb.runAllStoresReadWriteTx(async (tx) => {
+  await backupsDb.runAllStoresReadWriteTx({}, async (tx) => {
     await tx.backupMeta.add({
       name,
     });
@@ -577,7 +581,7 @@ async function listStoredBackups(
     storedBackups: [],
   };
   const backupsDb = await openStoredBackupsDatabase(wex.ws.idb);
-  await backupsDb.runAllStoresReadWriteTx(async (tx) => {
+  await backupsDb.runAllStoresReadWriteTx({}, async (tx) => {
     await tx.backupMeta.iter().forEach((x) => {
       storedBackups.storedBackups.push({
         name: x.name,
@@ -592,7 +596,7 @@ async function deleteStoredBackup(
   req: DeleteStoredBackupRequest,
 ): Promise<void> {
   const backupsDb = await openStoredBackupsDatabase(wex.ws.idb);
-  await backupsDb.runAllStoresReadWriteTx(async (tx) => {
+  await backupsDb.runAllStoresReadWriteTx({}, async (tx) => {
     await tx.backupData.delete(req.name);
     await tx.backupMeta.delete(req.name);
   });
@@ -605,7 +609,7 @@ async function recoverStoredBackup(
   logger.info(`Recovering stored backup ${req.name}`);
   const { name } = req;
   const backupsDb = await openStoredBackupsDatabase(wex.ws.idb);
-  const bd = await backupsDb.runAllStoresReadWriteTx(async (tx) => {
+  const bd = await backupsDb.runAllStoresReadWriteTx({}, async (tx) => {
     const backupMeta = tx.backupMeta.get(name);
     if (!backupMeta) {
       throw Error("backup not found");
@@ -631,7 +635,7 @@ async function handlePrepareWithdrawExchange(
   }
   const exchangeBaseUrl = parsedUri.exchangeBaseUrl;
   const exchange = await fetchFreshExchange(wex, exchangeBaseUrl);
-  if (exchange.masterPub != parsedUri.exchangePub) {
+  if (parsedUri.exchangePub && exchange.masterPub != parsedUri.exchangePub) {
     throw Error("mismatch of exchange master public key (URI vs actual)");
   }
   if (parsedUri.amount) {
@@ -663,6 +667,7 @@ export interface PendingOperationsResponse {
  */
 async function dispatchRequestInternal<Op extends WalletApiOperation>(
   wex: WalletExecutionContext,
+  cts: CancellationToken.Source,
   operation: WalletApiOperation,
   payload: unknown,
 ): Promise<WalletCoreResponseType<typeof operation>> {
@@ -688,19 +693,18 @@ async function dispatchRequestInternal<Op extends WalletApiOperation>(
       await recoverStoredBackup(wex, req);
       return {};
     }
+    case WalletApiOperation.SetWalletRunConfig:
     case WalletApiOperation.InitWallet: {
       const req = codecForInitRequest().decode(payload);
 
-      logger.info(`init request: ${req}`);
+      logger.info(`init request: ${j2s(req)}`);
 
       if (wex.ws.initCalled) {
-        logger.warn(
-          "initWallet called twice, new run configuration is ignored",
-        );
-        return;
+        logger.info("initializing wallet (repeat initialization)");
+      } else {
+        logger.info("initializing wallet (first initialization)");
       }
 
-      logger.trace("initializing wallet");
       // Write to the DB to make sure that we're failing early in
       // case the DB is not writeable.
       try {
@@ -794,6 +798,9 @@ async function dispatchRequestInternal<Op extends WalletApiOperation>(
       });
       return {};
     }
+    case WalletApiOperation.TestingPing: {
+      return {};
+    }
     case WalletApiOperation.UpdateExchangeEntry: {
       const req = codecForUpdateExchangeEntryRequest().decode(payload);
       await fetchFreshExchange(wex, req.exchangeBaseUrl, {
@@ -886,27 +893,7 @@ async function dispatchRequestInternal<Op extends WalletApiOperation>(
     case WalletApiOperation.GetWithdrawalDetailsForAmount: {
       const req =
         codecForGetWithdrawalDetailsForAmountRequest().decode(payload);
-      const wi = await getExchangeWithdrawalInfo(
-        wex,
-        req.exchangeBaseUrl,
-        Amounts.parseOrThrow(req.amount),
-        req.restrictAge,
-      );
-      let numCoins = 0;
-      for (const x of wi.selectedDenoms.selectedDenoms) {
-        numCoins += x.count;
-      }
-      const resp: WithdrawalDetailsForAmount = {
-        amountRaw: req.amount,
-        amountEffective: Amounts.stringify(wi.selectedDenoms.totalCoinValue),
-        paytoUris: wi.exchangePaytoUris,
-        tosAccepted: wi.termsOfServiceAccepted,
-        ageRestrictionOptions: wi.ageRestrictionOptions,
-        withdrawalAccountsList: wi.exchangeCreditAccountDetails,
-        numCoins,
-        // FIXME: Once we have proper scope info support, return correct info here.
-        scopeInfo: wi.scopeInfo,
-      };
+      const resp = await getWithdrawalDetailsForAmount(wex, cts, req);
       return resp;
     }
     case WalletApiOperation.GetBalances: {
@@ -965,7 +952,18 @@ async function dispatchRequestInternal<Op extends WalletApiOperation>(
     }
     case WalletApiOperation.GetContractTermsDetails: {
       const req = codecForGetContractTermsDetails().decode(payload);
-      return getContractTermsDetails(wex, req.proposalId);
+      if (req.proposalId) {
+        // FIXME: deprecated path
+        return getContractTermsDetails(wex, req.proposalId);
+      }
+      if (req.transactionId) {
+        const parsedTx = parseTransactionIdentifier(req.transactionId);
+        if (parsedTx?.tag === TransactionType.Payment) {
+          return getContractTermsDetails(wex, parsedTx.proposalId);
+        }
+        throw Error("transactionId is not a payment transaction");
+      }
+      throw Error("transactionId missing");
     }
     case WalletApiOperation.RetryPendingNow: {
       logger.error("retryPendingNow currently not implemented");
@@ -1193,7 +1191,7 @@ async function dispatchRequestInternal<Op extends WalletApiOperation>(
     }
     case WalletApiOperation.PrepareDeposit: {
       const req = codecForPrepareDepositRequest().decode(payload);
-      return await prepareDepositGroup(wex, req);
+      return await checkDepositGroup(wex, req);
     }
     case WalletApiOperation.GenerateDepositGroupTxId:
       return {
@@ -1221,9 +1219,11 @@ async function dispatchRequestInternal<Op extends WalletApiOperation>(
     case WalletApiOperation.TestCrypto: {
       return await wex.cryptoApi.hashString({ str: "hello world" });
     }
-    case WalletApiOperation.ClearDb:
+    case WalletApiOperation.ClearDb: {
+      wex.ws.clearAllCaches();
       await clearDatabase(wex.db.idbHandle());
       return {};
+    }
     case WalletApiOperation.Recycle: {
       throw Error("not implemented");
       return {};
@@ -1275,6 +1275,7 @@ async function dispatchRequestInternal<Op extends WalletApiOperation>(
         if (existingRec) {
           return;
         }
+        wex.ws.exchangeCache.clear();
         await tx.globalCurrencyExchanges.add({
           currency: req.currency,
           exchangeBaseUrl: req.exchangeBaseUrl,
@@ -1294,6 +1295,7 @@ async function dispatchRequestInternal<Op extends WalletApiOperation>(
         if (!existingRec) {
           return;
         }
+        wex.ws.exchangeCache.clear();
         checkDbInvariant(!!existingRec.id);
         await tx.globalCurrencyExchanges.delete(existingRec.id);
       });
@@ -1315,6 +1317,7 @@ async function dispatchRequestInternal<Op extends WalletApiOperation>(
           auditorBaseUrl: req.auditorBaseUrl,
           auditorPub: req.auditorPub,
         });
+        wex.ws.exchangeCache.clear();
       });
       return {};
     }
@@ -1331,6 +1334,7 @@ async function dispatchRequestInternal<Op extends WalletApiOperation>(
         }
         checkDbInvariant(!!existingRec.id);
         await tx.globalCurrencyAuditors.delete(existingRec.id);
+        wex.ws.exchangeCache.clear();
       });
       return {};
     }
@@ -1448,14 +1452,14 @@ export function getVersion(wex: WalletExecutionContext): WalletCoreVersion {
     implementationSemver: walletCoreBuildInfo.implementationSemver,
     implementationGitHash: walletCoreBuildInfo.implementationGitHash,
     hash: undefined,
-    version: WALLET_CORE_API_IMPLEMENTATION_VERSION,
+    version: WALLET_CORE_API_PROTOCOL_VERSION,
     exchange: WALLET_EXCHANGE_PROTOCOL_VERSION,
     merchant: WALLET_MERCHANT_PROTOCOL_VERSION,
     bankConversionApiRange: WALLET_BANK_CONVERSION_API_PROTOCOL_VERSION,
     bankIntegrationApiRange: WALLET_BANK_INTEGRATION_PROTOCOL_VERSION,
     corebankApiRange: WALLET_COREBANK_API_PROTOCOL_VERSION,
     bank: WALLET_BANK_INTEGRATION_PROTOCOL_VERSION,
-    devMode: false,
+    devMode: wex.ws.config.testing.devModeActive,
   };
   return result;
 }
@@ -1511,6 +1515,8 @@ async function handleCoreApiRequest(
   let wex: WalletExecutionContext;
   let oc: ObservabilityContext;
 
+  const cts = CancellationToken.create();
+
   if (ws.initCalled && ws.config.testing.emitObservabilityEvents) {
     oc = {
       observe(evt) {
@@ -1523,26 +1529,30 @@ async function handleCoreApiRequest(
       },
     };
 
-    wex = getObservedWalletExecutionContext(ws, CancellationToken.CONTINUE, oc);
+    wex = getObservedWalletExecutionContext(ws, cts.token, oc);
   } else {
     oc = {
       observe(evt) {},
     };
-    wex = getNormalWalletExecutionContext(ws, CancellationToken.CONTINUE, oc);
+    wex = getNormalWalletExecutionContext(ws, cts.token, oc);
   }
 
   try {
+    const start = performanceNow();
     await ws.ensureWalletDbOpen();
     oc.observe({
       type: ObservabilityEventType.RequestStart,
     });
     const result = await dispatchRequestInternal(
       wex,
+      cts,
       operation as any,
       payload,
     );
+    const end = performanceNow();
     oc.observe({
       type: ObservabilityEventType.RequestFinishSuccess,
+      durationMs: Number((end - start) / 1000n / 1000n),
     });
     return {
       type: "response",
@@ -1657,6 +1667,48 @@ export class Wallet {
   }
 }
 
+export interface DevExperimentState {
+  blockRefreshes?: boolean;
+}
+
+export class Cache<T> {
+  private map: Map<string, [AbsoluteTime, T]> = new Map();
+
+  constructor(
+    private maxCapacity: number,
+    private cacheDuration: Duration,
+  ) {}
+
+  get(key: string): T | undefined {
+    const r = this.map.get(key);
+    if (!r) {
+      return undefined;
+    }
+
+    if (AbsoluteTime.isExpired(r[0])) {
+      this.map.delete(key);
+      return undefined;
+    }
+
+    return r[1];
+  }
+
+  clear(): void {
+    this.map.clear();
+  }
+
+  put(key: string, value: T): void {
+    if (this.map.size > this.maxCapacity) {
+      this.map.clear();
+    }
+    const expiry = AbsoluteTime.addDuration(
+      AbsoluteTime.now(),
+      this.cacheDuration,
+    );
+    this.map.set(key, [expiry, value]);
+  }
+}
+
 /**
  * Internal state of the wallet.
  *
@@ -1674,7 +1726,20 @@ export class InternalWalletState {
 
   initCalled = false;
 
-  private denomCache: Map<string, DenominationInfo> = new Map();
+  refreshCostCache: Cache<AmountJson> = new Cache(
+    1000,
+    Duration.fromSpec({ minutes: 1 }),
+  );
+
+  denomInfoCache: Cache<DenominationInfo> = new Cache(
+    1000,
+    Duration.fromSpec({ minutes: 1 }),
+  );
+
+  exchangeCache: Cache<ReadyExchangeSummary> = new Cache(
+    1000,
+    Duration.fromSpec({ minutes: 1 }),
+  );
 
   /**
    * Promises that are waiting for a particular resource.
@@ -1701,23 +1766,21 @@ export class InternalWalletState {
     return this._db;
   }
 
-  lookupDenomCache(denomCacheKey: string): DenominationInfo | undefined {
-    return this.denomCache.get(denomCacheKey);
-  }
+  devExperimentState: DevExperimentState = {};
 
-  putDenomCache(denomCacheKey: string, denomInfo: DenominationInfo): void {
-    if (this.denomCache.size > 1000) {
-      this.denomCache.clear();
-    }
-    this.denomCache.set(denomCacheKey, denomInfo);
+  clientCancellationMap: Map<string, CancellationToken.Source> = new Map();
+
+  clearAllCaches(): void {
+    this.exchangeCache.clear();
+    this.denomInfoCache.clear();
+    this.refreshCostCache.clear();
   }
 
   initWithConfig(newConfig: WalletRunConfig): void {
-    if (this._config) {
-      throw Error("config already set");
-    }
     this._config = newConfig;
 
+    logger.info(`setting new config to ${j2s(newConfig)}`);
+
     this._http = this.httpFactory(newConfig);
 
     if (this.config.testing.devModeActive) {
