@@ -35,6 +35,7 @@ import {
   CoreApiResponse,
   CoreApiResponseSuccess,
   Logger,
+  PartialWalletRunConfig,
   WalletNotification,
   enableNativeLogging,
   getErrorDetailFromException,
@@ -68,6 +69,7 @@ function sendNativeMessage(ev: CoreApiMessageEnvelope): void {
 
 class NativeWalletMessageHandler {
   walletArgs: DefaultNodeWalletArgs | undefined;
+  walletConfig: PartialWalletRunConfig | undefined;
   maybeWallet: Wallet | undefined;
   wp = openPromise<Wallet>();
   httpLib = createPlatformHttpLib();
@@ -99,7 +101,9 @@ class NativeWalletMessageHandler {
       const resp = await w.handleCoreApiRequest(
         "initWallet",
         "native-init",
-        {},
+        {
+          config: this.walletConfig
+        },
       );
       initResponse = resp.type == "response" ? resp.result : resp.error;
       w.runTaskLoop().catch((e) => {
@@ -121,6 +125,7 @@ class NativeWalletMessageHandler {
           cryptoWorkerType: args.cryptoWorkerType,
           ...args,
         };
+        this.walletConfig = args.config ?? {};
         const logLevel = args.logLevel;
         if (logLevel) {
           setGlobalLogLevelFromString(logLevel);
