@@ -33,14 +33,13 @@ import { useBackendContext } from "../context/backend.js";
 import { useAsyncAsHook } from "../hooks/useAsyncAsHook.js";
 import { useSettings } from "../hooks/useSettings.js";
 import { Button } from "../mui/Button.js";
+import { WxApiType } from "../wxApi.js";
 import { Modal } from "./Modal.js";
 import { Time } from "./Time.js";
-import { WalletEvent, WxApiType } from "../wxApi.js";
 
 interface Props extends JSX.HTMLAttributes {
 }
 
-
 export function WalletActivity({ }: Props): VNode {
   const { i18n } = useTranslationContext()
   const [settings, updateSettings] = useSettings()
@@ -263,6 +262,7 @@ function ShowObservabilityEvent({ events, onClick }: MoreInfoPRops): VNode {
         case ObservabilityEventType.TaskReset: return "Task reset"
         case ObservabilityEventType.ShepherdTaskResult: return "Schedule"
         case ObservabilityEventType.DeclareTaskDependency: return "Task dependency"
+        case ObservabilityEventType.Message: return "Message"
       }
     })();
 
@@ -492,6 +492,9 @@ function ShowObervavilityDetails({ title, notif, onClick, prev }: { title: strin
         </td>
       </tr>
     }
+    case ObservabilityEventType.Message:
+      // FIXME
+      return <></>;
   }
 }
 
