@@ -35,7 +35,6 @@ export const WalletAction = styled.div`
   align-items: center;
 
   margin: auto;
-  height: 100%;
 
   & h1:first-child {
     margin-top: 0;
@@ -540,7 +539,7 @@ export const LinkPrimary = styled(Link)`
   color: black;
 `;
 
-export const ButtonPrimary = styled(ButtonVariant)<{ small?: boolean }>`
+export const ButtonPrimary = styled(ButtonVariant) <{ small?: boolean }>`
   font-size: ${({ small }: any) => (small ? "small" : "inherit")};
   background-color: #0042b2;
   border-color: #0042b2;
@@ -972,7 +971,7 @@ export const TermsSection = styled.a`
   }
 `;
 
-export const TermsOfService = styled.div`
+export const TermsOfServiceStyle = styled.div`
   display: flex;
   flex-direction: column;
   text-align: left;
