@@ -54,7 +54,7 @@ export function useComponentStateFromParams({
       : undefined;
     const exchangeByTalerUri = uri?.exchangeBaseUrl;
     let ex: ExchangeFullDetails | undefined;
-    if (exchangeByTalerUri && uri.exchangePub) {
+    if (exchangeByTalerUri) {
       await api.wallet.call(WalletApiOperation.AddExchange, {
         exchangeBaseUrl: exchangeByTalerUri,
         masterPub: uri.exchangePub,
@@ -141,8 +141,8 @@ export function useComponentStateFromParams({
         confirm: {
           onClick: isValid
             ? pushAlertOnError(async () => {
-                onAmountChanged(Amounts.stringify(amount));
-              })
+              onAmountChanged(Amounts.stringify(amount));
+            })
             : undefined,
         },
         amount: {
@@ -439,12 +439,12 @@ function exchangeSelectionState(
     //TODO: calculate based on exchange info
     const ageRestriction = ageRestrictionEnabled
       ? {
-          list: ageRestrictionOptions,
-          value: String(ageRestricted),
-          onChange: pushAlertOnError(async (v: string) =>
-            setAgeRestricted(parseInt(v, 10)),
-          ),
-        }
+        list: ageRestrictionOptions,
+        value: String(ageRestricted),
+        onChange: pushAlertOnError(async (v: string) =>
+          setAgeRestricted(parseInt(v, 10)),
+        ),
+      }
       : undefined;
 
     const altCurrencies = amountHook.response.accounts
@@ -454,6 +454,7 @@ function exchangeSelectionState(
       altCurrencies.length === 0
         ? []
         : [toBeReceived.currency, ...altCurrencies];
+
     const convAccount = amountHook.response.accounts.find((c) => {
       return (
         c.currencySpecification &&
@@ -463,9 +464,9 @@ function exchangeSelectionState(
     const conversionInfo = !convAccount
       ? undefined
       : {
-          spec: convAccount.currencySpecification!,
-          amount: Amounts.parseOrThrow(convAccount.transferAmount!),
-        };
+        spec: convAccount.currencySpecification!,
+        amount: Amounts.parseOrThrow(convAccount.transferAmount!),
+      };
 
     return {
       status: "success",
