@@ -17,10 +17,10 @@
 import {
   CoreApiResponse,
   TalerUri,
-  WalletNotification
+  WalletNotification,
+  WalletRunConfig
 } from "@gnu-taler/taler-util";
 import {
-  WalletConfig,
   WalletOperations
 } from "@gnu-taler/taler-wallet-core";
 import {
@@ -46,20 +46,36 @@ export interface Permissions {
  * Compatibility API that works on multiple browsers.
  */
 export interface CrossBrowserPermissionsApi {
-  // containsHostPermissions(): Promise<boolean>;
-  // requestHostPermissions(): Promise<boolean>;
-  // removeHostPermissions(): Promise<boolean>;
 
   containsClipboardPermissions(): Promise<boolean>;
   requestClipboardPermissions(): Promise<boolean>;
   removeClipboardPermissions(): Promise<boolean>;
 
-  addPermissionsListener(
-    callback: (p: Permissions, lastError?: string) => void,
-  ): void;
 }
 
-export type MessageFromBackend = WalletNotification;
+export enum ExtensionNotificationType {
+  SettingsChange = "settings-change",
+  ClearNotifications = "clear-notifications",
+}
+
+export interface SettingsChangeNotification {
+  type: ExtensionNotificationType.SettingsChange;
+
+  currentValue: Settings;
+}
+export interface ClearNotificaitonNotification {
+  type: ExtensionNotificationType.ClearNotifications;
+}
+
+export type ExtensionNotification = SettingsChangeNotification | ClearNotificaitonNotification
+
+export type MessageFromBackend = {
+  type: "wallet",
+  notification: WalletNotification
+} | {
+  type: "web-extension",
+  notification: ExtensionNotification
+};
 
 export type MessageFromFrontend<
   Op extends BackgroundOperations | WalletOperations | ExtensionOperations,
@@ -92,7 +108,7 @@ export interface WalletWebExVersion {
   version: string;
 }
 
-type F = WalletConfig["features"];
+type F = WalletRunConfig["features"];
 type kf = keyof F;
 type WebexWalletConfig = {
   [P in keyof F as `wallet${Capitalize<P>}`]: F[P];
@@ -101,24 +117,32 @@ type WebexWalletConfig = {
 export interface Settings extends WebexWalletConfig {
   injectTalerSupport: boolean;
   autoOpen: boolean;
-  advanceMode: boolean;
+  advancedMode: boolean;
   backup: boolean;
   langSelector: boolean;
   showJsonOnError: boolean;
   extendedAccountTypes: boolean;
+  showRefeshTransactions: boolean;
   suspendIndividualTransaction: boolean;
+  showExchangeManagement: boolean;
+  selectTosFormat: boolean;
+  showWalletActivity: boolean;
 }
 
 export const defaultSettings: Settings = {
   injectTalerSupport: true,
   autoOpen: true,
-  advanceMode: false,
+  advancedMode: false,
   backup: false,
   langSelector: false,
+  showRefeshTransactions: false,
   suspendIndividualTransaction: false,
   showJsonOnError: false,
   extendedAccountTypes: false,
+  showExchangeManagement: false,
   walletAllowHttp: false,
+  selectTosFormat: false,
+  showWalletActivity: false,
 };
 
 /**
@@ -209,6 +233,11 @@ export interface BackgroundPlatformAPI {
 
 }
 export interface ForegroundPlatformAPI {
+  /**
+   * Check if the extension is running under
+   * chrome incognito or firefox private mode.
+   */
+  runningOnPrivateMode(): boolean;
   /**
    * FIXME: should not be needed
    *
@@ -279,6 +308,12 @@ export interface ForegroundPlatformAPI {
     message: MessageFromFrontend<Op>,
   ): Promise<MessageResponse>;
 
+  /**
+ * Used by the wallet frontend to send notification about new information
+ * @param message
+ */
+  triggerWalletEvent(message: MessageFromBackend): void;
+
   /**
    * Used from the frontend to receive notifications about new information
    * @param listener
