@@ -221,6 +221,13 @@ function openWalletURIFromPopup(uri: TalerUri): void {
         )}`,
       );
       break;
+    case TalerUriAction.AddExchange:
+      url = chrome.runtime.getURL(
+        `static/wallet.html#/cta/add/exchange?talerUri=${encodeURIComponent(
+          talerUri,
+        )}`,
+      );
+      break;
     case TalerUriAction.DevExperiment:
       logger.warn(`taler://dev-experiment URIs are not allowed in headers`);
       return;
@@ -274,7 +281,11 @@ async function sendMessageToBackground<
     let timedout = false;
     const timerId = setTimeout(() => {
       timedout = true;
-      reject(TalerError.fromDetail(TalerErrorCode.GENERIC_TIMEOUT, {}));
+      reject(TalerError.fromDetail(TalerErrorCode.GENERIC_TIMEOUT, {
+        requestMethod: "wallet",
+        requestUrl: message.operation,
+        timeoutMs: 20 * 1000,
+      }));
     }, 20 * 1000);
     chrome.runtime.sendMessage(messageWithId, (backgroundResponse) => {
       if (timedout) {
@@ -470,26 +481,26 @@ function setAlertedIcon(): void {
 
 interface OffscreenCanvasRenderingContext2D
   extends CanvasState,
-    CanvasTransform,
-    CanvasCompositing,
-    CanvasImageSmoothing,
-    CanvasFillStrokeStyles,
-    CanvasShadowStyles,
-    CanvasFilters,
-    CanvasRect,
-    CanvasDrawPath,
-    CanvasUserInterface,
-    CanvasText,
-    CanvasDrawImage,
-    CanvasImageData,
-    CanvasPathDrawingStyles,
-    CanvasTextDrawingStyles,
-    CanvasPath {
+  CanvasTransform,
+  CanvasCompositing,
+  CanvasImageSmoothing,
+  CanvasFillStrokeStyles,
+  CanvasShadowStyles,
+  CanvasFilters,
+  CanvasRect,
+  CanvasDrawPath,
+  CanvasUserInterface,
+  CanvasText,
+  CanvasDrawImage,
+  CanvasImageData,
+  CanvasPathDrawingStyles,
+  CanvasTextDrawingStyles,
+  CanvasPath {
   readonly canvas: OffscreenCanvas;
 }
 declare const OffscreenCanvasRenderingContext2D: {
   prototype: OffscreenCanvasRenderingContext2D;
-  new (): OffscreenCanvasRenderingContext2D;
+  new(): OffscreenCanvasRenderingContext2D;
 };
 
 interface OffscreenCanvas extends EventTarget {
@@ -502,7 +513,7 @@ interface OffscreenCanvas extends EventTarget {
 }
 declare const OffscreenCanvas: {
   prototype: OffscreenCanvas;
-  new (width: number, height: number): OffscreenCanvas;
+  new(width: number, height: number): OffscreenCanvas;
 };
 
 function createCanvas(size: number): OffscreenCanvas {
