@@ -29,6 +29,7 @@ import {
 const logger = new Logger("dev.ts");
 
 const api: BackgroundPlatformAPI & ForegroundPlatformAPI = {
+  runningOnPrivateMode: () => false,
   isFirefox: () => false,
   getSettingsFromStorage: () => Promise.resolve(defaultSettings),
   keepAlive: (cb: VoidFunction) => cb(),
@@ -36,19 +37,13 @@ const api: BackgroundPlatformAPI & ForegroundPlatformAPI = {
   findTalerUriInClipboard: async () => undefined,
   listenNetworkConnectionState,
   openNewURLFromPopup: () => undefined,
+  triggerWalletEvent: () => undefined,
   getPermissionsApi: () => ({
-    addPermissionsListener: () => undefined,
-    containsHostPermissions: async () => true,
-    removeHostPermissions: async () => false,
-    requestHostPermissions: async () => false,
     containsClipboardPermissions: async () => true,
     removeClipboardPermissions: async () => false,
     requestClipboardPermissions: async () => false,
   }),
 
-  // registerDeclarativeRedirect: () => false,
-  // registerTalerHeaderListener: () => false,
-  // containsTalerHeaderListener: () => false,
   getWalletWebExVersion: () => ({
     version: "none",
   }),
