@@ -105,7 +105,8 @@ function useComponentState({
   if (state.hasError) {
     return {
       status: "error",
-      error: alertFromError(i18n.str`Could not load the balance`, state),
+      error: alertFromError(          i18n,
+        i18n.str`Could not load the balance`, state),
     };
   }
   if (addingAction) {
@@ -153,7 +154,10 @@ export function BalanceView(state: State.Balances): VNode {
   const { i18n } = useTranslationContext();
   const currencyWithNonZeroAmount = state.balances
     .filter((b) => !Amounts.isZero(b.available))
-    .map((b) => b.available.split(":")[0]);
+    .map((b) => {
+      b.flags
+      return b.available.split(":")[0]
+    });
 
   if (state.balances.length === 0) {
     return (
