@@ -14,9 +14,9 @@
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
 
-import { ExchangeEntryStatus, OperationFailWithBody, OperationOk, TalerExchangeApi, TalerExchangeHttpClient, canonicalizeBaseUrl, opKnownFailureWithBody } from "@gnu-taler/taler-util";
+import { ExchangeEntryStatus, TalerExchangeHttpClient, canonicalizeBaseUrl, opKnownFailureWithBody } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
-import { BrowserHttpLib } from "@gnu-taler/web-util/browser";
+import { BrowserFetchHttpLib } from "@gnu-taler/web-util/browser";
 import { useCallback, useEffect, useState } from "preact/hooks";
 import { useBackendContext } from "../../context/backend.js";
 import { useAsyncAsHook } from "../../hooks/useAsyncAsHook.js";
@@ -65,8 +65,27 @@ export function useComponentState({ onBack, currency, noDebounce }: Props): Recu
         if (found !== -1) {
           return opKnownFailureWithBody<CheckExchangeErrors>("already-active", undefined);
         }
-        const api = new TalerExchangeHttpClient(baseUrl.href, new BrowserHttpLib() as any);
+
+        /**
+         * FIXME: For some reason typescript doesn't like the next BrowserFetchHttpLib
+         * 
+         * │ src/wallet/AddExchange/state.ts(68,63): error TS2345: Argument of type 'BrowserFetchHttpLib' is not assignable to parameter of ty
+         * │   Types of property 'fetch' are incompatible.
+         * │     Type '(requestUrl: string, options?: HttpRequestOptions | undefined) => Promise<HttpResponse>' is not assignable to type '(ur
+         * │       Types of parameters 'options' and 'opt' are incompatible.
+         * │         Type 'import("$PATH/wallet.git/packages/taler-util/lib/http-common", { wi
+         * │           Type 'import("$PATH/wallet.git/packages/taler-util/lib/http-common", { 
+         * │             Types of property 'cancellationToken' are incompatible.
+         * │               Type 'import("$PATH/wallet.git/packages/taler-util/lib/Cancellation
+         * │                 Type 'import("$PATH/wallet.git/packages/taler-util/lib/Cancellati
+         * │                   Types have separate declarations of a private property '_isCancelled'.
+         *
+         */
+        const api = new TalerExchangeHttpClient(baseUrl.href, new BrowserFetchHttpLib() as any);
         const config = await api.getConfig()
+        if (config.type === "fail") {
+          return opKnownFailureWithBody<CheckExchangeErrors>("not-found", undefined)
+        }
         if (!api.isCompatible(config.body.version)) {
           return opKnownFailureWithBody<CheckExchangeErrors>("invalid-version", config.body.version)
         }
@@ -139,7 +158,7 @@ function useDebounce<T>(
   const [result, setResult] = useState<T | undefined>(undefined);
   const [error, setError] = useState<Error | undefined>(undefined);
 
-  const [handler, setHandler] = useState<any | undefined>(undefined);
+  const [handler, setHandler] = useState<number | undefined>(undefined);
 
   if (!disabled) {
     useEffect(() => {
@@ -164,7 +183,7 @@ function useDebounce<T>(
           setResult(undefined);
         }
       }, 500);
-      setHandler(h);
+      setHandler(h as unknown as number);
     }, [value, setHandler, onTrigger]);
   }
 
