@@ -23,7 +23,9 @@
 import {
   Amounts,
   TalerUri,
+  TalerUriAction,
   TranslatedString,
+  parseTalerUri,
   stringifyTalerUri,
 } from "@gnu-taler/taler-util";
 import {
@@ -83,6 +85,8 @@ import { TransactionPage } from "./Transaction.js";
 import { WelcomePage } from "./Welcome.js";
 import { WalletActivity } from "../components/WalletActivity.js";
 import { EnabledBySettings } from "../components/EnabledBySettings.js";
+import { DevExperimentPage } from "../cta/DevExperiment/index.js";
+import { ConfirmAddExchangeView } from "./AddExchange/views.js";
 
 export function Application(): VNode {
   const { i18n } = useTranslationContext();
@@ -497,7 +501,40 @@ export function Application(): VNode {
               </CallToActionTemplate>
             )}
           />
-
+          <Route
+            path={Pages.ctaExperiment}
+            component={({ talerUri }: { talerUri: string }) => (
+              <CallToActionTemplate title={i18n.str`Development experiment`}>
+                <DevExperimentPage
+                  talerExperimentUri={decodeURIComponent(talerUri)}
+                  onCancel={() => redirectTo(Pages.balanceHistory({}))}
+                  onSuccess={() => redirectTo(Pages.balanceHistory({}))}
+                />
+              </CallToActionTemplate>
+            )}
+          />
+          <Route
+            path={Pages.ctaAddExchange}
+            component={({ talerUri }: { talerUri: string }) => {
+              const tUri = parseTalerUri(decodeURIComponent(talerUri))
+              const baseUrl = tUri?.type === TalerUriAction.AddExchange ? tUri.exchangeBaseUrl : undefined
+              if (!baseUrl) {
+                redirectTo(Pages.balanceHistory({}))
+                return <div>
+                  invalid url {talerUri}
+                </div>
+              }
+              return <CallToActionTemplate title={i18n.str`Add exchange`}>
+                <ConfirmAddExchangeView
+                  url={baseUrl}
+                  status="confirm"
+                  error={undefined}
+                  onCancel={() => redirectTo(Pages.balanceHistory({}))}
+                  onConfirm={() => redirectTo(Pages.balanceHistory({}))}
+                />
+              </CallToActionTemplate>
+            }}
+          />
           {/**
            * NOT FOUND
            * all redirects should be at the end
