@@ -15,16 +15,19 @@
  */
 
 import {
+  assertUnreachable,
   parseTalerUri,
   TalerUri,
+  TalerUriAction,
   TranslatedString,
 } from "@gnu-taler/taler-util";
 import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { css } from "@linaria/core";
 import { styled } from "@linaria/react";
 import jsQR, * as pr from "jsqr";
-import { Fragment, h, VNode } from "preact";
+import { h, VNode } from "preact";
 import { useRef, useState } from "preact/hooks";
+import { EnabledBySettings } from "../components/EnabledBySettings.js";
 import { Alert } from "../mui/Alert.js";
 import { Button } from "../mui/Button.js";
 import { Grid } from "../mui/Grid.js";
@@ -182,7 +185,7 @@ async function createCanvasFromFile(
   canvas.width = img.width;
   canvas.height = img.height;
   return new Promise<string | undefined>((ok, bad) => {
-    img.addEventListener("load", (e) => {
+    img.addEventListener("load", () => {
       try {
         const code = drawIntoCanvasAndGetQR(img, canvas);
         ok(code);
@@ -194,7 +197,7 @@ async function createCanvasFromFile(
 }
 
 async function waitUntilReady(video: HTMLVideoElement): Promise<void> {
-  return new Promise((ok, bad) => {
+  return new Promise((ok, _bad) => {
     if (video.readyState === video.HAVE_ENOUGH_DATA) {
       return ok();
     }
@@ -211,8 +214,25 @@ export function QrReaderPage({ onDetected }: Props): VNode {
 
   const { i18n } = useTranslationContext();
 
+  function onChangeDetect(str: string) {
+    if (str) {
+      const uri = parseTalerUri(str);
+      if (!uri) {
+        setError(
+          i18n.str`URI is not valid. Taler URI should start with "taler://"`,
+        );
+      } else {
+        onDetected(uri);
+        setError(undefined);
+      }
+    } else {
+      setError(undefined);
+    }
+    setValue(str);
+  }
+
   function onChange(str: string) {
-    if (!!str) {
+    if (str) {
       if (!parseTalerUri(str)) {
         setError(
           i18n.str`URI is not valid. Taler URI should start with "taler://"`,
@@ -244,7 +264,7 @@ export function QrReaderPage({ onDetected }: Props): VNode {
     try {
       const code = await createCanvasFromVideo(video, canvasRef.current);
       if (code) {
-        onChange(code);
+        onChangeDetect(code);
         setShow("canvas");
       }
       stream.getTracks().forEach((e) => {
@@ -264,7 +284,7 @@ export function QrReaderPage({ onDetected }: Props): VNode {
     try {
       const code = await createCanvasFromFile(fileContent, canvasRef.current);
       if (code) {
-        onChange(code);
+        onChangeDetect(code);
         setShow("canvas");
       } else {
         setError(i18n.str`Could not found a QR code in the file`);
@@ -273,8 +293,8 @@ export function QrReaderPage({ onDetected }: Props): VNode {
       setError(i18n.str`something unexpected happen: ${error}`);
     }
   }
+  const uri = parseTalerUri(value);
 
-  const active = value === "";
   return (
     <Container>
       <section>
@@ -283,59 +303,75 @@ export function QrReaderPage({ onDetected }: Props): VNode {
             Scan a QR code or enter taler:// URI below
           </i18n.Translate>
         </h1>
-
-        <p>
-          <TextField
-            label="Taler URI"
-            variant="standard"
-            fullWidth
-            value={value}
-            onChange={onChange}
-          />
-        </p>
+        <div style={{ justifyContent: "space-between", display: "flex" }}>
+          <div style={{ width: "75%" }}>
+            <TextField
+              label="Taler URI"
+              variant="standard"
+              fullWidth
+              value={value}
+              onChange={onChange}
+            />
+          </div>
+          {uri && (
+            <Button
+              disabled={!!error}
+              variant="contained"
+              color="success"
+              onClick={async () => {
+                if (uri) onDetected(uri);
+              }}
+            >
+              {(function (talerUri: TalerUri): VNode {
+                switch (talerUri.type) {
+                  case TalerUriAction.Pay:
+                    return <i18n.Translate>Pay invoice</i18n.Translate>;
+                  case TalerUriAction.Withdraw:
+                    return (
+                      <i18n.Translate>Withdrawal from bank</i18n.Translate>
+                    );
+                  case TalerUriAction.Refund:
+                    return <i18n.Translate>Claim refund</i18n.Translate>;
+                  case TalerUriAction.PayPull:
+                    return <i18n.Translate>Pay invoice</i18n.Translate>;
+                  case TalerUriAction.PayPush:
+                    return <i18n.Translate>Accept payment</i18n.Translate>;
+                  case TalerUriAction.PayTemplate:
+                    return <i18n.Translate>Complete order</i18n.Translate>;
+                  case TalerUriAction.Restore:
+                    return <i18n.Translate>Restore wallet</i18n.Translate>;
+                  case TalerUriAction.DevExperiment:
+                    return <i18n.Translate>Enable experiment</i18n.Translate>;
+                  case TalerUriAction.WithdrawExchange:
+                    return (
+                      <i18n.Translate>Withdraw from exchange</i18n.Translate>
+                    );
+                  case TalerUriAction.AddExchange:
+                    return <i18n.Translate>Add exchange</i18n.Translate>;
+                  default: {
+                    assertUnreachable(talerUri);
+                  }
+                }
+              })(uri)}
+            </Button>
+          )}
+        </div>
         <Grid container justifyContent="space-around" columns={2}>
           <Grid item xs={2}>
             <p>{error && <Alert severity="error">{error}</Alert>}</p>
           </Grid>
-          <Grid item xs={1}>
-            {!active && (
-              <Button
-                variant="contained"
-                onClick={async () => {
-                  setShow("nothing");
-                  onChange("");
-                }}
-                color="error"
-              >
-                <i18n.Translate>Clear</i18n.Translate>
-              </Button>
-            )}
-          </Grid>
-          <Grid item xs={1}>
-            {value && (
-              <Button
-                disabled={!!error}
-                variant="contained"
-                color="success"
-                onClick={async () => {
-                  const uri = parseTalerUri(value);
-                  if (uri) onDetected(uri);
-                }}
-              >
-                <i18n.Translate>Open</i18n.Translate>
-              </Button>
-            )}
-          </Grid>
-          <Grid item xs={1}>
-            <InputFile onChange={onFileRead}>Read QR from file</InputFile>
-          </Grid>
-          <Grid item xs={1}>
+          <Grid item xs={2}>
             <p>
               <Button variant="contained" onClick={startVideo}>
                 Use Camera
               </Button>
             </p>
           </Grid>
+          <EnabledBySettings name="advancedMode">
+            <Grid item xs={2}>
+              <InputFile onChange={onFileRead}>Read QR from file</InputFile>
+            </Grid>
+          </EnabledBySettings>
         </Grid>
       </section>
       <div>
