@@ -15,28 +15,21 @@
  */
 
 import {
-  ExchangeListItem,
-  ExchangeTosStatus,
   LibtoolVersion,
   TranslatedString,
-  WalletCoreVersion,
+  WalletCoreVersion
 } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
 import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { Fragment, VNode, h } from "preact";
-import { Pages } from "../NavigationBar.js";
 import { Checkbox } from "../components/Checkbox.js";
 import { EnabledBySettings } from "../components/EnabledBySettings.js";
 import { Part } from "../components/Part.js";
 import { SelectList } from "../components/SelectList.js";
 import {
-  DestructiveText,
   Input,
-  LinkPrimary,
   SubTitle,
-  SuccessText,
-  WarningBox,
-  WarningText,
+  WarningBox
 } from "../components/styled/index.js";
 import { useAlertContext } from "../context/alert.js";
 import { useBackendContext } from "../context/backend.js";
@@ -57,19 +50,15 @@ export function SettingsPage(): VNode {
   const webex = platform.getWalletWebExVersion();
   const api = useBackendContext();
 
-  const exchangesHook = useAsyncAsHook(async () => {
-    const list = await api.wallet.call(WalletApiOperation.ListExchanges, {});
+  const hook = useAsyncAsHook(async () => {
     const version = await api.wallet.call(WalletApiOperation.GetVersion, {});
-    return { exchanges: list.exchanges, version };
+    return { version };
   });
-  const { exchanges, version } =
-    !exchangesHook || exchangesHook.hasError
-      ? { exchanges: [], version: undefined }
-      : exchangesHook.response;
+
+  const version = hook && !hook.hasError ? hook.response.version : undefined
 
   return (
     <SettingsView
-      knownExchanges={exchanges}
       deviceName={name}
       setDeviceName={update}
       autoOpenToggle={{
@@ -80,19 +69,11 @@ export function SettingsPage(): VNode {
           }),
         },
       }}
-      injectTalerToggle={{
-        value: settings.injectTalerSupport,
-        button: {
-          onClick: safely("update support injection", async () => {
-            updateSettings("injectTalerSupport", !settings.injectTalerSupport);
-          }),
-        },
-      }}
       advanceToggle={{
-        value: settings.advanceMode,
+        value: settings.advancedMode,
         button: {
           onClick: safely("update advance mode", async () => {
-            updateSettings("advanceMode", !settings.advanceMode);
+            updateSettings("advancedMode", !settings.advancedMode);
           }),
         },
       }}
@@ -117,10 +98,8 @@ export interface ViewProps {
   deviceName: string;
   setDeviceName: (s: string) => Promise<void>;
   autoOpenToggle: ToggleHandler;
-  injectTalerToggle: ToggleHandler;
   advanceToggle: ToggleHandler;
   langToggle: ToggleHandler;
-  knownExchanges: Array<ExchangeListItem>;
   coreVersion: WalletCoreVersion | undefined;
   webexVersion: {
     version: string;
@@ -129,9 +108,7 @@ export interface ViewProps {
 }
 
 export function SettingsView({
-  knownExchanges,
   autoOpenToggle,
-  injectTalerToggle,
   advanceToggle,
   langToggle,
   coreVersion,
@@ -139,123 +116,84 @@ export function SettingsView({
 }: ViewProps): VNode {
   const { i18n, lang, supportedLang, changeLanguage } = useTranslationContext();
 
+  const api = useBackendContext();
+
   return (
     <Fragment>
       <section>
         <SubTitle>
-          <i18n.Translate>Trust</i18n.Translate>
-        </SubTitle>
-        {!knownExchanges || !knownExchanges.length ? (
-          <div>
-            <i18n.Translate>No exchange yet</i18n.Translate>
-          </div>
-        ) : (
-          <Fragment>
-            <table>
-              <thead>
-                <tr>
-                  <th>
-                    <i18n.Translate>Currency</i18n.Translate>
-                  </th>
-                  <th>
-                    <i18n.Translate>URL</i18n.Translate>
-                  </th>
-                  <th>
-                    <i18n.Translate>Term of Service</i18n.Translate>
-                  </th>
-                </tr>
-              </thead>
-              <tbody>
-                {knownExchanges.map((e, idx) => {
-                  function Status(): VNode {
-                    switch (e.tosStatus) {
-                      case ExchangeTosStatus.Accepted:
-                        return (
-                          <SuccessText>
-                            <i18n.Translate>ok</i18n.Translate>
-                          </SuccessText>
-                        );
-                      case ExchangeTosStatus.Pending:
-                        return (
-                          <WarningText>
-                            <i18n.Translate>pending</i18n.Translate>
-                          </WarningText>
-                        );
-                      case ExchangeTosStatus.Proposed:
-                        return (
-                          <i18n.Translate>proposed</i18n.Translate>
-                        );
-                      default:
-                        return (
-                          <DestructiveText>
-                            <i18n.Translate>
-                              unknown (exchange status should be updated)
-                            </i18n.Translate>
-                          </DestructiveText>
-                        );
-                    }
-                  }
-                  return (
-                    <tr key={idx}>
-                      <td>{e.currency}</td>
-                      <td>
-                        <a href={e.exchangeBaseUrl}>{e.exchangeBaseUrl}</a>
-                      </td>
-                      <td>
-                        <Status />
-                      </td>
-                    </tr>
-                  );
-                })}
-              </tbody>
-            </table>
-          </Fragment>
-        )}
-        <div style={{ display: "flex", justifyContent: "space-between" }}>
-          <div />
-          <LinkPrimary href={Pages.settingsExchangeAdd({})}>
-            <i18n.Translate>Add an exchange</i18n.Translate>
-          </LinkPrimary>
-        </div>
-
-        {coreVersion && (<Fragment>
-          {LibtoolVersion.compare(coreVersion.version, WALLET_CORE_SUPPORTED_VERSION)?.compatible ? undefined :
-            <WarningBox>
-              <i18n.Translate>
-                The version of wallet core is not supported. (supported version: {WALLET_CORE_SUPPORTED_VERSION})
-              </i18n.Translate>
-            </WarningBox>}
-          <EnabledBySettings name="advanceMode">
-            <Part
-              title={i18n.str`Exchange compatibility`}
-              text={<span>{coreVersion.exchange}</span>}
-            />
-            <Part
-              title={i18n.str`Merchant compatibility`}
-              text={<span>{coreVersion.merchant}</span>}
-            />
-            <Part
-              title={i18n.str`Bank compatibility`}
-              text={<span>{coreVersion.bank}</span>}
-            />
-            <Part
-              title={i18n.str`Wallet Core compatibility`}
-              text={<span>{coreVersion.version}</span>}
-            />
-          </EnabledBySettings>
-        </Fragment>
-        )}
-        <SubTitle>
-          <i18n.Translate>Advance mode</i18n.Translate>
+          <i18n.Translate>Navigator</i18n.Translate>
         </SubTitle>
         <Checkbox
-          label={i18n.str`Enable advance mode`}
+          label={i18n.str`Automatically open wallet`}
+          name="autoOpen"
+          description={
+            <i18n.Translate>
+              Open the wallet when a payment action is found.
+            </i18n.Translate>
+          }
+          enabled={autoOpenToggle.value!}
+          onToggle={autoOpenToggle.button.onClick!}
+        />
+
+        <SubTitle>
+          <i18n.Translate>Version Info</i18n.Translate>
+        </SubTitle>
+        <Part
+          title={i18n.str`Web Extension`}
+          text={
+            <span>
+              {webexVersion.version}{" "}
+              <EnabledBySettings name="advancedMode">
+                {webexVersion.hash}
+              </EnabledBySettings>
+            </span>
+          }
+        />
+        {coreVersion && (
+          <Fragment>
+            {LibtoolVersion.compare(
+              coreVersion.version,
+              WALLET_CORE_SUPPORTED_VERSION,
+            )?.compatible ? undefined : (
+              <WarningBox>
+                <i18n.Translate>
+                  The version of wallet core is not supported. (supported
+                  version: {WALLET_CORE_SUPPORTED_VERSION})
+                </i18n.Translate>
+              </WarningBox>
+            )}
+            <EnabledBySettings name="advancedMode">
+              <Part
+                title={i18n.str`Exchange compatibility`}
+                text={<span>{coreVersion.exchange}</span>}
+              />
+              <Part
+                title={i18n.str`Merchant compatibility`}
+                text={<span>{coreVersion.merchant}</span>}
+              />
+              <Part
+                title={i18n.str`Bank compatibility`}
+                text={<span>{coreVersion.bank}</span>}
+              />
+              <Part
+                title={i18n.str`Wallet Core compatibility`}
+                text={<span>{coreVersion.version}</span>}
+              />
+            </EnabledBySettings>
+          </Fragment>
+        )}
+        <SubTitle>
+          <i18n.Translate>Settings</i18n.Translate>
+        </SubTitle>
+        <Checkbox
+          label={i18n.str`Enable developer mode`}
           name="devMode"
           description={i18n.str`Show more information and options in the UI`}
           enabled={advanceToggle.value!}
           onToggle={advanceToggle.button.onClick!}
         />
-        <EnabledBySettings name="advanceMode">
+        <EnabledBySettings name="advancedMode">
           <AdvanceSettings />
         </EnabledBySettings>
         <EnabledBySettings name="langSelector">
@@ -272,47 +210,6 @@ export function SettingsView({
             />
           </Input>
         </EnabledBySettings>
-        <SubTitle>
-          <i18n.Translate>Navigator</i18n.Translate>
-        </SubTitle>
-        <Checkbox
-          label={i18n.str`Inject Taler support in all pages`}
-          name="inject"
-          description={
-            <i18n.Translate>
-              Disabling this option will make some web application not able to
-              trigger the wallet when clicking links but you will be able to
-              open the wallet using the keyboard shortcut
-            </i18n.Translate>
-          }
-          enabled={injectTalerToggle.value!}
-          onToggle={injectTalerToggle.button.onClick!}
-        />
-        <Checkbox
-          label={i18n.str`Automatically open wallet`}
-          name="autoOpen"
-          description={
-            <i18n.Translate>
-              Open the wallet when a payment action is found.
-            </i18n.Translate>
-          }
-          enabled={autoOpenToggle.value!}
-          onToggle={autoOpenToggle.button.onClick!}
-        />
-        <SubTitle>
-          <i18n.Translate>Version</i18n.Translate>
-        </SubTitle>
-        <Part
-          title={i18n.str`Web Extension`}
-          text={
-            <span>
-              {webexVersion.version}{" "}
-              <EnabledBySettings name="advanceMode">
-                {webexVersion.hash}
-              </EnabledBySettings>
-            </span>
-          }
-        />
       </section>
     </Fragment>
   );
@@ -324,6 +221,7 @@ type Options = {
 };
 function AdvanceSettings(): VNode {
   const [settings, updateSettings] = useSettings();
+  const api = useBackendContext();
   const { i18n } = useTranslationContext();
   const o: Options = {
     backup: {
@@ -334,6 +232,10 @@ function AdvanceSettings(): VNode {
       label: i18n.str`Show suspend/resume transaction`,
       description: i18n.str`Prevent transaction from doing network request.`,
     },
+    showRefeshTransactions: {
+      label: i18n.str`Show refresh transaction type in the transaction list`,
+      description: i18n.str`Refresh transaction will be hidden by default if the refresh operation doesn't have fee.`,
+    },
     extendedAccountTypes: {
       label: i18n.str`Show more account types on deposit`,
       description: i18n.str`Extends the UI to more payment target types.`,
@@ -350,6 +252,18 @@ function AdvanceSettings(): VNode {
       label: i18n.str`Lang selector`,
       description: i18n.str`Allows to manually change the language of the UI. Otherwise it will be automatically selected by your browser configuration.`,
     },
+    showExchangeManagement: {
+      label: i18n.str`Edit exchange management`,
+      description: i18n.str`Allows to see the list of exchange, remove, add and switch before withdrawal.`,
+    },
+    selectTosFormat: {
+      label: i18n.str`Select terms of service format`,
+      description: i18n.str`Allows to render the terms of service on different format selected by the user.`,
+    },
+    showWalletActivity: {
+      label: i18n.str`Show wallet activity`,
+      description: i18n.str`Show the wallet notification and observability event in the UI.`,
+    },
   };
   return (
     <Fragment>
@@ -364,6 +278,7 @@ function AdvanceSettings(): VNode {
               enabled={settings[settingsName]}
               onToggle={async () => {
                 updateSettings(settingsName, !settings[settingsName]);
+                await api.background.call("reinitWallet", undefined);
               }}
             />
           );
