@@ -1,6 +1,7 @@
 import { AmountJson, Amounts, TranslatedString } from "@gnu-taler/taler-util";
 import { VNode, h } from "preact";
-import { InputLine, UIFormProps } from "./InputLine.js";
+import { UIFormProps } from "./FormProvider.js";
+import { InputLine } from "./InputLine.js";
 import { useField } from "./useField.js";
 
 export function InputAmount<T extends object, K extends keyof T>(
@@ -21,7 +22,8 @@ export function InputAmount<T extends object, K extends keyof T>(
       converter={{
         //@ts-ignore
         fromStringUI: (v): AmountJson => {
-          return Amounts.parseOrThrow(`${currency}:${v}`);
+
+          return Amounts.parse(`${currency}:${v}`) ?? Amounts.zeroOfCurrency(currency);
         },
         //@ts-ignore
         toStringUI: (v: AmountJson) => {
